<?xml version="1.0" encoding="UTF-8"?>

<!DOCTYPE xsl:stylesheet [ <!ENTITY nbsp "&#160;"> ]>

<xsl:stylesheet exclude-result-prefixes="xs" version="2.0" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">


    <xsl:output indent="yes" method="xml"/>

    <xsl:template match="Bericht/@*" mode="sb7">
        <xsl:variable as="xs:string" name="name">
            <xsl:value-of select="translate(substring-after(name(.),'-'),'ÄÖÜäöüß-','AOUaousU')"/>
        </xsl:variable>
    
        <xsl:if test="string-length($name) > 0">
            <xsl:attribute name="{$name}">
                <xsl:value-of select="."/>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template match="Bericht/@*">
        <xsl:variable as="xs:string" name="name">
            <xsl:value-of select="translate(substring-after(substring-after(name(.),'-'),'-'),'ÄÖÜäöüß-','AOUaousU')"/>
        </xsl:variable>

        <xsl:if test="string-length($name) > 0">
            <xsl:attribute name="{$name}">
                <xsl:value-of select="."/>
            </xsl:attribute>
        </xsl:if>
    </xsl:template>

    <xsl:template match="Spieler">
        <Spieler>
            <xsl:copy-of select="@id | @name | @vorname |  @geburtsjahr |  @trikotnummer | @strafen | @tore | @passNr | @dsr | @ausweiskontrolliert"/>
        </Spieler>
    </xsl:template>

    <xsl:template match="Person">
        <Offizieller>
            <xsl:copy-of select="@id | @name | @vorname |  @geburtsjahr |  @trikotnummer | @strafen | @tore | @passNr | @dsr | @ausweiskontrolliert"/>
        </Offizieller>
    </xsl:template>

    <xsl:template name="ProtokollEintrag">
        <Eintrag>
            <xsl:copy-of select="@*[name(.) != 'spielnummer']"/>
        </Eintrag>
    </xsl:template>
  
    <xsl:template name="LogEintrag">
        <Eintrag>
            <xsl:copy-of select="@*[name(.) != 'spielnummer']"/>
        </Eintrag>
    </xsl:template>

    <xsl:template match="/">
        <Spiel>
            <xsl:copy-of select="//Spiel/@*"/>
            <TeamHeim>
                <xsl:attribute name="Name">
                    <xsl:value-of select="//Spiel/@NameHeim"/>
                </xsl:attribute>
                <xsl:attribute name="Farbe">
                    <xsl:value-of select="//Bericht/@farbeSpieler_heim"/>
                </xsl:attribute>
                <xsl:attribute name="FarbeTorhueter">
                    <xsl:value-of select="//Bericht/@farbeTorhüter_heim"/>
                </xsl:attribute>
                <xsl:apply-templates select="//Spieler[@aktiv='1' and @gast='0']"/>
                <xsl:apply-templates select="//Person[@aktiv='1' and @gast='0']"/>
            </TeamHeim>
            <TeamGast>
                <xsl:attribute name="Name">
                    <xsl:value-of select="//Spiel/@NameGast"/>
                </xsl:attribute>
                <xsl:attribute name="Farbe">
                    <xsl:value-of select="//Bericht/@farbeSpieler_gast"/>
                </xsl:attribute>
                <xsl:attribute name="FarbeTorhueter">
                    <xsl:value-of select="//Bericht/@farbeTorhüter_gast"/>
                </xsl:attribute>
                <xsl:apply-templates select="//Spieler[@aktiv='1' and @gast='1']"/>
                <xsl:apply-templates select="//Person[@aktiv='1' and @gast='1']"/>
            </TeamGast>
            <Spielleitung>
                <Person typ="sr-a">
                    <xsl:attribute name="pin">
                        <xsl:value-of select="//Bericht/@PIN-SR-A"/>
                    </xsl:attribute>
                  <xsl:apply-templates select="//Bericht/@*[starts-with(name( .),'sb4-A')]"/>
                </Person>
                <Person typ="sr-b">
                    <xsl:attribute name="pin">
                      <xsl:value-of select="//Bericht/@PIN-SR-B"/>
                    </xsl:attribute>
                  <xsl:apply-templates select="//Bericht/@*[starts-with(name( .),'sb4-B')]"/>
                </Person>
                <Person typ="sr-beobachter">
                  <xsl:apply-templates select="//Bericht/@*[starts-with(name( .),'sb6-SrB')]"/>
                </Person>
                <Person typ="zeitnehmer">
                  <xsl:apply-templates select="//Bericht/@*[starts-with(name( .),'sb5-ZN')]"/>
                </Person>
                <Person typ="sekretaer">
                  <xsl:apply-templates select="//Bericht/@*[starts-with(name( .),'sb5-SK')]"/>
                </Person>
                <Person typ="aufsicht">
                    <xsl:attribute name="pin">
                      <xsl:value-of select="//Bericht/@PIN-aufsicht"/>
                    </xsl:attribute>
                  <xsl:apply-templates select="//Bericht/@*[starts-with(name( .),'sb6-SpA')]"/>
                </Person>
            </Spielleitung>
            <Bericht>
                <Bericht1>
                    <xsl:apply-templates select="//Bericht/@*[starts-with(name(.),'sb7')]" mode="sb7"/>
                </Bericht1>
                <Bericht2>
                    <xsl:value-of select="//sb8-Bericht2"/>
                </Bericht2>
                <EinspruchHeim>
                    <xsl:value-of select="//sb9-EinspruchHeimBericht"/>
                </EinspruchHeim>
                <EinspruchGast>
                    <xsl:value-of select="//sb9-EinspruchGastBericht"/>
                </EinspruchGast>
            </Bericht>
            <Protokoll>
                <xsl:for-each select="//ProtokollEintrag">
                    <xsl:sort select="@zeit"/>
                    <xsl:call-template name="ProtokollEintrag"/>
                </xsl:for-each>
            </Protokoll>
            <Log>
                <xsl:for-each select="//LogEintrag">
                    <xsl:sort select="@zeit"/>
                    <xsl:call-template name="LogEintrag"/>
                </xsl:for-each>
            </Log>     
        </Spiel>
    </xsl:template>
</xsl:stylesheet>
