/* global Aes */

if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}
if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.nextId = -1;
me_apps.logging = new Array();

me_apps.SCHIRI_1_FLAG = 1;      //  0b01
me_apps.SCHIRI_2_FLAG = 2;      //  0b10
me_apps.SCHIRI_ALL_FLAG = 3;    //  0b11

me_apps.DETAILS_REFEREE_1=1;            // matches SCHIRI*
me_apps.DETAILS_REFEREE_2=2;
me_apps.DETAILS_REFEREES=3;             // REF_1 | REF_2
me_apps.DETAILS_TEAM_HOME=4;
me_apps.DETAILS_TEAM_GUEST=8;
me_apps.DETAILS_TICKER_DATA=16;
me_apps.DETAILS_LOG=32;
me_apps.DETAILS_METADATA=64;            // settings; only when uploading
me_apps.DETAILS_HEADERDATA=128;
me_apps.DETAILS_REPORT=256;
me_apps.DETAILS_PROTOCOL=512;

me_apps.DETAILS_DEFAULT=128 + 15;             // data that's downloaded by default

/*
 * ***************************************************************************
 * Initialize the DB and handle DB-global stuff
 * ***************************************************************************
 */

me_apps.obj.db =
function () {
    me_apps.log.info("Initializing me_apps.obj.db");
    var object;
    if (typeof(localStorage) === "undefined") {
        me_apps.log.error("Browser does not support LocalStorage");
    } else {
        var serialized;
        if ((serialized=localStorage.getItem("Appinfo"))===null) {
            object={
                uuid: ''
            };
            me_apps.log.info("Database init creating new item");
        } else {
            object=JSON.parse(serialized);
            me_apps.log.info("Database init reading existing item");
        }
    }
    me_apps.permStorage=object;
    if (typeof(me_apps.permStorage.games)==="undefined") {
        me_apps.permStorage.games=[];
        me_apps.permStorage.lastRecentlyUsedGameId=-1;
    }
    if (typeof(me_apps.permStorage.log)==="undefined") {
        me_apps.permStorage.log=[];
    }
    if (typeof(me_apps.permStorage.finalizedGames)==="undefined") {
        me_apps.permStorage.finalizedGames=[];
    }
    if (typeof(me_apps.permStorage.savedFirstFinalizes) === "undefined") {
        me_apps.permStorage.savedFirstFinalizes={};
    }
};

me_apps.obj.db.prototype.initDB =
function (clear) {
    me_apps.log.logCall();;
    
    if (clear) {
        var olduuid="";
        if (typeof(me_apps.permStorage !== "undefined"))
            olduuid=me_apps.permStorage.uuid;
        me_apps.permStorage={};
        me_apps.permStorage.uuid=olduuid;
        me_apps.permStorage.games=[];
        me_apps.permStorage.lastRecentlyUsedGameId=-1;
        me_apps.permStorage.log=[];
        me_apps.permStorage.finalizedGames = [];
        me_apps.permStorage.savedFirstFinalizes=[];
    }
    this.setUUID();
};

me_apps.obj.db.prototype.setUUID =
function () {
    me_apps.log.logCall();;
    
    if (me_apps.permStorage.uuid === "") {
        me_apps.permStorage.uuid = me_apps.util.createUUID();
        me_apps.log.info("Created new uuid - using "+me_apps.permStorage.uuid);
        me_apps.db.persist();
    } else {
        me_apps.log.info("Re-using previous uuid - "+me_apps.permStorage.uuid);
    }
    me_apps.db.uuid=me_apps.permStorage.uuid;
};

me_apps.obj.db.prototype.removeNPAttributes =
function(key, value) {
    if (key.substring(0, 4) === "_np_")
        return undefined;
    else
        return value;
};

/**
 * 
 * @function persist
 * Copies the memory-internal database to localStorage.
 * Some properties, like the needsUpload / uploadScheduled property
 * should not be persisted, because they represent internal state
 * of the running engine. These properties should be named
 * _np_*, and will be omitted by this function.
 */
me_apps.obj.db.prototype.persist =
function() {
    me_apps.log.logCall();
    localStorage.setItem("Appinfo", JSON.stringify(me_apps.permStorage, this.removeNPAttributes));
};

/*
 * ***************************************************************************
 * Deal with the game list and what the currently active game is
 * ***************************************************************************
 */

/**
 * @function hasCurrentGame
 * Check if a game is current (which may or may not have loaded details)
 * @return {Boolean} true if a game is current, false else
 */
me_apps.obj.db.prototype.hasCurrentGame =
function() {
    me_apps.log.logCall();
    return (me_apps.permStorage.lastRecentlyUsedGameId>=0 && me_apps.permStorage.lastRecentlyUsedGameId<me_apps.permStorage.games.length);
};

/*
 * @function getCurrentGame
 * Get the current game object, or undefined if there is none.
 * @return {undefined|object.games}
 */
me_apps.obj.db.prototype.getCurrentGame =
function() {
    me_apps.log.logCall();
    if (!(this.hasCurrentGame())) {
        me_apps.log.warn("getCurrentGame has no game, returning undefined");
        return undefined;
    }
    return me_apps.permStorage.games[me_apps.permStorage.lastRecentlyUsedGameId];
};

me_apps.obj.db.prototype.getGameById = function(id) {
    var index = this.findGameIndexByID(id);
    return me_apps.permStorage.games[index];
};

me_apps.obj.db.prototype.deleteGame =
function(matchId) {

    delete me_apps.permStorage.savedFirstFinalizes["Game"+matchId];

    var index=me_apps.db.findGameIndexByID(matchId);
    if (index === -1) {
        me_apps.log.warn("was asked to delete info from "+matchId+" which is a match I don't know about, index is "+index);
        return;
    }
    me_apps.log.info("Deleting index "+index);
    var games=me_apps.permStorage.games;
    var length=games.length;
    for (var i=index+1; i<length; i++)
        games[i-1]=games[i];
    delete games[length-1];
    games.length--; // yes this is how you remove an element - http://ecma-international.org/ecma-262/5.1/#sec-15.4
    if (index===me_apps.permStorage.lastRecentlyUsedGameId) {
        me_apps.permStorage.lastRecentlyUsedGameId=-1;
        me_apps.spiel.setDefaultValues();
    } else if (index < me_apps.permStorage.lastRecentlyUsedGameId) {
        me_apps.permStorage.lastRecentlyUsedGameId--;
    }
    me_apps.db.persist();
};

/**
 * Delete all information about a match that's not in the match list as well.
 * 
 * This function is/will probably be unused. I thought I'd need this when pressing
 * the GUI trashcan icon, but we really want to delete all about the game, not
 * just the extended info, so a new load operation will be able to reload it.
 * 
 * @param {Integer} matchId - the unique database Id of this match
 */
me_apps.obj.db.prototype.deleteExtendedGameInfo =
function(matchId) {
    me_apps.log.logCall();
    var index=me_apps.db.findGameIndexByID(matchId);
    if (index === -1) {
        me_apps.log.warn("was asked to delete info from "+matchId+" which is a match I don't know about");
        return;
    }
    var toChange=me_apps.permStorage.games[index];
    delete toChange.referees;
    delete toChange.team_guest;
    delete toChange.team_home;
    delete toChange.protocol;
    delete toChange.log;
    me_apps.db.persist();
};

/**
 * This sets the match that's counted as "active" - the one that's shown
 * in the other parts of the GUI. This function may be used to create a new,
 * empty game or use one of the games that's in the current game database.
 * If game details are missing, they get loaded from the server.
 *
 * @function setCurrentGame
 * @param {integer} no - the number of the game as displayed in the GUI.
 * @param {integer} matchId - the id of the match as present in the database.
 * @param {string} date - The date of the match. We may need this to query
 *          the server for the match.
 * @param {function} callback - this function will be called after the load
 *          succeeds, with a status code, the game object, a message, and a flag
 *          if the operation has finished. In case the
 *          game can't be found in the database, this function will still be
 *          called, with a null parameter.
 */

me_apps.obj.db.prototype.setCurrentGame =
function (no, matchId, date, callback) {
    me_apps.log.logCall();

    // case a - creating a new game. I think this is unused now.

    if (matchId === me_apps.DEFAULT_SPIELID) {
        index=me_apps.db.createDefaultGame(me_apps.DEFAULT_SPIELNUMMER, me_apps.DEFAULT_SPIELID, date);
        me_apps.permStorage.lastRecentlyUsedGameId=index;
        me_apps.db.persist();
        callback(0, me_apps.permStorage.games[index], "Default-Spiel erzeugt", true);
        return;
    }   

    // case b - we think we're loading an old game, but that game isn't in our
    // database
    var index=me_apps.obj.db.prototype.findGameIndexByID(matchId);
    if (index === -1) {
        me_apps.log.warn("Trying to set game "+no+"/"+matchId+" current, but we don't have it in our database");
        callback(1, null, "Spiel nicht gefunden?", true);
        return;
    } 
    me_apps.permStorage.lastRecentlyUsedGameId=index;
    me_apps.db.persist();
    
    // case c - we're loading an old game and that old game is in our database,
    // including detail information. In this case, re-schedule ticker uploads
    // in case there are any outstanding ones; if there aren't, scheduleTicker
    // won't do anything.
    var game=me_apps.permStorage.games[index];
    if (me_apps.util.gameHasDetailsLoaded(game)) {
        callback(0, game, "Vorhandenes Spiel aktiviert", true);
        me_apps.db.scheduleUploadTicker(game);
        return;
    }

    // case d - we're loading an old game which we have in our database, but
    // we're missing details and need to load them
    callback(0, null, "Details für Spiel "+game.no+" werden geladen", false);
    me_apps.db.downloadGameDetails(matchId, me_apps.DETAILS_DEFAULT, callback);
};

/**
 * Loads the game that was used most recently, or initializes a new one.
 * After this call, me_apps.spiel should be initialized properly.
 * 
 * @returns {undefined}
 */
me_apps.obj.db.prototype.loadRecentlyUsedGame =
function() {
    me_apps.log.logCall();;
    me_apps.spiel = new me_apps.obj.spiel(me_apps.DEFAULT_SPIELID, me_apps.DEFAULT_SPIELNUMMER);

    var games=me_apps.permStorage.games;
    if (games.length===0) {
        me_apps.spiel.setDefaultValues();
    } else {
        var index=me_apps.permStorage.lastRecentlyUsedGameId;
        if (typeof index === "undefined" || index<0 || index>=games.length || games[index]===0)
            me_apps.spiel.setDefaultValues();
        else
            me_apps.spiel.copyValuesFromPermStore(games[index]);
    }
};

/*
 * ***************************************************************************
 * Change settings withing the current game
 * ***************************************************************************
 */

/**
 * Change a game setting. Also schedules an upload to the server. This upload
 * is delayed by 10 seconds so we can collect several settings changes in one.
 * @param {type} setting the name of the setting to change (game structure component)
 * @param {type} value the new value of the setting
 * @returns {undefined}
 */

me_apps.obj.db.prototype.setGameSetting =
function (setting, value) {
    me_apps.log.logCall();
    var game=me_apps.obj.db.prototype.getCurrentGame();
    if (game === undefined) {
        me_apps.warn("setting changed while no game loaded - ignoring");
        return;
    }
    if (game[setting] !== value) {
        game[setting] = value;
        this.saveGame(game);
    }
};

me_apps.obj.db.prototype.saveGame =
function(game) {
    game._np_needUpload |= me_apps.DETAILS_METADATA;
    game.change_count++;
    me_apps.db.scheduleUploadGameData(game);
    me_apps.db.persist();
};

/**
 * Copy the current game time into the game object. We don't persist this
 * every time the time updates ( every second ) for performance reasons,
 * but the object gets persisted every time something happens anyway,
 * and if nothing happens, we persist the time every 15 seconds.
 * 
 * @param {integer} time
 * @param {boolean} persist
 * @returns {undefined}
 */
me_apps.obj.db.prototype.setGameTime =
function (time, persist) {
    me_apps.log.logCall();
    var game=me_apps.db.getCurrentGame();
    if (game === undefined)
        return;
    if (time>game.game_time+15 || persist===true) {
        game.game_time=time;
        me_apps.db.persist();
    } else {
        game.game_time=time;
    }
};

me_apps.obj.db.prototype.setTeamTimeouts =
function (home, guest) {
    me_apps.log.logCall();
    var game=me_apps.db.getCurrentGame();
    if (game === undefined)
        return;
    game.timeout_home=home;
    game.timeout_guest=guest;
    me_apps.db.persist();
};

/**
 * @function updateSpieler -puts back a player (in app format) to the database
 * Mir gefällt nicht wirklich wie hier beide Teams durchsucht werden, aber ich
 * kanns nicht (ohne großen Aufwand) ändern.
 * @param {me_apps.obj.spieler} spieler
 * @param {string} teamCode the team to use, H/heim/home or G/gast/guest, only first letter checked
 * @param {boolean} alsoSendGame send game info as well, increase game change count
 */
me_apps.obj.db.prototype.updateSpieler =
function (spieler, teamCode, alsoSendGame) {
    me_apps.log.logCall();
    var game=this.getCurrentGame();
    var update=0;
    var teamCharacter=teamCode.charAt(0).toUpperCase();
    
    if (teamCharacter === 'H' && this.tryUpdateSpielerInTeam(game.team_home, spieler))
        update=me_apps.DETAILS_TEAM_HOME;
    else if (teamCharacter === 'G' && this.tryUpdateSpielerInTeam(game.team_guest, spieler))
        update=me_apps.DETAILS_TEAM_GUEST;
    else
        me_apps.log.warn("trying to updateSpieler id "+spieler.id+" in team "+teamCharacter+" (code was "+teamCode+") that wasn't found");
    
    if (alsoSendGame && update!==0) {
        game.change_count++;
        game._np_needUpload |= update;
        me_apps.db.scheduleUploadGameData(game);
        me_apps.db.persist();
    }
};
        
/**
 * @function tryUpdateSpielerInTeam - puts back a player (in app format) 
 * to the database assuming (s)he is a member of the team.
 * @param {me_apps.obj.spieler} spieler the player (in app format) to save
 * @param team the team (in db format) to update
 * @return true if the player was found, false else
 */
me_apps.obj.db.prototype.tryUpdateSpielerInTeam =
function(team, spieler) {
    for (var i=0; i<team.team_members.length; i++) {
        if (team.team_members[i].person_id === spieler.id) {
            spieler.copyValuesToPermStore(team.team_members[i]);
            return true;
        }
    }
    return false;
};

/**
 * function updateTeam
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {boolean} alsoSendGame send game info as well, increase game change count
 * @returns {undefined}
 */
me_apps.obj.db.prototype.updateTeam =
function(mannschaft, alsoSendGame) {
    me_apps.log.logCall();
    var game=this.getCurrentGame();
    var update=0;

    if (mannschaft.selector === "heim") {
        mannschaft.copyValuesToPermStore(game.team_home);
        update=me_apps.DETAILS_TEAM_HOME;
    } else if (mannschaft.selector === "gast") {
        mannschaft.copyValuesToPermStore(game.team_guest);
        update=me_apps.DETAILS_TEAM_GUEST;
    } else {
        me_apps.log.warn("trying to updateTeam selector "+mannschaft.selector);
    }
    if (alsoSendGame && update!==0) {
        game.change_count++;
        game._np_needUpload |= update;
        me_apps.db.scheduleUploadGameData(game);
    }
    me_apps.db.persist();
};

me_apps.obj.db.prototype.clearTeamMembers = 
function(mannschaft) {
    var game=this.getCurrentGame();
    if (mannschaft.selector === "heim") {
        game.team_home.team_members=[];
    } else if (mannschaft.selector === "gast") {
        game.team_guest.team_members=[];
    }
    me_apps.db.persist();
};

/**
 * Add a new team member to the persistent database, assuming this new team
 * member has not been sent by the server and isn't in the DB yet.
 * @param {type} mannschaft
 * @param {type} spieler
 * @returns {undefined}
 */
me_apps.obj.db.prototype.addManualTeamMember =
function(mannschaft, spieler) {
    me_apps.log.logCall();
    var game=this.getCurrentGame();
    var newMember = {};
    spieler.copyValuesToPermStore(newMember);
    if (mannschaft.selector === "heim") {
        game.team_home.team_members.push(newMember);
        update=me_apps.DETAILS_TEAM_HOME;
    } else if (mannschaft.selector === "gast") {
        game.team_guest.team_members.push(newMember);
        update=me_apps.DETAILS_TEAM_GUEST;
    } else {
        me_apps.log.warn("trying to updateTeam selector "+mannschaft.selector);
    }
    game.change_count++;
    game._np_needUpload |= update;
    me_apps.db.scheduleUploadGameData(game);
    me_apps.db.persist();
};

/*
 * ***************************************************************************
 * Handle the event protocol
 * ***************************************************************************
 */

// should be named addProtokollEntry. Really belongs into a not yet existing ProtocolList class.
me_apps.obj.db.prototype.addProtokoll =
function (protokoll) {
    me_apps.log.logCall();
    
    var game=me_apps.obj.db.prototype.getCurrentGame();
    var protocolList=game.protocol;
    if (typeof(protocolList)==="undefined") {
        protocolList=game.protocol=[];
    }
    var newEntry = { change_count : game.change_count };
    protokoll.copyValuesToPermStore(newEntry);
    if (newEntry.sequenceno !== protocolList.length) {
        me_apps.log.warn("protocol sequenceno "+newEntry.sequenceno+" does not match length "+protocolList.length);
    }
    protocolList.push(newEntry);
    game.change_count++;
    game._np_needUpload |= me_apps.DETAILS_PROTOCOL;
    me_apps.db.scheduleUploadGameData(game);
    me_apps.db.scheduleUploadTicker(game);
    me_apps.db.persist();
};

// Really belongs into a not yet existing ProtocolList class.
me_apps.obj.db.prototype.updateProtokollEntry =
function (aktion, zeit, newTrikotNr, newTeam, newActionCode, newaktion, score, createID) {
    me_apps.log.logCall();
    var game=me_apps.obj.db.prototype.getCurrentGame();
    var dbEntry=game.protocol[createID];
    if (dbEntry.sequenceno !== createID) {
        me_apps.log.warn("protocol sequenceno "+dbEntry.sequenceno+" does not match createID "+createID);
    }
    var appEntry = new me_apps.obj.protokoll();
    appEntry.copyValuesFromPermStore(dbEntry);
    appEntry.update(aktion, zeit, newTrikotNr, newTeam, newActionCode, newaktion, score);
    appEntry.copyValuesToPermStore(dbEntry);
    game._np_needUpload |= me_apps.DETAILS_PROTOCOL;
    me_apps.db.scheduleUploadGameData(game);
    me_apps.db.scheduleUploadTicker(game);
    me_apps.db.persist();
};

// Really belongs into a not yet existing ProtocolList class.
me_apps.obj.db.prototype.deleteProtokollEntry =
function (aktion, zeit, score, createID) {
    me_apps.log.logCall();
    var game=me_apps.obj.db.prototype.getCurrentGame();
    var dbEntry=game.protocol[createID];
    if (dbEntry.sequenceno !== createID) {
        me_apps.log.warn("protocol sequenceno "+dbEntry.sequenceno+" does not match createID "+createID);
    }
    var appEntry = new me_apps.obj.protokoll();
    appEntry.copyValuesFromPermStore(dbEntry);
    appEntry.delete(aktion, zeit, score);
    appEntry.copyValuesToPermStore(dbEntry);
    game._np_needUpload |= me_apps.DETAILS_PROTOCOL;
    me_apps.db.scheduleUploadGameData(game);
    me_apps.db.scheduleUploadTicker(game);
    me_apps.db.persist();
};

me_apps.obj.db.prototype.getProtocolList =
function() {
    if (!me_apps.db.hasCurrentGame())
        return [];
    var game=me_apps.obj.db.prototype.getCurrentGame();
    var protocolList=game.protocol;
    if (typeof(protocolList)==="undefined") {
        protocolList=game.protocol=[];
    }
    return protocolList;
};

// should really be ProtocolList.toGUI()   @TODO
me_apps.obj.db.prototype.loadProtokoll =
function () {
    me_apps.log.logCall();;
    me_apps.log.warn("WARNING - DEPRECATED FUNCTION loadProtokoll, please use me_apps.obj.protokollDialog.prototype.toGUI instead");
    return me_apps.obj.protokollDialog.prototype.toGUI(me_apps.obj.db.prototype.getProtocolList());
};

/**
 * Load the list of matches for a club, or a list of clubs, from the server
 * 
 * @param {String} vereine - a comma separated list of clubs for which to load
 * the matches. This string is passed to the server unchanged, so its syntax
 * should be checked with server documentation.
 * @param {function} callback a callback function to call after the download
 * is finished
 * 
 * @return {undefined} - no return value
 */

me_apps.obj.db.prototype.downloadGameList =
function (vereine, callback) {
    me_apps.log.logCall();
    
    if (typeof(me_apps.permStorage.games)==="undefined") {
        me_apps.permStorage.games=[];
    }
    var url=me_apps.URL_PROTOKOLL_SERVER + "cmd=gameList&clublist="+vereine+ "&version=" + me_apps.VERSION + "." + me_apps.BUILD + "&v_interface=" + me_apps.V_INTERFACE + "&uuid=" + me_apps.db.uuid;
    me_apps.log.info("loading match list for "+vereine+" from "+url);
    callback(0, "Spieleliste für Verein "+ vereine + " wird geladen ...", false);
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'L');
    $.ajax({
        url: url,
        type: 'POST',
        dataType: "json",
        async: true
    }).done(function(data, textStatus, jqXHR) {
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
        if (typeof(data.answer) === "undefined"
        ||  typeof(data.answer) === "undefined"
        ||  !Array.isArray(data.answer.gameList)) {
            callback(1, "Serverantwort ok, aber keine Spiele gefunden", true);
        } else {
            callback(0, "Verarbeite "+data.answer.num_gameList+" vom Server erhaltene Spiele", false);
            var newEntries=me_apps.obj.db.prototype.parseDownloadedGameList(data);
            callback(0, ""+data.answer.num_gameList+" Spiele vom Server geladen, "
                        +newEntries[0]+" davon neu, "+ newEntries[1]+" gelöscht", true);
        }
    }).fail(function(jqXHR, textStatus, errorThrown) {
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
        me_apps.log.warn("failed to download matches: "+JSON.stringify(jqXHR));
        callback(1, "Fehler bei Laden der Spieleliste.\nEs konnten keine neuen Spiele vom Server geladen werden.\nBitte prüfen Sie die Netzverbindung!", true);
    });
};

/**
 * Parse a match list from the server, and merge it with the matches already
 * known to the client. Games known to the client will not be overwritten,
 * because we might have local changes that aren't know to the server yet.
 * 
 * @param {type} downloadedBlob
 * @return {Number}
 */
me_apps.obj.db.prototype.parseDownloadedGameList=
function(downloadedBlob) {
    me_apps.log.logCall();
    me_apps.log.debug("downloaded matches : "+downloadedBlob);
    me_apps.log.debug(downloadedBlob);
    me_apps.log.info("merging "+downloadedBlob.answer.num_gameList+" downloaded matches with "+me_apps.permStorage.games.length+" present ones");
    var gameList=downloadedBlob.answer.gameList;
    var knownBefore=me_apps.permStorage.games.length;
    for (var downloadIndex=0; downloadIndex<gameList.length; downloadIndex++) {
        var downloadid=gameList[downloadIndex].id;
        me_apps.log.debug("inserting match "+downloadid);
        var storedIndex=me_apps.obj.db.prototype.findGameIndexByID(downloadid);
        if (storedIndex===-1) {
            me_apps.log.debug("    Unknown match "+downloadid+", adding");
            me_apps.permStorage.games[me_apps.permStorage.games.length] = gameList[downloadIndex];
        } else {
            if (gameList[downloadIndex].confirmedUpload) {
                me_apps.permStorage.games[storedIndex].confirmedUpload=true;
                me_apps.log.info("Mark game "+downloadid+" to delete because it was confirmed");
            } else if (!me_apps.util.gameHasDetailsLoaded(me_apps.permStorage.games[storedIndex])) {
                me_apps.log.info("Game "+downloadid+" has no details, overwriting");
                me_apps.permStorage.games[storedIndex] = gameList[downloadIndex];
            } else {
                me_apps.log.debug("    Known match index "+storedIndex +", not touching anything");
            }
        }
    }
    var created = me_apps.permStorage.games.length-knownBefore;
    var deleted = me_apps.db.cleanOldGames();
    me_apps.db.persist();
    return [ created, deleted ];
};

/**
 * 
 * @returns {integer} the number of games that got deleted
 */

me_apps.obj.db.prototype.cleanOldGames =
function() {
    var twoDaysAgo=new Date(Date.now()-86400*1000*2);
    var mm=twoDaysAgo.getMonth()+1;
    var dd=twoDaysAgo.getDate();
    var twoDaysAgoStamp=twoDaysAgo.getFullYear()+"-"+(mm>9 ? '' : '0')+mm+'-'+(dd>9 ? '' : '0')+dd;
    var games=me_apps.permStorage.games;
    var target=0;
    for (var source=0; source<games.length; source++) {
        if (!games[source].confirmedUpload 
        && (games[source].datetime > twoDaysAgoStamp || games[source].game_time>0)) {
            // keep the game if it isn't confirmed, and it's either new or started
            if (source!==target) {
                games[target]=games[source];
                if (me_apps.permStorage.lastRecentlyUsedGameId===source)
                        me_apps.permStorage.lastRecentlyUsedGameId=target;
            }
            target++;
        } else if (me_apps.permStorage.lastRecentlyUsedGameId===source) {
            me_apps.permStorage.lastRecentlyUsedGameId=-1;
        }
    }
    var deleted=games.length-target;
    games.length=target;
    
    for (var key in me_apps.permStorage.savedFirstFinalizes) {
        if (!key.startsWith("Game") || me_apps.db.findGameIndexByID(key.substring(4)) === -1) {
            delete me_apps.permStorage.savedFirstFinalizes[key];
        }
    }
    return deleted;
};

/**
 * Search for a match by its database ID.
 * 
 * @param {Integer} id - the id of the match to search for
 * @return {Integer} - the index of the match, or -1 if the match isn't in the database
 */
me_apps.obj.db.prototype.findGameIndexByID =
function (id) {
    var list=me_apps.permStorage.games;
    for (var i=0; i<list.length && list[i]!==null; i++)
        if (list[i].id === id)
            return i;
    return -1;
};

/**
 * Iterate through all known games, and calls
 * the callback function for each of them.
 * @function enumGameList
 * @param {function} callback
 * The callback function to call for each match. This function gets two
 * parameters. The first one is an associative array that has entries like
 * "id", "StaffelKurzName", "Harzregelung", see documentation for @TODO@. The
 * second parameter is true if the game is the one that's marked as currently
 * loaded as active game.
 * @param {function} order
 * A function to compare two games to order the enum list
 */
me_apps.obj.db.prototype.enumGameList =
function(callback, order) {
    var list=me_apps.permStorage.games;
    me_apps.log.debug("list is "+list);
    if (typeof(list)==="undefined" || typeof(list.length)==="undefined")
        return;     // no games loaded yet

    // remove bad elements
    for (var i=0; i<list.length; i++) {
        if (list[i].datetime===null) {
            list.splice(i, 1);
            i--;
        }
    }

    if (typeof(order) === "function") {
        var reorder=[];
        for (var i=0; i<list.length && list[i]!==null; i++) {
            reorder[i]=i;
        }
        reorder.sort(function(a, b) {
            return order(list[a], list[b]);
        });
        for (var i=0; i<list.length && list[i]!==null; i++)
            callback(list[reorder[i]], reorder[i]===me_apps.permStorage.lastRecentlyUsedGameId);
    } else {
        for (var i=0; i<list.length && list[i]!==null; i++)
            callback(list[i], i===me_apps.permStorage.lastRecentlyUsedGameId);
    }
};

/**
 * Decrypts all members of a team. Returns the team after
 * decryption, or undefined if there is no current game.
 * 
 * @param {type} team game.team_home or game.team_guest
 * @param {type} password the password to use for decryption
 * @returns {undefined|(dbEntry) Team}
 */
me_apps.obj.db.prototype.decryptTeam =
function(team, password) {
    var game=me_apps.obj.db.prototype.getCurrentGame();
    if (game===undefined) {
        me_apps.warn("decrypt attempt while no game loaded - ignoring");
        return undefined;
    }

    var properties=["active", "player_no", "lic_number",
            "prename", "name", "birthday", "club_no", "club_lname", "dsr",
            "person_id"
    ];
    // server doesn't send this so we check for !== false, not == true)
    if (team.encrypted !== false) {
        for (var i=0; i<team.team_members.length; i++) {
            var member=team.team_members[i];
            if (member.encrypted !== false) {
                for (var j=0; j<properties.length; j++) {
                    if (member[properties[j]])
                        member[properties[j]]=Aes.Ctr.decrypt(member[properties[j]], password, 128);
                    // else
                        // This actually happens a lot since we get empty players, so don't report it.
                        // me_apps.log.warn("trying to decrypt unset property "+j+":"+properties[j]);
                }
                member.encrypted=false;
            }
        }
        team.ticker_hash=Aes.Ctr.decrypt(team.ticker_hash, password, 128);
        team.encrypted=false;
    }
    game.secret=game.team_home.ticker_hash+game.team_guest.ticker_hash;
    me_apps.db.persist();
    return team;
};

/**
 * @function downloadGameDetails
 * Get the detailed data for a certain game from the server, and call a callback
 * function afterwards that shows that data in the gui.
 * 
 * @param {integer} matchId - the database id of the game to load
 * @param {bitmap} flags - bitmapped flags from the DETAILS_* set, determines which
 * data parts are/are not copied
 * @param {function} callback - a callback function to show results in the GUI. 
 * This callback function gets two parameters, which are the game structure in
 * our persistent object cache ( or null in case of error ) and an error string
 * (empty string in case of success)
 */

me_apps.obj.db.prototype.downloadGameDetails =
function(matchId, flags, callback) {
    me_apps.log.logCall();
    var index;
    var url=me_apps.db.getUrl('getSboGame')+"&id="+matchId;
    me_apps.log.info("loading game details for "+matchId+" from "+url);
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'D');
    $.ajax({
        url: url,
        type: 'POST',
        dataType: "json",
        async: true
    }).done(function(data, textStatus, jqXHR) {
        me_apps.log.debug(data);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
        if (!data.common || !data.common.datetime) {
            me_apps.log.warn("downloading game details for "+matchId+" did not return date&time");
            me_apps.db.deleteGame(matchId);     // den zuerst, callback zeigt die Liste dann neu an
            callback(1, null, "Kein Datum/Uhrzeit für dieses Spiel gefunden", true);
        } else {
            index=me_apps.obj.db.prototype.parseDownloadedGameDetails(matchId, data, flags);
            callback(0, me_apps.permStorage.games[index], "Spieldaten geladen", true);
        }
    }).fail(function(jqXHR, textStatus, errorThrown) {
        me_apps.log.warn("failed to download match data: "+JSON.stringify(jqXHR));
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
        callback(2, null, "Fehler beim Laden der Spieldetails (Netzverbindung testen?)", true);
    });
};

/**
 * @function parseDownloadedGameDetails
 * Parse data of a certain game after it's downloaded from the server, 
 * and call a callback
 * function afterwards that shows that data in the gui.
 * 
 * @param {integer} id - the database id of the game to load
 * @param {datestring} data - the json object the server sent us
 * @param {bitmap} flags - bitmapped flags from the DETAILS_* set, determines which
 *                  data parts are/are not copied
 * @return {integer} An index into the permanent storage game structure table
 *                  that refers to the downloaded game
 */

me_apps.obj.db.prototype.parseDownloadedGameDetails =
function (id, data, flags) {
    var index=me_apps.obj.db.prototype.findGameIndexByID(id);
    if (index===-1) {
        // This shouldn't happen; the GUI shouldn't allow us tor request
        // details for a match that hasn't its data loaded already.
        me_apps.log.warn("Loading details for game "+id+" that we don't know about yet");
        index=me_apps.permStorage.games.length;
        me_apps.permStorage.games[index] = data.common;
    }
    var toChange=me_apps.permStorage.games[index];

    // There is some metadata (ticker secret ...) that the list doesn't send, but details do.
    if (flags & me_apps.DETAILS_HEADERDATA) {
        for (var k in data.common)
            toChange[k]=data.common[k];
    }
    if (flags & me_apps.DETAILS_REFEREES) {
        if (toChange.report === undefined)
            toChange.report=new me_apps.obj.spielbericht();
        if (flags & me_apps.DETAILS_REFEREE_1) {
            if (data.referees && data.referees["1"])             // altes Format
                toChange.report.refereeA=data.referees["1"];
            else if (data.report)
                toChange.report.refereeA=data.report.refereeA;
            else
                toChange.report.refereeA={};
        }
        if (flags & me_apps.DETAILS_REFEREE_2)
            if (data.referees && data.referees["2"])             // altes Format
                toChange.report.refereeB=data.referees["2"];
            else if (data.report)
                toChange.report.refereeB=data.report.refereeB;
            else
                toChange.report.refereeB={};
    }
    if (flags & me_apps.DETAILS_TEAM_GUEST) {
        if (!(data.team_guest.team_members instanceof Array)) {
            data.team_guest.team_members = [];
        }
        toChange.team_guest=data.team_guest;
    }
    if (flags & me_apps.DETAILS_TEAM_HOME) {
        if (!(data.team_home.team_members instanceof Array)) {
            data.team_home.team_members = [];
        }
        toChange.team_home=data.team_home;
    }
    if (flags & me_apps.DETAILS_REPORT) {
        toChange.report=data.report;
    }
    if (flags & me_apps.DETAILS_PROTOCOL) {
        toChange.protocol=data.protocol;     // nB the server doesn't send this at the moment - 20180125
    }
    if (toChange.report   === undefined)     toChange.report=new me_apps.obj.spielbericht();
    if (toChange.protocol === undefined)     toChange.protocol=[];
    
    if (data.class !== undefined) {
        toChange.report.refereeA   = toChange.report.refereeA || {};
        me_apps.db.fillClassCost(toChange.report.refereeA, data.class, 'ref');
        toChange.report.refereeB   = toChange.report.refereeB || {};
        me_apps.db.fillClassCost(toChange.report.refereeB, data.class, 'ref');
        toChange.report.timekeeper = toChange.report.timekeeper || {};
        me_apps.db.fillClassCost(toChange.report.timekeeper, data.class, 'tk');
        toChange.report.supervisor = toChange.report.supervisor || {};
        me_apps.db.fillClassCost(toChange.report.supervisor, data.class, 'del');
        toChange.report.secretary  = toChange.report.secretary || {};
        me_apps.db.fillClassCost(toChange.report.secretary, data.class, 'del');
        toChange.report.observer   = toChange.report.observer || {};
        me_apps.db.fillClassCost(toChange.report.observer, data.class, 'refw');
    }
    
    me_apps.db.persist();
    return index;
};

me_apps.obj.db.prototype.fillClassCost = 
function(person, cost, prefix) {
    if (cost[prefix+'_compensation'] > 0) {
        person.compensation = cost[prefix+'_compensation'];
        person.compensation_fixed = true;
    }
    if (cost[prefix+'_factor1_km'] > 0) {
        person.km1_factor = cost[prefix+'_factor1_km'];
        person.km1_factor_fixed = true;
    }
    if (cost[prefix+'_factor2_km'] > 0) {
        person.km2_factor = cost[prefix+'_factor2_km'];
        person.km2_factor_fixed = true;
    }
};

me_apps.obj.db.prototype.downloadReferee =
function(ab, searchparam, callback) {
    me_apps.log.logCall();
    var url=me_apps.db.getUrl('getReferee');
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'R');
    $.ajax({
        url: url,
        type: 'POST',
        dataType: "json",
        data: JSON.stringify(searchparam),
        async: true
    }).done(function(data, textStatus, jqXHR) {
        me_apps.log.debug(JSON.stringify(data));
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
        if (data.error === undefined) {
            var game=me_apps.db.getCurrentGame();
            if (ab==='A')
                game.report.refereeA=data;
            else if (ab==='B')
                game.report.refereeB=data;
	    else
                me_apps.log.error("downloadReferee() ab="+ab);		    
            me_apps.db.saveGame(game);
            callback(data, "");
        } else {
            callback(data, data.error);
        }
    }).fail(function(jqXHR, textStatus, errorThrown) {
        me_apps.log.warn("Load Referee: Error "+JSON.stringify(jqXHR));
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
        callback(null, "Serverfehler");
    }).always(function() {
    });
};



/*
 * ***************************************************************************
 * Upload stuff to the server
 * ***************************************************************************
 */

/**
 * Schedule a future upload of game data.
 * Can be called more than once; subsequent calls won't trigger another upload
 * until the first one is finished; so we collect several changes into one.
 * 
 * @param {type} game - the game to be uploaded
 * @param {type} wait - time to wait before uploading
 * @returns {undefined}
 */
me_apps.obj.db.prototype.scheduleUploadGameData =
function(game, wait) {
    me_apps.log.logCall();
    if (!me_apps.onlineState.isUploadEnabled())
        return;
    if (wait !== undefined && game._np_uploadScheduled) {
        clearTimeout(game._np_uploadScheduled);
        game._np_uploadScheduled=0;
    } else {
        wait=me_apps.UPLOAD_DELAY;
    }
    if (!game._np_uploadScheduled) {
        game._np_uploadScheduled=setTimeout(function() {
            me_apps.obj.db.prototype.uploadGameData(game);
        }, wait * 1000);
    }
};

me_apps.obj.db.prototype.addDeviceInfo =
function(game) {
    game.deviceinfo = {
        version:    me_apps.VERSION,
        build:      me_apps.BUILD,
        uuid:       me_apps.db.uuid,
        navigator:  {
            appCodeName:    navigator.appCodeName,
            appName:        navigator.appName,
            appVersion:     navigator.appVersion,
            buildID:        navigator.buildID,
            oscpu:          navigator.oscpu,
            userAgent:      navigator.userAgent,
            product:        navigator.product,
            language:       navigator.language,
            languages:      navigator.languages
        },
        dimensions: {
            devicePixelRatio:   window.devicePixelRatio,
            screen_width:       screen.width,
            screen_height:      screen.height,
            innerWidth:         innerWidth,
            innerHeight:        innerHeight,
            screenX:            screenX,
            screenY:            screenY
        }
    };
};

/**
 * @function uploadGameData
 * Normally used to upload data in the background, should be scheduled 
 *  by setTimeout(). If successful, clears _np_uploadScheduled and
 *  the _np_needUpload bitmap. If unsuccessful, re-schedules itself.
 * @param {Object} game - the game object to upload
 * @returns {undefined}
 */
me_apps.obj.db.prototype.uploadGameData =
function(game) {
    me_apps.log.logCall();
    if (!me_apps.onlineState.isUploadEnabled()) {
        game._np_uploadScheduled=0;
        return;
    }

    this.addDeviceInfo(game);
    var json=JSON.stringify(game, this.removeNPAttributes);
    delete this.deviceinfo;
    
    var command="gameUpload";
    var url=me_apps.db.getUrl(command);
    url += "&id="+game.id;
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'S');
    var change_before = game.change_count;
    // _np_uploadScheduled needs to be changed after the call is done, not now;
    // but it needs to be set to 0 before calling scheduleUploadGameData again.
    $.ajax({
        url: url,
        type: 'POST',
        data: json,
        dataType: 'json',
        async: true,
        timeout: me_apps.UPLOAD_DELAY *1000 *8 /10
    }) .done (function(data, textStatus, jqXHR) {
        me_apps.log.info("uploadGameData successful with data "+JSON.stringify(data)+" and status "+textStatus);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
        game._np_uploadScheduled=0;
        if (data.server_status.no === 0 && game.change_count === change_before) {
            game._np_needUpload=0;      // not false - bitmap
        } else {
            me_apps.db.scheduleUploadGameData(game, data.server_status.no === 0 ? 0 : me_apps.UPLOAD_DELAY);
        }
    }) .fail (function(jqXHR, textStatus, errorThrown) {
        me_apps.log.info("uploadGameData failed with data "+JSON.stringify(jqXHR)+" and status "+textStatus);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
        game._np_uploadScheduled=0;
        me_apps.db.scheduleUploadGameData(game, me_apps.UPLOAD_DELAY);
    });
};

/* ------------------ game finalization .. finalize, and put into background queue on error ---------------- */

/**
 * @function finalizeGame
 * Set the finalized flag on a game, and send it with finishReport. Try sending
 * even if sending is disabled from the UI. If sending fails. put the current
 * JSON string into a queue that keeps trying to re-send itself.
 * @param {Object} game - the game object to upload
 * @param {function} callback - a function to be called after upload. This
 * function gets the upload status as parameter.
 * @returns {undefined}
 */
me_apps.obj.db.prototype.finalizeGame =
function(game, callback) {
    me_apps.log.logCall();
    var isFirstFinalize = false;
    var replacedByFirstFinalize = false;
    if (!game.finalized) {
        game.finalized=true;
        isFirstFinalize = true;
    }
    var url=me_apps.db.getUrl("finishReport");
    url += "&id="+game.id;
    
    this.addDeviceInfo(game);
    var uploadString=JSON.stringify(game, this.removeNPAttributes);
    delete this.deviceinfo;
    
    me_apps.db.addFinalizeUpload(url, uploadString, game.id);
    if (isFirstFinalize) {
        me_apps.permStorage.savedFirstFinalizes["Game"+game.id] = uploadString;
        me_apps.db.persist();
    } else if (me_apps.permStorage.savedFirstFinalizes["Game"+game.id] !== "") {
        uploadString = me_apps.permStorage.savedFirstFinalizes["Game"+game.id];
        replacedByFirstFinalize = true;
    }

    var success=false;
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'B');
    $.ajax ({
        url: url,
        type: 'POST',
        data: uploadString,
        dataType: 'json',
        async: true,
        timeout: me_apps.UPLOAD_DELAY *1000 * 8/10
    }) .done (function(data, textStatus, jqXHR) {
        me_apps.log.info("finalizeGame successful with data "+JSON.stringify(data)+" and status "+textStatus);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
        if (data.server_status.no === 0)
            success=true;
        if (callback)
            callback("done", data, textStatus);
    }) .fail (function(jqXHR, textStatus, errorThrown) {
        me_apps.log.info("finalizeGame failed with data "+JSON.stringify(jqXHR)+", status "+textStatus+ " and error "+errorThrown);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
        if (callback)
            callback("fail", jqXHR, textStatus);
    }) .always(function() {
        if (success === true) {
            if (!replacedByFirstFinalize) {
                // In this case, we have sent that version, can confirm it, and
                // we will already have added the "current" version to the list.
                me_apps.permStorage.finalizedGames[0].confirmed=true;
            }
        }
        // If we were able to send data now, process the queue rapidly, else,
        // retry in a few minutes, it won't make sense immediately. 
        me_apps.db.scheduleFinalizedUploads(success);
    });
};

me_apps.obj.db.prototype.finalizedUploadsAllConfirmed = function() {
    for (var i=0; i<me_apps.permStorage.finalizedGames.length; i++) {
        var entry=me_apps.permStorage.finalizedGames[i];
        if (!entry.confirmed) {
            return false;
        }
    }
    return true;
};

me_apps.obj.db.prototype.getPendingFinalizedUploads = function() {
    var count=0;
    for (var i=0; i<me_apps.permStorage.finalizedGames.length; i++) {
        var entry=me_apps.permStorage.finalizedGames[i];
        if (!entry.confirmed)
            count++;
    }
    return count;
};

me_apps.obj.db.prototype.cleanupFinalizedUploads = function() {
// Only clear the array if everything is confirmed, as that ensures
// there are no outstanding AJAX calls left.
    if (this.finalizedUploadsAllConfirmed()) {
        me_apps.permStorage.finalizedGames = [];
        me_apps.db.persist();
    }
};

me_apps.obj.db.prototype.addFinalizeUpload = function(url, uploadString, id) {
    this.cleanupFinalizedUploads();
    me_apps.permStorage.finalizedGames.unshift({ url: url, confirmed: false, uploadString: uploadString, id: id });
    me_apps.db.persist();
};

me_apps.obj.db.prototype.scheduleFinalizedUploads = function(doItNow) {
    if (this.timeoutScheduled) {
        clearTimeout(this.timeoutScheduled);
        this.timeoutScheduled=false;
    }
    this.cleanupFinalizedUploads();
    this.timeoutScheduled=setTimeout(function() { me_apps.db.performFinalizedUploads(); }, doItNow ? 100 : me_apps.UPLOAD_DELAY*1000);
};

me_apps.obj.db.prototype.performFinalizedUploads = function() {
    // Only start one single upload. When done, we will get scheduled again.
    // But we don't want more than 1 ajax call outstanding.
    for (var i=0; i<me_apps.permStorage.finalizedGames.length; i++) {
        var entry=me_apps.permStorage.finalizedGames[i];
        if (!entry.confirmed) {
            this.performOneFinalizedUpload(entry);
            return;
        }
    }
};

me_apps.obj.db.prototype.performOneFinalizedUpload = function(entry) {

    this.timeoutScheduled = false;
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'A');
    $.ajax ({
        url: entry.url,
        type: 'POST',
        data: entry.uploadString,
        dataType: 'json',
        async: true,
        timeout: me_apps.UPLOAD_DELAY *1000 *8 /10
    }) .done (function(data, textStatus, jqXHR) {
        me_apps.log.info("performOneFinalizedUpload successful with data "+JSON.stringify(data)+" and status "+textStatus);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
        if (data.server_status.no === 0) {
            entry.confirmed=true;
            me_apps.einstellungenDialog.setUploadButtonCount();
            me_apps.spiel.patchSettingsNavButton();
            me_apps.db.persist();
            me_apps.db.scheduleFinalizedUploads(true);
        } else {
            me_apps.db.scheduleFinalizedUploads(false);
        }
    }) .fail (function(jqXHR, textStatus, errorThrown) {
        me_apps.log.info("performOneFinalizedUpload failed with data "+JSON.stringify(jqXHR)+", status "+textStatus+ " and error "+errorThrown);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
        me_apps.db.scheduleFinalizedUploads(false);
    });
};

/* ----------------- Ticker upload --------------------- */

me_apps.obj.db.prototype.scheduleUploadTicker =
function(game, wait) {
    me_apps.log.logCall();
    if (typeof("wait")==="undefined") {
        wait=1;
    }
    if (game._np_tickerScheduled!==true) {
        game._np_tickerScheduled=true;
        setTimeout(function() {
            me_apps.obj.db.prototype.uploadTicker(game);
        }, wait*1000);
    }
};

me_apps.obj.db.prototype.uploadTicker =
function(game) {
    me_apps.log.logCall();
    game._np_tickerScheduled=false;
    if (!me_apps.onlineState.isUploadEnabled())
        return;
    
    var protocolList=game.protocol;
    if (typeof(protocolList)==="undefined") {
        protocolList=game.protocol=[];
    }
    
    var url=me_apps.db.getUrl("tickerUpload");
    url += "&secret="+game.secret;
    url += "&id="+game.id;

    var now=Date.now();
    for (var i=0; i<protocolList.length; i++) {
        if (protocolList[i].uploadTime===0) {
            var toUpload=protocolList[i];
            if (protocolList[i].creationTime + 3600*1000*2 < now) {
                protocolList[i].uploadTime = now;
                protocolList[i].uploadError="aborted after 2 unsuccessful hours";
                continue;
            }
            toUpload.gameId=game.id;
            me_apps.onlineState.setState(me_apps.onlineState.SENDING, 'T');
            $.ajax({
                url: url,
                type: 'POST',
                data: JSON.stringify(toUpload),
                dataType: 'json',
                async: true,
                cache: false,
                indexValue: i           // custom item, see https://stackoverflow.com/a/20384128/3066467
            }) .done (function(data, textStatus, jqXHR) {
                me_apps.log.info("uploadTicker for "+this.indexValue+" successful with data "+JSON.stringify(data)+" and status "+textStatus);
                me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
                if (data.no === 0) {
                    protocolList[this.indexValue].uploadTime=Date.now();
                }
                me_apps.db.scheduleUploadTicker(game, 1);
            }) .fail (function(jqXHR, textStatus, errorThrown) {
                me_apps.log.info("uploadTicker for "+this.indexValue+" failed with data "+JSON.stringify(jqXHR)+" and status "+textStatus);
                me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
                me_apps.db.scheduleUploadTicker(game, 10);
            }) .always (function() {
                me_apps.obj.db.prototype.persist();
            });
            // Only one at a time; if we found an entry, ajax.always will schedule us again
            break;
        }
    }
};

me_apps.obj.db.prototype.checkServerConnection = function() {
    me_apps.log.logCall();
    var url=me_apps.db.getUrl("ping");          // TODO Statusabfrage in den Server einbauen
    me_apps.onlineState.setState(me_apps.onlineState.SENDING, '?');
    $.ajax({
        url: url,
        type: 'POST',
        dataType: 'json',
        async: true,
        cache: false,
        timeout: 3000
    }) .done (function(data, textStatus, jqXHR) {
        me_apps.log.info("checkServerConnection successful with data "+JSON.stringify(data)+" and status "+textStatus);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_OK);
    }) .fail (function(jqXHR, textStatus, errorThrown) {
        me_apps.log.info("checkServerConnection failed with data "+JSON.stringify(jqXHR)+" and status "+textStatus);
        me_apps.onlineState.setState(me_apps.onlineState.SEND_FAILED);
    });
};

me_apps.obj.db.prototype.getUrl =
function(f) {
    var url=me_apps.URL_PROTOKOLL_SERVER + "cmd=" + f + "&version=" + me_apps.VERSION + "." + me_apps.BUILD + "&v_interface=" + me_apps.V_INTERFACE + "&uuid=" + me_apps.db.uuid;
    return url;
};
