if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}
if (typeof (me_apps.log) === 'undefined') {
    me_apps.log = {};
    me_apps.log.entries = [];
    // Getting old entries from localStorage won't work until the logging
    // functions are defined so let's do that later.
}

me_apps.log.LEVEL_ALL=0;
me_apps.log.LEVEL_VERBOSE=1;
me_apps.log.LEVEL_DEBUG=2;
me_apps.log.LEVEL_INFO=3;
me_apps.log.LEVEL_IMPORTANT=4;
me_apps.log.LEVEL_WARN=5;
me_apps.log.LEVEL_ERROR=6;
me_apps.log.LEVEL_FATAL=7;
me_apps.log.LEVEL_OFF=8;

me_apps.log.logLevelConsole=me_apps.log.LEVEL_INFO;
me_apps.log.logLevelServer =me_apps.log.LEVEL_IMPORTANT;
me_apps.log.logLevelAlert  =me_apps.log.LEVEL_ERROR;

me_apps.log.log =
function(level, string) {
    if (me_apps.log.logLevelConsole <= level) {
        if (level <= me_apps.log.LEVEL_INFO)
            console.info(string);
        else if (level <= me_apps.log.LEVEL_WARN)
            console.warn(string);
        else
            console.error(string);
    }
    if (me_apps.log.logLevelServer  <= level) {
        me_apps.log.addToLog(string);
    }
    if (me_apps.log.logLevelAlert   <= level) {
        alert(string);
    }
};

me_apps.log.verbose  =function(string) {   me_apps.log.log(me_apps.log.LEVEL_VERBOSE,     string); };
me_apps.log.debug    =function(string) {   me_apps.log.log(me_apps.log.LEVEL_DEBUG,       string); };
me_apps.log.info     =function(string) {   me_apps.log.log(me_apps.log.LEVEL_INFO ,       string); };
me_apps.log.important=function(string) {   me_apps.log.log(me_apps.log.LEVEL_IMPORTANT,   string); };
me_apps.log.warn     =function(string) {   me_apps.log.log(me_apps.log.LEVEL_WARN ,       string); };
me_apps.log.error    =function(string) {   me_apps.log.log(me_apps.log.LEVEL_ERROR,       string); };
me_apps.log.fatal    =function(string) {   me_apps.log.log(me_apps.log.LEVEL_FATAL,       string); };

/*
 * In den folgenden Funktionen ist new Error() relativ teuer, deshalb haben sie
 * den nicht unbedingt nötigen (if logLevel <= level) drumrum. Das bedeutet
 * aber auch, dass logLevelConsole kleiner als die andren sein sollte.
 */

me_apps.log.logPosition = 
function(message) {
    if (me_apps.log.logLevelConsole <= me_apps.log.LEVEL_DEBUG) {
        me_apps.log.log(me_apps.log.LEVEL_DEBUG,
            (new Error().stack.split("\n")[2].replace(/@.*\//, " @ "))
            + (typeof message === "undefined" ? "" : " "+message)
        );
    }
};

me_apps.log.logCall = 
function() {
    if (me_apps.log.logLevelConsole <= me_apps.log.LEVEL_DEBUG) {
        var callstack=new Error().stack.split("\n");
        var calledfunc=callstack[2].replace(/@.*/, "");
        var caller    =callstack[3].replace(/.*\//, "");
        me_apps.log.log(me_apps.log.LEVEL_DEBUG, calledfunc + " called from "+caller);
    }
};

me_apps.log.addToLog =
function (text) {
    var now=new Date();
    var logEntry = {
        ms: now.getTime(),
        dateText: now.toISOString(),
        text: text
    };
    var log=me_apps.log.entries;
    if (log.length > 125) {
        log.splice(0, log.length-75);
    }
    log.push(logEntry);
    me_apps.log.persist();
};

me_apps.log.cleanupLog =
function () {
    me_apps.log.logCall();
    var nowms=new Date().getTime();
    var log=me_apps.log.entries;
    for (var i=0; i<log.length && log[i].ms<nowms-86400*1000*1; i++)        // @TODO : 21 statt 1 wenns mal prod ist
        ;
    log.splice(0, i);
    me_apps.log.persist();
};

me_apps.log.uploadLog =
function() {
    var url=me_apps.URL_PROTOKOLL_SERVER + "cmd=uploadLog&version=" + me_apps.VERSION + "." + me_apps.BUILD + "&v_interface=" + me_apps.V_INTERFACE + "&uuid=" + me_apps.db.uuid;
    $.mobile.loading( "show" , { text: "Anwendungsprotokoll wird gesendet", textVisible: true, theme: "a" });
    $.ajax({
        url: url,
        type: 'POST',
        dataType: "json",
        data: JSON.stringify(me_apps.log.entries),
        async: true
    }).done(function(data, textStatus, jqXHR) {
        $.mobile.loading( "show" , { text: "Anwendungsprotokoll gesendet", textVisible: true, theme: "a" });
        me_apps.log.entries=[];
        me_apps.log.persist();
    }).fail(function() {
        $.mobile.loading( "show" , { text: "Fehler beim Senden des Protokolls", textVisible: true, theme: "a" });
    }).always(function() {
        setTimeout(function() { $.mobile.loading("hide"); }, 3000);
    });
};

me_apps.log.persist =
function() {
    me_apps.log.logCall();;
    localStorage.setItem("AppLog", JSON.stringify(me_apps.log.entries));
};


// Get log entries from localStorage. We can't do that until now because
// we need me_apps.log.error defined.

var serialized;

if (typeof(localStorage) === "undefined") {
    me_apps.log.error("Browser does not support LocalStorage");
} else {
    if ((serialized=localStorage.getItem("AppLog"))===null) {
        me_apps.log.entries = [];
        me_apps.log.info("Initializing new log entries");
    } else {
        me_apps.log.entries=JSON.parse(serialized);
        me_apps.log.info("Log entries read from AppLog");
        me_apps.log.cleanupLog();
    }
}
