/*
 * Handle the UI of the main dialog
 */

if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}
if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.mainDialog = function () {};

/**
 * @param {char} team - team who scored the goal, 'G' or 'H'
 * @param {Integer} id - internal id of the player who scored the goal
 * @param {Bool} siebenMeter decides the type of protocol entry
 */

me_apps.obj.mainDialog.prototype.addTor =
function (team, id, siebenMeter) {
    var mannschaft = (team === "G") ? me_apps.spiel.gast : me_apps.spiel.heim;
    var spieler = mannschaft.getSpielerById(id);
    me_apps.spiel.goalScored(mannschaft, spieler);
    if (siebenMeter) {
        me_apps.protokollDialog.addProtocol(mannschaft, spieler.id, "7MT");
    } else {
        me_apps.protokollDialog.addProtocol(mannschaft, spieler.id, "T");
    }
    me_apps.mainDialog.mannschaftGoalCountToGUI(mannschaft);
    me_apps.mainDialog.spielerGoalCountToGUI(mannschaft, spieler);
};

/**
 * Add a penalty to a team member. 
 * @param {char} team - G or H
 * @param {integer} id - the ID of the team member. May be A,B,C,D in case of officers.
 * @param {string} strafe - 'H', 'D', 'B'
 * @param {integer} duration Duration, only used in case of suspension (H). 
 *                  Not the game default if we're editing the protocol.
 * @returns {undefined}
 */
me_apps.obj.mainDialog.prototype.addStrafe = function (team, id, strafe, duration) {
    var mannschaft = (team === "G") ? me_apps.spiel.gast : me_apps.spiel.heim;
    var spieler = mannschaft.getSpielerById(id);
    spieler.addDiscipline(team, strafe);
    me_apps.obj.mainDialog.prototype.penaltiesToGUI(mannschaft, spieler);
    
    if (strafe === 'H' || strafe === 'D' || strafe === "B") {
        me_apps.spielerDialog.addZweiMinutenStrafe(mannschaft, id, duration);
    }
};

/**
 * Remove a penalty from a team member. Used when editing the protocol.
 * @param {char} team - G or H
 * @param {integer} id - the ID of the team member. May be A,B,C,D in case of officers.
 * @param {string} strafe
 * @returns {undefined}
 * TODO: If a team member has more than one suspension, how do we find the
 * correct one? Example: Get one 2 minute suspension. Wait 10 secs. Get another
 * 2 minute suspension. Wait 20 secs. This code seems to delete both!
 */
me_apps.obj.mainDialog.prototype.deleteStrafe = function (team, id, strafe) {
    var mannschaft = (team === "G") ? me_apps.spiel.gast : me_apps.spiel.heim;
    var spieler = mannschaft.getSpielerById(id);
    spieler.removeDiscipline(team, strafe);
    me_apps.obj.mainDialog.prototype.penaltiesToGUI(mannschaft, spieler);

    if (strafe === 'H' || strafe === 'D' || strafe === "B" || strafe === 'Z') {
        for(var i = 0; i < mannschaft.hinausstellungen.length; i++) {
            var element = mannschaft.hinausstellungen[i];
            if (element.id === id) {
                    var rest = element.ende - me_apps.spiel.zeit;
                    if (rest > me_apps.spiel.dauerHinausstellung) {
                            element.ende = element.ende - me_apps.spiel.dauerHinausstellung;
                    } else {
                            mannschaft.hinausstellungen.splice($.inArray(element, mannschaft.hinausstellungen), 1);
                            i--;
                    }
            }
        };
        me_apps.db.updateTeam(mannschaft);
        me_apps.mainDialog.suspensionsToGUI(mannschaft);
    }
};

/**
 * Show the table of suspensions a team has.
 * @param {me_apps.obj.mannschaft} mannschaft
 * @returns {undefined}
 */
me_apps.obj.mainDialog.prototype.suspensionsToGUI = function (mannschaft) {

    var html = "";
    for(var i = 0; i < mannschaft.hinausstellungen.length; i++) {
	var element = mannschaft.hinausstellungen[i];
        var rest = element.ende - me_apps.spiel.zeit;
        if (rest > 0) {
            var nr = mannschaft.getSpielerById(element.id).getNr();
            html += "<tr><td>" + nr + "</td><td>" + Math.floor(rest / 60) + ":" + ("0" + rest % 60).slice(-2) + "</td></tr>";
        } else {
		mannschaft.hinausstellungen.splice($.inArray(element, mannschaft.hinausstellungen), 1);
                // db.updateTeam not needed here. If we reload the undeleted suspension it will just get removed again.
		i--;
        }
    };
    var selector = '#ZweiMinuten' + mannschaft.selector.toString() + '  tbody';
    $(selector)[0].innerHTML = html;
};

/**
 * @param {me_apps.obj.mannschaft} mannschaft
 * Shows the number of goals the team scored.
 */
me_apps.obj.mainDialog.prototype.mannschaftGoalCountToGUI =
function(mannschaft) {
    $(('#Tore' + mannschaft.selector)).text(mannschaft.getTore());
};

/**
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {me_apps.obj.spieler} spieler
 * Shows the number of goals a certain player scored.
 */
me_apps.obj.mainDialog.prototype.spielerGoalCountToGUI =
function(mannschaft, spieler) {
    var toreSelector = '#Spieler' + mannschaft.selector.toString() + spieler.id + ' > td';
    var tore = $(toreSelector)[1];
    tore.innerHTML = spieler.tore;
};

/**
 * @param {me_apps.obj.mannschaft} mannschaft "heim" oder "gast" zB aus me_apps.obj.mannschaft.selector
 * @param {me_apps.obj.player}  player  the team member whose penalties should be shown
 * Shows the penalty string for a player or officer
 */

me_apps.obj.mainDialog.prototype.penaltiesToGUI =
function(mannschaft, player) {
    // Hier ist das Problem, dass Verwarnungen auch an Offizielle gehen können,
    // und dass diese nicht unbedingt eine ID haben. Deshalb können auch die
    // Tabellenelemente nicht mit der id bezeichnet werden, sondern haben leider
    // das Suffix A, B, C, oder D.
    var suffix;
    if (player.isOfficer())
        suffix=player.bezeichner;
    else
        suffix=player.id;
    var selector = '#Spieler' + mannschaft.selector + suffix + ' > td';
    var disc = $(selector)[2];
    disc.innerHTML = player.getDisciplineString();
};

/**
 * update the game Time in the GUI - bottom center of the main screen, and
 * the 'Spielzeit korrigieren' dialog.
 * @param {integer} gameTime time in seconds
 */
me_apps.obj.mainDialog.prototype.gameTimeToGUI =
function(gameTime) {
    $("#Time").text(me_apps.spiel.timeToMMSS(gameTime));
    if (!$("#SpielzeitMinuten").is(":focus")) {
        $("#SpielzeitMinuten").val(("0" + Math.floor(gameTime / 60)).slice(-2));
    }
    if (!$("#SpielzeitSekunden").is(":focus")) {
        $("#SpielzeitSekunden").val(gameTime % 60);
    }
};

/**
 * Mark game time running in the GUI, by setting/removing classes that change
 * the game time running display to red or green.
 * @param {boolean} isRunning
 */
me_apps.obj.mainDialog.prototype.gameTimeRunningToGUI =
function(isRunning, isBlocked) {
    var elem=$("#Time");
    if (isBlocked) {
        elem.removeClass("ZeitGestoppt");
        elem.removeClass("ZeitLaeuft");
        elem.addClass("ZeitBlockiert");
    }
    else if (isRunning) {
        elem.removeClass("ZeitGestoppt");
        elem.removeClass("ZeitBlockiert");
        elem.addClass("ZeitLaeuft");
    } else {
        elem.removeClass("ZeitLaeuft");
        elem.removeClass("ZeitBlockiert");
        elem.addClass("ZeitGestoppt");
    }
};

/**
 * Shows timeout buttons for both teams. The number of buttons depends on the
 * game settings.
 */
me_apps.obj.mainDialog.prototype.createTimeoutButtons = function () {
    if (me_apps.spiel.anzahlTimeOut === 0) {
        $("#Timeouts").hide();
    } else {
        var html = "<div class='Farbeheim'>Auszeit Heim</div>";
        for (i = 1; i <= me_apps.spiel.anzahlTimeOut; i++) {
            html += "<div class='Farbeheim'>" + i + "</div>";
        }
        $("#Timeoutheim")[0].innerHTML = html + "</div>";

        html = "<div class='Farbegast'>Auszeit Gast</div>";
        for (i = 1; i <= me_apps.spiel.anzahlTimeOut; i++) {
            html += "<div class='Farbegast'>" + i + "</div>";
        }
        $("#Timeoutgast")[0].innerHTML = html + "</div>";
        $("#Timeouts").show();
    }
    me_apps.mainDialog.disableTimeouts();
};

/**
 * Show the remaining timeout time. Play the buzzer when the timeout duration
 *  is reached.
 * @param {type} seconds
 * @returns {undefined}
 */
me_apps.obj.mainDialog.prototype.timeoutTimeToGUI =
function(seconds) {
    $("#TimeOut").text(me_apps.spiel.timeToMMSS(seconds));
    if (seconds === me_apps.spiel.dauerTimeout && me_apps.spiel.buzzerNachTimeout) {
        document.getElementById('Buzzer').cloneNode(true).play();
    }
};

/**
 * Bind or Unbind the click function to/from timeout buttons
 * @param {type} flag enable or disable the timeout buttons
 */

me_apps.obj.mainDialog.prototype.setTimeoutButtonsEnabled = 
function(flag) {
    $("#Timeoutheim").unbind('vclick');
    $("#Timeoutgast").unbind('vclick');

    if (flag) {
        $("#Timeoutheim").bind('vclick', function () {
            me_apps.mainDialog.startTimeout(me_apps.spiel.heim);
        });
        $("#Timeoutgast").bind('vclick', function () {
            me_apps.mainDialog.startTimeout(me_apps.spiel.gast);
        });
    }
};

/**
 * Start a timeout for a team.
 * @param {me_apps.obj.mannschaft} mannschaft
 */

me_apps.obj.mainDialog.prototype.startTimeout =
function(mannschaft) {
    if (me_apps.spiel.startTimeout(mannschaft)) {
        me_apps.mainDialog.disableTimeouts();
        me_apps.protokollDialog.addProtocol(mannschaft, "-", "Auszeit");
    }
};

/**
 * Show the availability of timeouts to a team - normal color, or red and
 * strikethrough, depending on how many timeouts the team has already taken.
 * @param {me_apps.obj.mannschaft} mannschaft The team. Can be omitted; in this
 * case both teams are updated.
 * @returns {boolean} true if there was a mannschaft parameter and that team had
 * at least one timeout available.
 */
me_apps.obj.mainDialog.prototype.timeoutAvailabilityToGUI =
function(mannschaft) {
    if (mannschaft === undefined) {
        this.timeoutAvailabilityToGUI(me_apps.spiel.heim);
        this.timeoutAvailabilityToGUI(me_apps.spiel.gast);
        return false;
    } else {
        var selectorStart="#Timeout"+mannschaft.selector+" div:nth-child";
        var anyAvailable=false;
        for (var i=0; i<me_apps.spiel.anzahlTimeOut; i++) {
            var selector=selectorStart+"("+(i+2)+")";
            var element=$(selector);
            var isTaken, isBlocked;
            if (mannschaft.timeoutsTaken[i]) {
                isTaken=true;
                element.addClass('taken');
            } else {
                isTaken=false;
                element.removeClass('taken');
            }
            
            if (me_apps.spiel.zeit < me_apps.spiel.dauerHalbzeit && i === me_apps.spiel.anzahlTimeOut-1            // letzter (2. oder 3.) Timeout nicht in der 1. HZ
            ||  me_apps.spiel.zeit >= me_apps.spiel.dauerHalbzeit && i === 0                                       // 1. Timeout nur in der 1. HZ
            ||  me_apps.spiel.zeit >= me_apps.spiel.dauerHalbzeit*2-300 && i === 2 && (                            // 3. Timeout in den letzten 5 Minuten ....
                mannschaft.timeoutsTaken[1] === false                                                              //  ... nicht wenn der 2. nicht genommen wurde
                || mannschaft.timeoutsTaken[1] >= me_apps.spiel.dauerHalbzeit*2-300                                //  ... und auch nicht wenn der 2. in den letzten 5 Minuten war
            )
            ||  me_apps.spiel.zeit >= me_apps.spiel.dauerHalbzeit*2                                                // keine Timeouts in der Verlängerung
            ) {
                isBlocked=true;
                $(selector).addClass('notavailable');
            }
            else {
                isBlocked=false;
                $(selector).removeClass('notavailable');
            }
            if (!isBlocked && !isTaken)
                anyAvailable=true;
        }
        return anyAvailable;
    }
};

/**
 * Enable timeouts buttons - bind click action, show availability of
 * remaining timeouts
 * @returns {undefined}
 */
me_apps.obj.mainDialog.prototype.enableTimeouts = function () {
    this.setTimeoutButtonsEnabled(true);
    if (this.timeoutAvailabilityToGUI(me_apps.spiel.heim))
        $("#Timeoutheim").removeClass('disabled');
    if (this.timeoutAvailabilityToGUI(me_apps.spiel.gast))
        $("#Timeoutgast").removeClass('disabled');
};

/**
 * Disable timeout buttons - unbind click actions and grey them out
 * @returns {undefined}
 */
me_apps.obj.mainDialog.prototype.disableTimeouts = function () {
    this.setTimeoutButtonsEnabled(false);
    this.timeoutAvailabilityToGUI();
    $("#Timeoutheim").addClass('disabled');
    $("#Timeoutgast").addClass('disabled');
};
