if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}
if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.mannschaft = function (name, selector) {
    this.setDefaultValues(name, selector);
};

me_apps.obj.mannschaft.prototype.setDefaultValues = 
function(name, selector) {
    this.name = name;
    this.spieler = new Array();
    this.offizielle = new Array();
    this.selector = selector;
    if (this.selector === "heim") {
        // $("#Nameheim").text(this.name);      ghört net hier her
        this.keeperColor="FF0000";
        this.playerColor="808000";
    }
    if (this.selector === "gast") {
        // $("#Namegast").text(this.name);      ghört net hier her
        this.keeperColor="000000";
        this.playerColor="FFFF00";
    }
    this.pin = null;
    this.hinausstellungen = new Array();
    this.status = "";
    this.token="";
    this.quotaPlayer=this.quotaOfficer=0;
    this.timeoutsTaken=[false, false, false];
};

/*
 * @function mannschaft.copyValuesFromPermStore
 * @param {type} dbEntry        // team object in database format
 * @returns {undefined}
 */

me_apps.obj.mannschaft.prototype.copyValuesFromPermStore =
function(dbEntry) {
    this.name               = dbEntry.team_name;
    this.spieler            = new Array();
    this.offizielle         = new Array();
    this.pin                = dbEntry.pin_check;
    this.hinausstellungen   = dbEntry.suspensions;
    if (this.hinausstellungen === undefined) {
        this.hinausstellungen = new Array();
    }
    this.status             = dbEntry.status;
    this.timeoutsTaken      = dbEntry.timeouts_taken || [ false, false, false ];
    this.token              = dbEntry.ticker_hash;
    this.encrypted          = dbEntry.encrypted;
    
    this.quotaPlayer        = dbEntry.quota_player || 0;
    this.quotaOfficer       = dbEntry.quota_officer || 0;
    
    // If we don't have values in the database yet, we don't assign them
    // at all. Default values will have been provided within setDefaultValues
    // that should have been called by the constructor before this.
    if (dbEntry.player_color !== "")        this.playerColor = dbEntry.player_color;
    if (dbEntry.keeper_color !== "")        this.keeperColor = dbEntry.keeper_color;
    
    this.copyTeamMembersFromPermStore(dbEntry.team_members);
};

/*
 * @function mannschaft.copyValuesToPermStore
 * @param {type} dbEntry        // team object in database format
 * @returns {undefined}
 */

me_apps.obj.mannschaft.prototype.copyValuesToPermStore =
function(dbEntry) {
    dbEntry.team_name           = this.name;
    dbEntry.pin_check           = this.pin;
    dbEntry.suspensions         = this.hinausstellungen;
    dbEntry.status              = this.status;
    dbEntry.timeouts_taken      = this.timeoutsTaken;
    dbEntry.ticker_hash         = this.token;

    dbEntry.player_color        = this.playerColor;
    dbEntry.keeper_color        = this.keeperColor;
    dbEntry.encrypted           = this.encrypted;
    dbEntry.quota_player        = this.quotaPlayer;
    dbEntry.quota_officer       = this.quotaOfficer;
};

me_apps.obj.mannschaft.prototype.copyTeamMembersFromPermStore =
function(dbList) {
    var ns=0, no=0;
    this.offizielle=[];
    this.spieler=[];
    for (i=0; i<dbList.length; i++) {
        var desc=dbList[i].player_no;
        if (desc !== "" && "ABCD".indexOf(desc)!==-1) {
            if (this.offizielle[no]===undefined)
                this.offizielle[no]=new me_apps.obj.spieler();
            this.offizielle[no].copyValuesFromPermStore(dbList[i]);
            no++;
        } else {
            if (this.spieler[ns]===undefined)
                this.spieler[ns]=new me_apps.obj.spieler();
            this.spieler[ns].copyValuesFromPermStore(dbList[i]);
            ns++;
        }
    }
    this.sortOfficers();
    this.sortSpieler();
};

me_apps.obj.mannschaft.prototype.sortOfficers =
function() {
    this.offizielle.sort(function (s1, s2) {
        var ret = s1.bezeichner.charCodeAt(0) - s2.bezeichner.charCodeAt(0);
        return ret;
    });
};

me_apps.obj.mannschaft.prototype.sortSpieler =
function() {
    this.spieler.sort(function (s1, s2) {
        var b1=s1.bezeichner || 0;
        var b2=s2.bezeichner || 0;
        var ret = b1-b2;
//        console.log("sortSpieler: "+b1+"/"+b2+": "+ret);
        return ret;
    });
};

me_apps.obj.mannschaft.prototype.toGUI =
function() {
    me_apps.spielbericht.teamToGUI((this.selector === 'heim') ? 2 : 3, this);
    this.farbeToGUI();
};

me_apps.obj.mannschaft.prototype.getTore = function () {
    var result = 0;
    this.spieler.forEach(function (spieler) {
        // Solange die Mannschaft verschlüsselt ist werden Offizielle nicht
        // als solche erkannt sondern als Spieler geführt, und dann kann
        // spieler.tore undefined sein, was als result zu NAN führt. Deshalb
        // die Abfrage.
        if (spieler.tore)
            result += spieler.tore;
    });
    this.offizielle.forEach(function (officer) {
        if (officer.tore)
            result += officer.tore;
    });
    return result;
};

me_apps.obj.spieler.prototype.getNr = function () {
    return this.bezeichner;
};
me_apps.obj.spieler.prototype.getId = function () {
    return this.id;
};

/**
 * @function changeFarbe
 * @param {obj} select id the GUI element that got changed
 * @param {obj} element 
 */
me_apps.obj.mannschaft.prototype.changeFarbe = function (select, element) {
    var value = $(select).val();
    if (value !== this[element]) {
        me_apps.log.info("New color "+value+" for "+element+" from "+select);
        this[element]=value;
        me_apps.db.updateTeam(this, true);
    }
    // farbeToGUI may be neccesary if we just loaded a new game. In this case,
    // changeFarbe doesn't need to change the color of the team, but we still
    // need to update the screen.
    this.farbeToGUI();
};

me_apps.obj.mannschaft.prototype.farbeToGUI = function () {
    // gehört net hierher sondern in ein *dialog.js. Aber dies setzt die
    // Farben im Hauptfenster UND im Berichtsfenster weil beide einen
    // entsprechenden Klasseneintrag haben.
    var guiclass='Farbe'+this.selector;
    var backgroundColors = {
        "000000": "#FFFFFF",
        "808080": "#FFFFFF",
        "C0C0C0": "#000000",
        "FFFFFF": "#000000",
        "FFFF00": "#000000",
        "00FF00": "#000000",
        "008000": "#FFFFFF",
        "808000": "#FFFFFF",
        "008080": "#FFFFFF",
        "00FFFF": "#000000",
        "0000FF": "#FFFFFF",
        "000080": "#FFFFFF",
        "800080": "#FFFFFF",
        "FF0000": "#FFFFFF",
        "800000": "#FFFFFF",
        "FFA500": "#FFFFFF",
	"FF99BB": "#FFFFFF"
    };
    $(".Farbe"+this.selector).css('color', backgroundColors[this.playerColor]);
    $(".Farbe"+this.selector).css('background-color', '#'+this.playerColor);
    $(".FarbeTorhüter"+this.selector).css('color', backgroundColors[this.keeperColor]);
    $(".FarbeTorhüter"+this.selector).css('background-color', '#'+this.keeperColor);
};

me_apps.obj.mannschaft.prototype.istDraussen = function (nr) {
    var result = false;
    var rows = $('#ZweiMinuten' + this.selector + ' tr');
    $.each(rows, function (i, row) {
        if (row.cells[0].innerHTML === nr) {
            result = true;
        }
    });
    return result;
};

/**
 * @function getGastAsInt
 * @return {int} heim = 0 gast = 1
 */
me_apps.obj.mannschaft.prototype.getGastAsInt = function () {
    var gast = 0;
    if (this.selector === "gast") {
        gast = 1;
    }
    return gast;
};

/**
 * @function addSpieler
 * Add a player to the list of players of a team. Does NOT check any game 
 * rules (so this function can't fail), but does update the quota counts if
 * neccesary.
 * @param {me_apps.obj.spieler} spieler
 * @param {boolean} addToDatabase set this if the officer is a new object and
 * has to be added to the permanent database. Do not set this when moving team
 * members between officers, active players, and inactive players.
 */
me_apps.obj.mannschaft.prototype.addSpieler = function (spieler, addToDatabase) {
    var needSave = false;
    if (me_apps.spiel.zeit > 0 && spieler.aktiv && !spieler.countedInPlayerQuota) {
        this.quotaPlayer--;
        spieler.countedInPlayerQuota=true;
        var dbTeam;
        if (this.selector == 'heim') {
            dbTeam = me_apps.db.getCurrentGame().team_home;
        } else {
            dbTeam = me_apps.db.getCurrentGame().team_guest;
        }
        dbTeam.quota_player = this.quotaPlayer;
        me_apps.db.tryUpdateSpielerInTeam(dbTeam, spieler);
        needSave = true;        
    }
    this.spieler.push(spieler);
    this.sortSpieler();
    if (addToDatabase) {
        me_apps.db.addManualTeamMember(this, spieler);
        me_apps.protokollDialog.addProtocol(this, spieler.id, 
            "Einsatz",
            "Spieler " + spieler.bezeichner + " eingesetzt");
    } else if (needSave) {
        me_apps.db.persist();
    }
    me_apps.teamMemberDialog.spielerToGUI(this);
};

/**
 * @function addOfficer
 * Add an officer to the list of players of a team. Does NOT check any game 
 * rules (so this function can't fail), but does update the quota counts if
 * neccesary.
 * @param {me_apps.obj.spieler} officer
 * @param {boolean} addToDatabase set this if the officer is a new object and
 * has to be added to the permanent database. Do not set this when moving team
 * members between officers, active players, and inactive players.
 */
me_apps.obj.mannschaft.prototype.addOfficer = function (officer, addToDatabase) {
    var needSave = false;
    if (me_apps.spiel.zeit > 0 && officer.aktiv && !officer.countedInOfficerQuota) {
        this.quotaOfficer--;
        officer.countedInOfficerQuota=true;
        var dbTeam;
        if (this.selector == 'heim') {
            dbTeam = me_apps.db.getCurrentGame().team_home;
        } else {
            dbTeam = me_apps.db.getCurrentGame().team_guest;
        }
        dbTeam.quota_officer = this.quotaOfficer;
        me_apps.db.tryUpdateSpielerInTeam(dbTeam, officer);
        needSave = true;
    }
    this.offizielle.push(officer);
    this.sortOfficers();
    if (addToDatabase) {
        me_apps.db.addManualTeamMember(this, officer);
        me_apps.protokollDialog.addProtocol(this, officer.bezeichner,
            "Einsatz",
            "Offizieller " + officer.bezeichner + " eingesetzt");
    } else if (needSave) {
        me_apps.db.persist();
    }
    me_apps.teamMemberDialog.officersToGUI(this);
};

/*
 * @function removeOffizieller
 * Removes an officer from a team. Does not check any game rules.
 * @param {me_apps.obj.offizieller} off
 * @returns keinen Rückgabewert
 */
me_apps.obj.mannschaft.prototype.removeOffizieller = function (off) {
    for (i = 0; i < this.offizielle.length; i++) {
        if (this.offizielle[i].bezeichner === off.bezeichner) {
            this.offizielle.splice(i, 1);
            me_apps.teamMemberDialog.officersToGUI(this);
            return;
        }
    }
};
/*
 * @function removeSpieler
 * Removes a player from a team. Does not check any game rules.
 * @param {me_apps.obj.spieler} plyr
 * @returns keinen Rückgabewert
 */
me_apps.obj.mannschaft.prototype.removeSpieler = function (plyr) {
    for (i = 0; i < this.spieler.length; i++) {
        if (this.spieler[i].id === plyr.id) {
            this.spieler.splice(i, 1);
            me_apps.teamMemberDialog.spielerToGUI(this);
            return;
        }
    }
};

/**
 * @function getSpieler
 * @param {int} id
 */
me_apps.obj.mannschaft.prototype.getSpieler = function (id) {
// test ob ein offizieller gemeint ist
    var result;
    if (isNaN(id)) {
        result = $.grep(this.offizielle, function (s, idx) {
            return s.bezeichner === id;
        });
        if (result.length > 0) {
            result = result[0];
        } else {
            result = null;
        }
    } else {
        result = $.grep(this.spieler, function (s, idx) {
            return s.bezeichner === id;
        });
        if (result.length > 0) {
            result = result[0];
        } else {
            result = null;
        }
    }
    return result;
};
/**
 * @function getSpielerById
 * @param {int} id
 */
me_apps.obj.mannschaft.prototype.getSpielerById = function (id) {
    var result;
    
    // Dies kann auch ein Offizieller sein, nicht nur Spieler.
    // Offizielle haben nicht notwendigerweise eine ID, deshalb sind
    // sie auch über ihren Bezeichner ansprechbar.

    id=""+id;
    result = $.grep(this.offizielle, function (s, idx) {
        return s.bezeichner === id;
    });
    if (result.length > 0)
        return result[0];

    result = $.grep(this.spieler, function (s, idx) {
        return s.id === id;
    });
    if (result.length > 0)
        return result[0];
    return null;
};
/**
 * @function getTeamSpielerById
 * @param {int} id
 */
me_apps.obj.mannschaft.prototype.getTeamSpielerById = function (id) {
    var result;
    id=""+id;
    result = $.grep(this.spieler, function (s, idx) {
        return s.id === id;
    });
    if (result.length > 0)
        return result[0];
    return null;
};

/**
 * @function getOffiziellerById
 * @param id
 */
me_apps.obj.mannschaft.prototype.getOffiziellerById = function (id) {
// test ob ein offizieller gemeint ist
    var result;
    id=""+id;
    result = $.grep(this.offizielle, function (s, idx) {
        return s.bezeichner === id;
    });
    if (result.length > 0)
        return result[0];
    return null;
};

/**
 * @function getSpielerWithTrikotNr
 * @param trikotNr
 */
me_apps.obj.mannschaft.prototype.getSpielerWithTrikotNr = function (trikotNr) {
    var result;
    if (trikotNr === 'A' || trikotNr === 'B' || trikotNr === 'C' || trikotNr === 'D') {
        result = $.grep(this.offizielle, function (s, idx) {
            return s.bezeichner === trikotNr && s.aktiv;
        });
        if (result.length > 0) {
            result = result[0];
        } else {
            result = undefined;
        }
    } else {
        result = $.grep(this.spieler, function (s, idx) {
            return s.bezeichner === trikotNr;
        });
        if (result.length > 0) {
            result = result[0];
        } else {
            result = undefined;
        }
    }
    return result;
};

/**
 * @function aktiveSpieler
 * @return {list} Liste aller aktiven Spieler
 */
me_apps.obj.mannschaft.prototype.aktiveSpieler = function () {
    var result = $.grep(this.spieler, function (s, idx) {
        return s.aktiv  && !isNaN(s.bezeichner);
    });
    return (result);
};

/**
 * @function inaktiveSpieler
 * @return {list} Liste aller inaktiven Spieler
 */
me_apps.obj.mannschaft.prototype.inaktiveSpieler = function () {
    var spieler = $.grep(this.spieler, function (s, idx) {
        return !s.aktiv;
    });
    return spieler;
};

/**
 * @function testIfSpielerExists
 * @param {int} trikotNr
 * @return {boolean} true falls ein spieler mit der trikotnummer in der Aufstellung schon vorhanden ist
 */
me_apps.obj.mannschaft.prototype.testIfSpielerExists = function (trikotNr) {
    var result = $.grep(this.spieler, function (s, idx) {
        return (s.bezeichner === trikotNr) && s.aktiv;
    });
    return (result.length > 0);
};

/**
 * @function aktiveOffizielle
 * @return {list} Liste aller aktiven Offiziellen
 */
me_apps.obj.mannschaft.prototype.aktiveOffizielle = function () {
    var result = $.grep(this.offizielle, function (s, idx) {
        return s.aktiv;
    });
    return (result);
};

me_apps.obj.mannschaft.prototype.calcQuota = function() {
    this.quotaOfficer = 4;
    for (var i=0; i<this.offizielle.length; i++) {
        if (this.offizielle[i].aktiv) {
            this.offizielle[i].countedInOfficerQuota=true;
            this.quotaOfficer--;
        } else {
            this.offizielle[i].countedInOfficerQuota=false;
        }
    }
    this.quotaPlayer=me_apps.spiel.anzahlSpieler;
    for (var i=0; i<this.spieler.length; i++) {
        if (this.spieler[i].aktiv) {
            this.spieler[i].countedInPlayerQuota=true;
            this.quotaPlayer--;
        } else {
            this.spieler[i].countedInPlayerQuota=false;
        }
    }
    me_apps.log.info("Setting quota to officer="+this.quotaOfficer+
                ", player="+this.quotaPlayer+" for "+this.selector);
        
    var game = me_apps.db.getCurrentGame();
    var team = (this.selector.charAt(0) === 'h' ? 'home' : 'guest');        // convert heim/gast to home/guest
    game["team_"+team]['quota_player'] = this.quotaPlayer;
    game["team_"+team]['quota_officer'] = this.quotaOfficer;
    me_apps.db.saveGame(game);
};

me_apps.obj.mannschaft.errcodes = {};
me_apps.obj.mannschaft.errcodes.OK =                 { code: 0, text: "OK", gui: "" };
me_apps.obj.mannschaft.errcodes.TOO_MANY_PLAYERS =   { code: 1, text: "Zu viele Spieler", gui: "#DialogListFull" };
me_apps.obj.mannschaft.errcodes.PLAYER_EXISTS =      { code: 2, text: "Trikotnummer schon vorhanden", gui: "#DialogDoppelterSpieler" };
me_apps.obj.mannschaft.errcodes.BAD_OFFICER_LETTER = { code: 3, text: "Falscher Offiziellenbezeichner", gui: "#DialogUngültigerOffizieller" };
me_apps.obj.mannschaft.errcodes.PLAYERNO_TOO_BIG =   { code: 4, text: "Trikotnummer zu hoch", gui: "#DialogFalscheSpielernummer" };
me_apps.obj.mannschaft.errcodes.PLAYERNO_INVALID =   { code: 5, text: "Trikotnummer ungültig"};
me_apps.obj.mannschaft.errcodes.QUOTA_EXHAUSTED =    { code: 6, text: "Kontingent ausgeschöpft", gui: "#DialogKontingentAusgeschöpft" };
me_apps.obj.mannschaft.errcodes.PASS_NUMBER_EMPTY =  { code: 7, text: "Leere Passnummer", gui: "" };
me_apps.obj.mannschaft.errcodes.PASS_NUMBER_INVAL =  { code: 8, text: "Passnummer nur Ziffern!", gui: "" };

/*
 * @function officerRoleChange
 * Changes an officer to a spieler. Also handles changing the letter of an
 * officer.
 * @param {me_apps.obj.spieler} teamMember
 * @param {integer} newTrikotNr
 * @returns {me_apps.obj.mannschaft.errcodes.*}
 */

me_apps.obj.mannschaft.prototype.officerRoleChange=
function(teamMember, newTrikotNr) {
    if (newTrikotNr === "" || isNaN(newTrikotNr)) {
        // Offizieller zu Offizieller
        if (newTrikotNr.length !== 1 || "ABCD".indexOf(newTrikotNr)===-1) {
            return me_apps.obj.mannschaft.errcodes.BAD_OFFICER_LETTER;
        }
    } else {
        // Offizieller zu Spieler
        if (newTrikotNr <= 0 || newTrikotNr > me_apps.gMaxTrikotnumber) {
            return me_apps.obj.mannschaft.errcodes.PLAYERNO_TOO_BIG;
        }
        if (this.aktiveSpieler().length >= me_apps.spiel.anzahlSpieler) {
            return me_apps.obj.mannschaft.errcodes.TOO_MANY_PLAYERS;
        }
        if (this.getSpielerWithTrikotNr(newTrikotNr)) {
            return me_apps.obj.mannschaft.errcodes.PLAYER_EXISTS;
        }
        if (me_apps.spiel.zeit > 0 &&  this.quotaPlayer < 1 && !teamMember.countedInPlayerQuota) {
            return me_apps.obj.mannschaft.errcodes.QUOTA_EXHAUSTED;
        }
        if (teamMember.passNr === "") {
            return me_apps.obj.mannschaft.errcodes.PASS_NUMBER_EMPTY;
        }
        if (!teamMember.passNr.match(/^\d+$/)) {
            return me_apps.obj.mannschaft.errcodes.PASS_NUMBER_INVAL;
        }
        
        if (teamMember.passNr !== "0000") {
            for (var i=0; i<this.spieler.length; i++) {
                if (teamMember.passNr === this.spieler[i].passNr) {
                    return {
                        // don't use one of me_apps.obj.mannschaft.errcodes because of special text
                        code: -1,
                        text: "Passnummer "+teamMember.passNr+" schon als "+this.spieler[i].vorname+" "+this.spieler[i].name+" vorhanden"
                    };
                    break;
                }
            }
        }
    }
    var oldLetter=teamMember.bezeichner;
    this.removeOffizieller(teamMember);
    teamMember.bezeichner=newTrikotNr;
    if (teamMember.isOfficer()) {
        this.addOfficer(teamMember);
        me_apps.protokollDialog.addProtocol(this, '-', 
        "Änderung",
        "Offizieller " + oldLetter + " zu " + newTrikotNr + " geändert");
    } else {
        teamMember.aktiv=true;
        teamMember.addSpielerPropertiesToOfficer();
        this.addSpieler(teamMember);
        me_apps.protokollDialog.addProtocol(this, teamMember.id, 
        "Änderung",
        "Offizieller " + oldLetter + " als Spieler " + newTrikotNr + " eingesetzt");
    }
    me_apps.db.updateSpieler(teamMember, this.selector, true);
    return me_apps.obj.mannschaft.errcodes.OK;
};

/**
 * @function spielerRoleChange
 * Changes the role of a spieler. Might make him an officer or change his id.
 * @param {me_apps.obj.spieler} teamMember
 * @param {type} newNumber
 * @returns {me_apps.obj.mannschaft.errcodes.*}
 */

me_apps.obj.mannschaft.prototype.spielerRoleChange =
function(teamMember, newNumber) {
    me_apps.log.logCall();
    me_apps.log.debug("spierlRoleChange newNumber='"+newNumber+"'");
    if (newNumber === "") {
        return me_apps.obj.mannschaft.errcodes.PLAYERNO_INVALID;
    } else if (isNaN(newNumber)) {
        // Spieler zu Offizieller
        if (newNumber.length !== 1 || "ABCD".indexOf(newNumber)===-1) {
            return me_apps.obj.mannschaft.errcodes.BAD_OFFICER_LETTER;
        }
        if (me_apps.spiel.zeit > 0 && this.quotaOfficer<1 && !teamMember.countedInOfficerQuota) {
            return me_apps.obj.mannschaft.errcodes.QUOTA_EXHAUSTED;
        }
    } else {
        if (newNumber <= 0) {
            return me_apps.obj.mannschaft.errcodes.PLAYERNO_INVALID;
        }
        if (newNumber > me_apps.gMaxTrikotnumber) {
            me_apps.log.debug("number is bigger than "+me_apps.gMaxTrikotnumber);
            return me_apps.obj.mannschaft.errcodes.PLAYERNO_TOO_BIG;
        }
        if (this.getSpielerWithTrikotNr(newNumber)) {
            me_apps.log.debug("new Number already exists");
            return me_apps.obj.mannschaft.errcodes.PLAYER_EXISTS;
        }
    }
    var oldLetter=teamMember.bezeichner;
    this.removeSpieler(teamMember);
    teamMember.bezeichner=newNumber;
    teamMember.aktiv=true;
    if (teamMember.isOfficer()) {
        this.addOfficer(teamMember);
        me_apps.protokollDialog.addProtocol(this, '-', 
        "Änderung",
        "Spieler " +oldLetter+" als Offizieller "+ newNumber + " eingesetzt");
    } else {
        this.addSpieler(teamMember);
        me_apps.protokollDialog.addProtocol(this, teamMember.id, 
        "Änderung", "Spieler " + oldLetter + " zu " + newNumber + " geändert");
    }
    me_apps.db.updateSpieler(teamMember, this.selector, true);
    return me_apps.obj.mannschaft.errcodes.OK;
};

/**
 * @function setMemberActive - activates a player from the inactive players list.
 * This function only activates players, it can't turn them into officers.
 * @param {int} spielerId - the (database internal) id of the player to change
 * @param {string} newTrikotNr - the new number (bezeichner) of that player
 * @returns {me_apps.obj.mannschaft.errcodes.*}
 */
me_apps.obj.mannschaft.prototype.setMemberActive = function (spielerId, newTrikotNr) {
    var spieler = this.getSpielerById(spielerId);
    
    if (newTrikotNr !== spieler.bezeichner) {
        if (this.testIfSpielerExists(newTrikotNr))
            return me_apps.obj.mannschaft.errcodes.PLAYER_EXISTS;
        if (me_apps.spiel.zeit > 0 && this.quotaPlayer<1 && !spieler.countedInPlayerQuota)
            return me_apps.obj.mannschaft.errcodes.QUOTA_EXHAUSTED;
        
        if (me_apps.spiel.zeit > 0) {
            if (!spieler.aktiv && !spieler.countedInPlayerQuota) {
                this.quotaPlayer--;
                spieler.countedInPlayerQuota = true;
                
                // meh - gbl 20200820
                var dbTeam;
                if (this.selector == 'heim') {
                    dbTeam = me_apps.db.getCurrentGame().team_home;
                } else {
                    dbTeam = me_apps.db.getCurrentGame().team_guest;
                }
                dbTeam.quota_player = this.quotaPlayer;         // will be saved by updateSpieler
                me_apps.db.tryUpdateSpielerInTeam(dbTeam, spieler);
            }
            var code, txt;
            if (spieler.bezeichner === "") {           
                code = "Einsatz";
                txt = "Spieler mit Nr. " + newTrikotNr + " eingesetzt";
            } else {
                code = "Änderung";
                txt = "Spieler " + spieler.bezeichner + " Nr. geändert auf " + newTrikotNr;
            }
            me_apps.protokollDialog.addProtocol(this, spielerId, code, txt);
        } else {
            me_apps.log.info(this.selector + " Spieler (id=" + spielerId + ") Nr. geändert von  " + spieler.bezeichner + " auf " + newTrikotNr);
        }
        spieler.bezeichner=newTrikotNr;
        this.sortSpieler();
    }
    spieler.aktiv=true;
    me_apps.teamMemberDialog.spielerToGUI(this);
    me_apps.teamMemberDialog.officersToGUI(this);
    me_apps.db.updateSpieler(spieler, this.selector, true);
    return me_apps.obj.mannschaft.errcodes.OK;
};

/**
 * @function setMemberInaktiv
 * @param {me_apps.obj.spieler} spieler
 */
me_apps.obj.mannschaft.prototype.setMemberInaktiv = function (spieler) {
    spieler.aktiv = false;
    if (spieler.isOfficer()) {
        if (typeof(spieler.passNr) === "undefined") {
            spieler.passNr = "";
        }
        this.removeOffizieller(spieler);
        spieler.bezeichner = '';    // must be done here or the letter bezeichner will trigger a NAN in sortSpieler() leading to #286
        this.addSpieler(spieler);
        me_apps.teamMemberDialog.officersToGUI(this);
    }
    spieler.bezeichner = '';
    me_apps.teamMemberDialog.spielerToGUI(this);
    me_apps.db.updateSpieler(spieler, this.selector, true);
};

me_apps.obj.mannschaft.spielerDialogHandler = function (event) {
// for ANDROID APP we must prevent underlying onclicks to be fired   
    //if (event.target.parentElement.className !== "PopupTime") {
    if (event.target.parentElement.className === "PopupSpieler" || 
        event.target.parentElement.parentElement.parentElement.className === "PopupSpieler") {
		event.preventDefault();
		event.stopPropagation();
    }
    var target;
    if (event.target instanceof HTMLDivElement) {
        target = event.target.children[0];
    } else {
        target = event.target;
    }
    if (target) {
        if (event.type === "click" || event.type === "tap") {
            if (target.attributes["data-handler"]) {
                eval(target.attributes["data-handler"].value);
            }
        } else {
            if (target.attributes["data-dblclick-handler"]) {
                eval(target.attributes["data-dblclick-handler"].value);
            }
        }
    }
};

me_apps.obj.mannschaft.selectRow = function (event) {
    $('#DialogOffiziellerAdd tr').removeClass('rowSelected');
    var row = (event.currentTarget instanceof HTMLTableRowElement) ? event.currentTarget : event.currentTarget.parentElement;
    if (event.target instanceof HTMLInputElement) {
        row.className = "rowSelected";
    } else {
        row.className = (row.className === "rowSelected") ? "" : "rowSelected";
    }
};

me_apps.obj.mannschaft.addSpielerRow = function () {
    var table = $("#DialogSpielerAddAdditional tbody");
    var row = "<tr>"
            + "<td><input type='number' step='1' min='1' max='99' style='width:4em' data-mini='true' placeholder='T-Nr.'></input></td>"
            + "<td><input type='text' data-mini='true' placeholder='Vorname'/></input></td>"
            + "<td><input type='text' data-mini='true' placeholder='Name'/></input></td>"
            + "<td><input type='text' data-mini='true' placeholder='TT.MM.JJJJ'/></td>"
            + "<td><input type='number' data-mini='true' placeholder='PassNr'/></td>"
            + "<td><label><input type='checkbox'/>DSR</label></td>"
            + "</tr>";
    table.append(row);
    table.trigger('create');
};

me_apps.obj.mannschaft.editSpieler = function (event) {
    me_apps.log.logCall();
    event.stopPropagation();
    event.preventDefault();
    var idOfTable = event.target.parentElement.parentElement.parentElement.parentElement.parentElement.id;
    var mannschaft = (idOfTable === "Spielerheim") ? me_apps.spiel.heim : me_apps.spiel.gast;
    var id = event.target.parentElement.getAttribute("data-id");
    var spieler=mannschaft.getSpielerById(id);
    if (spieler !== null) {
        me_apps.teamMemberDialog.showChangeSpielerDialog(mannschaft, spieler);
    } else {
        me_apps.teamMemberDialog.showAddSpielerDialog(mannschaft, spieler);
    }
};

me_apps.obj.mannschaft.editOffizieller = function (event) {
    me_apps.log.logCall();
    event.stopPropagation();
    event.preventDefault();
    var idOfTable = event.target.parentElement.parentElement.parentElement.parentElement.id;
    var mannschaft = (idOfTable === "Spielerheim") ? me_apps.spiel.heim : me_apps.spiel.gast;

    var bezeichner = "";
    if (event.target.parentElement.cells[0].childNodes.length > 0) {
        bezeichner = event.target.parentElement.cells[0].childNodes[0].data;
    }
    if (bezeichner !== 'A' && bezeichner !== 'B' && bezeichner !== 'C' && bezeichner !== 'D')
        return;
    var offizieller = mannschaft.getOffiziellerById(bezeichner);
    
    if (offizieller !== null) {
        me_apps.teamMemberDialog.showChangeOfficerDialog(mannschaft, offizieller);
    } else {
        me_apps.teamMemberDialog.showAddOfficerDialog(mannschaft, bezeichner);
    }
};

/**
 * @function toggleTeamNamenStrafen
 * @param {event} event - click event that triggered the toggle
 * Anzeige von Namen oder Strafen in der Anzeige des Sekretaers
 */
me_apps.obj.mannschaft.toggleTeamNamenStrafen = function (event) {
    event.stopPropagation();
    event.preventDefault();

    var target = event.target;
    var text   = target.innerHTML;
    text = (text === 'Name') ? 'Strafen' : 'Name';

    var team = target.parentElement.parentElement.parentElement.parentElement.parentElement.getAttribute("id");
    team = (team ==='Spielerheim') ? 'heim' : 'gast';

    me_apps.obj.mannschaft.changeAnzeigeNamenStrafen(team,text);
};

me_apps.obj.mannschaft.changeAnzeigeNamenStrafen = function (team, anzeige) {
    team = (team === 'undefined') ? 'heim' : team;

	if (anzeige === 'Name') {
		$("#Spieler"+ team).addClass('NamenAnzeigen');
		$("#Spieler"+ team + '>div:first-child >table>tbody>tr>td:nth-child(4)').show();
		$("#Spieler"+ team + '>div:first-child >table>thead>tr>th:nth-child(4)').show();
		$("#Spieler"+ team + '>table>tbody>tr>td:nth-child(4)').show();
		$("#Spieler"+ team + '>div:first-child >table>tbody>tr>td:nth-child(3)').hide();
		$("#Spieler"+ team + '>div:first-child >table>thead>tr>th:nth-child(3)').hide();
		$("#Spieler"+ team + '>table>tbody>tr>td:nth-child(3)').hide();
	} else {
		$("#Spieler"+ team).removeClass('NamenAnzeigen');
		$("#Spieler"+ team + '>div:first-child >table>tbody>tr>td:nth-child(3)').show();
		$("#Spieler"+ team + '>div:first-child >table>thead>tr>th:nth-child(3)').show();
		$("#Spieler"+ team + '>table>tbody>tr>td:nth-child(3)').show();
		$("#Spieler"+ team + '>div:first-child >table>tbody>tr>td:nth-child(4)').hide();
		$("#Spieler"+ team + '>div:first-child >table>thead>tr>th:nth-child(4)').hide();
		$("#Spieler"+ team + '>table>tbody>tr>td:nth-child(4)').hide();
	}
};

/**
 * This checks if a player has a penalty that ends exactly 4 minutes in the
 * future. We can't just look for the highest penalty time because the
 * game time might have been changed.
 * 
 * @param {spieler} player
 * @returns {boolean}
 */
me_apps.obj.mannschaft.prototype.hasPlayerMaxPenaltyStarting =
function(player) {
    var requiredEnd=me_apps.spiel.zeit + me_apps.spiel.dauerHinausstellung*2;
    var ary=this.hinausstellungen;
    for (var i=0; i<ary.length; i++) {
        if (ary[i].id === player.id && ary[i].ende === requiredEnd)
            return true;
    }
    return false;
};