if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}

if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.protokoll = function (mannschaft, id, actionCode, aktion, createID) {
    // Allow undefined if we're planning to call copyValuesFromPermStore
    if (mannschaft!==undefined)
        this.init(mannschaft, id, actionCode, aktion, createID);
};

me_apps.obj.protokoll.prototype.init =
function (mannschaft, id, actionCode, aktion, createID) {        
    var d = new Date();
    this.spielzeit = me_apps.spiel.timeToMMSS(me_apps.spiel.zeit);
    this.spielstand = me_apps.spiel.heim.getTore() + ":" + me_apps.spiel.gast.getTore();
    if (mannschaft === null) {
        this.hg = "-";
    } else {
        this.hg = (mannschaft.getGastAsInt() === 0) ? "H" : "G";
    }
    if (mannschaft === null || id === "-") {
        this.trikotNr = "-";
        this.spielerID = 0;
    } else {
        var spieler = mannschaft.getSpielerById(id);
        this.trikotNr = spieler.bezeichner;
        this.spielerID = id;
    }
    this.actionCode = actionCode;
    if (aktion) {
        this.aktion   = aktion;
    } else {
        this.aktion = me_apps.protokollEintraege[actionCode];
    }

    this.createID = createID;
    this.inactive = 0;
    this.zeit = d.getHours() + ":" + ("0" + d.getMinutes()).slice(-2) + ":" + ("0" + d.getSeconds()).slice(-2);
};

me_apps.obj.protokoll.prototype.copyValuesFromPermStore =
function(dbEntry) {
    this.spielzeit              = dbEntry.gametime;
    this.spielstand             = dbEntry.gamescore;
    this.hg                     = dbEntry.origin;
    this.trikotNr               = dbEntry.player_no;
    this.aktion                 = dbEntry.action;
    this.zeit                   = dbEntry.time;
    this.spielerID              = dbEntry.person_id;
    this.createID               = dbEntry.sequenceno;
    this.inactive               = dbEntry.inactive;
    this.actionCode             = dbEntry.actionCode;
};

me_apps.obj.protokoll.prototype.copyValuesToPermStore =
function(dbEntry) {
    dbEntry.gametime		=    this.spielzeit;
    dbEntry.gamescore		=    this.spielstand;
    dbEntry.origin          =    this.hg;
    dbEntry.player_no		=    this.trikotNr;
    dbEntry.action          =    this.aktion;
    dbEntry.time            =    this.zeit;
    dbEntry.person_id		=    this.spielerID;
    dbEntry.sequenceno      =    this.createID;
    dbEntry.inactive		=    this.inactive;
    dbEntry.actionCode      =    this.actionCode;
    dbEntry.uploadTime      =    0;
    dbEntry.creationTime    =    Date.now();
};

me_apps.obj.protokoll.prototype.update = 
function (aktion, zeit, newTrikotNr, newTeam, newActionCode, newaktion, score) {
    if (this.zeit !== zeit || this.aktion !== aktion) {
        // This emulates the original which said "where aktion = and zeit = "
        me_apps.log.warn("update called for zeit "+zeit+" and aktion "+aktion+" but original was "+this.zeit+" and "+this.aktion);
        return;
    }
    this.trikotNr           = newTrikotNr;
    this.hg                 = newTeam;
    if (newaktion!==undefined)
        this.aktion         = newaktion;
    if (score!==undefined && this.spielstand !== score) {
        me_apps.protokollDialog.addProtocol(null, '-', 'Spielstand', 
            'Änderung, '+this.spielzeit+' '+this.spielstand + ' -> '+ score);
        this.spielstand     = score;
    }
    if (newActionCode !== undefined) {
        this.actionCode = newActionCode;
    }
    // Das gefällt mir gar nicht. Geht aber nicht anders ohne jede Menge
    // umzubauen.
    if (newTrikotNr > 0) {
        var mannschaft = (newTeam === 'G' ? me_apps.spiel.gast : me_apps.spiel.heim);
        var spieler=mannschaft.getSpielerWithTrikotNr(newTrikotNr);
        if (!spieler) {
            this.spielerID="-1 ? spieler unknown in protokoll.update";
        } else {
            this.spielerID=spieler.id;
        }
    }
};

me_apps.obj.protokoll.prototype.delete =
function (aktion, zeit, spielstand) {
    if (this.zeit !== zeit || this.spielstand !== spielstand) {
        // This emulates the original which said "where aktion = and zeit = "
        me_apps.log.warn("delete called for zeit "+zeit+" and spielstand "+spielstand+" but original was "+this.zeit+" and "+this.spielstand);
        return;
    }
    this.inactive=true;
};

me_apps.obj.protokoll.prototype.toHTML =
function() {
    
    var spieler=undefined;

    if (this.hg !== '-') {
        var mannschaft  = (this.hg === "H") ? me_apps.spiel.heim : me_apps.spiel.gast;
        // See #895. Our entry can refer to an officer (always has a trikotNr
        // (A..D) but no ID), a player who has a trikotNr, or a player who does not.
        //  So we can't really use getSpielerById or getSpielerWithTrikotNr alone.
        if (this.trikotNr)
            spieler = mannschaft.getSpielerWithTrikotNr(this.trikotNr);
        else
            spieler = mannschaft.getSpielerById(this.spielerID);
    }

    html  = '<tr id="ProtokollRow' + this.createID + '" createid="' + this.createID + '"';
    if (this.inactive === true) {
            html += ' class="deleted"';
    }
    html += ">";
    html += "<td>" + this.spielzeit;
    html += "</td><td>" + this.spielstand;
    html += "</td><td>" + this.hg;
    if (spieler !== undefined && !spieler.isOfficer()) {
        html += '</td><td data-id="' + this.spielerID + '">' + this.trikotNr;
    } else {
        html += '</td><td data-id="' + this.trikotNr + '">' + this.trikotNr;
    }
    html += "</td><td actioncode='" + this.actionCode+"'>" + this.aktion;
    html += "</td><td>" + this.zeit;
    html += "</tr>";
    return html;
};
