/**
 * Methods in this file handle the GUI part of the protocol within the
 * main screen (index.html).
 */

if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}

if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.protokollDialog = function () {};

me_apps.obj.protokollDialog.prototype.toGUI = function() {
    var dbEntries=me_apps.db.getProtocolList();

    var protokollTable = $('#Protokoll tbody');
    protokollTable.empty();
    for (var i=0; i<dbEntries.length; i++) {
        var appEntry = new me_apps.obj.protokoll();
        appEntry.copyValuesFromPermStore(dbEntries[i]);
        var row=appEntry.toHTML();
        protokollTable.prepend(row);
        if (dbEntries[i].inactive) {
            $("#Protokoll tbody tr:first").unbind("taphold dblclick");
        } else {
            $("#Protokoll tbody tr:first").unbind("taphold dblclick").bind("taphold dblclick", function (event) {
                event.stopPropagation();
                event.preventDefault();
                me_apps.protokollDialog.openChangeProtocolDialog(event);
            });
        }
    }
};

me_apps.obj.protokollDialog.prototype.disqualificationTime =
function(spieler) {
    var dbEntries=me_apps.db.getProtocolList();
    for (var i=dbEntries.length-1; i>=0; i--) {
        if (dbEntries[i].person_id === spieler.id
        &&  dbEntries[i].action.substr(0, 1) === 'D') {     // can be Disqualifikation or Disqualifikation mit Bericht so just check for D
            return dbEntries[i].gametime.substr(0, 2)*60 + dbEntries[i].gametime.substr(3, 2)*1;
        }
    }
    return -1;
};

me_apps.obj.protokollDialog.prototype.getKuerzel = function (aktion) {
    var kuerzel = "";
    if (aktion.search(/^verwarnung/i) === 0) {
        kuerzel = "V";
    }
    if (aktion.search(/^disqualifikation/i) === 0) {
        kuerzel = "D";
    }
    if (aktion.search(/^disqualifikation mit bericht/i) === 0) {
        kuerzel = "B";
    }
    if (aktion.search(/^hinausstellung/i) === 0) {
        kuerzel = "H";
    }
    if (aktion.search(/^tor/i) === 0) {
        kuerzel = "T";
    }
    if (aktion.search(/^7m mit tor/i) === 0) {
        kuerzel = "T";
    }
    if (aktion.search(/^auszeit/i) === 0) {
        kuerzel = "AZ";
    }
    if (aktion.search(/^zusätzlich/i) === 0) {
        kuerzel = "Z";
    }    return kuerzel;
};

/**
 * addProtocol
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {Integer} id
 * @param {String} actionCode
 * @param {String} aktion entweder Tor, Verwarnung, ...
 */
me_apps.obj.protokollDialog.prototype.addProtocol = function (mannschaft, id, actionCode, aktion) {
    var protokollTable = $('#Protokoll tbody');
    var createID = protokollTable[0].children.length;
    var protokoll = new me_apps.obj.protokoll(mannschaft, id, actionCode, aktion, createID);
    protokollTable.prepend(protokoll.toHTML());
    $("#Protokoll tbody tr:first").unbind("taphold dblclick").bind("taphold dblclick", function (event) {
        event.stopPropagation();
        event.preventDefault();
        me_apps.protokollDialog.openChangeProtocolDialog(event);
    });
    me_apps.db.addProtokoll(protokoll);
};

me_apps.obj.protokollDialog.prototype.removeRow = function () {
    var row = $("#ProtokollTabelle tbody tr")[me_apps.protokollDialog.protokollIdx];
    
    var createID = row.getAttribute('createid');
    var rowID = row.id;
    var cells = row.children;
    var spielstand = cells[1].innerHTML;
    var team = cells[2].innerHTML;
    var nr = cells[3].innerHTML;
    var id = cells[3].getAttribute('data-id');
    var aktion = cells[4].innerHTML;
    var zeit = cells[5].innerHTML;
    var kuerzel = me_apps.protokollDialog.getKuerzel(aktion);
    switch (kuerzel) {
        case "T":
            this.invalidateTor(team, id);
            break;
        case "V":
        case "Z":
        case "D":
        case "B":
        case "H":
            me_apps.mainDialog.deleteStrafe(team, id, kuerzel);
            break;
        case "AZ":
            this.invalidateTimeout(team);
            if (this.protokollIdx === 0)
                me_apps.spiel.stopTimeout();
            break;
    }
    me_apps.db.deleteProtokollEntry(aktion, zeit, spielstand, createID);
    $("#DialogProtokollChange").popup('close');
    me_apps.log.important("Protokoll gelöscht: " + aktion + " nr=" + nr + " id=" + id +" zeit=" + zeit + " createID=" + createID);
    $("#"+rowID).addClass('deleted');
    $("#"+rowID).unbind("taphold dblclick");
};

me_apps.obj.protokollDialog.prototype.changeProtocolTeamMember = function (newTeamMemberId, mannschaftAsInt) {
    var row = $("#ProtokollTabelle tbody tr")[me_apps.protokollDialog.protokollIdx];
    var createID = row.getAttribute('createid');
    var cells = row.children;
    var score = cells[1].innerHTML;
    var arrScore = score.split(':');
    var team = cells[2].innerHTML;
    var nr = cells[3].innerHTML;
    var teamMemberId = cells[3].getAttribute('data-id');
    var aktion = cells[4].innerHTML;
    var kuerzel = cells[4].getAttribute("actioncode");
    var zeit = cells[5].innerHTML;
    var newTeam = (mannschaftAsInt === 0) ? "H" : "G";
    // .. saved directly now ... var kuerzel = me_apps.protokollDialog.getKuerzel(aktion);
    switch (kuerzel) {
        case "T":
        case "7MT":
            me_apps.protokollDialog.invalidateTor(team, teamMemberId);
            me_apps.protokollDialog.assignTor(newTeam, newTeamMemberId);
            if (team === newTeam)
                break;
            if (team === 'G') {
                arrScore[0]++;
                arrScore[1]--;
            } else {
                arrScore[0]--;
                arrScore[1]++;
            }
            break;
        case "V":
            me_apps.mainDialog.deleteStrafe(team, teamMemberId, kuerzel);
            me_apps.mainDialog.addStrafe(newTeam, newTeamMemberId, kuerzel);
            break;
        case "D":
        case "B":
        case "H":
        case "Z":
            var time = "0:00";
            var selector = '#ZweiMinuten' + ((cells[2].innerHTML === "H") ? "heim" : "gast") + ' tbody tr';
            var rows = $(selector);
            var size = rows.size();
            for (var i = 0; i < size; i++) {
                if (rows[i].children[0].innerHTML === nr) {
                    time = rows[i].children[1].innerHTML;
		}
            }
            me_apps.mainDialog.deleteStrafe(team, teamMemberId, kuerzel);
            me_apps.mainDialog.addStrafe(newTeam, newTeamMemberId, kuerzel, time);
            break;
    }
    var newScore = arrScore[0] + ":" + arrScore[1];
    $("#DialogProtokollChange").popup('close');
    var newNr = ((mannschaftAsInt === 0) ? me_apps.spiel.heim : me_apps.spiel.gast).getSpielerById(newTeamMemberId).bezeichner;
    me_apps.log.warn("Protokolländerung: " + aktion + " vorher: nr=" + nr + " id=" + teamMemberId + " team=" + team + " nachher: nr=" + newNr + " id=" + newTeamMemberId + " team=" + newTeam);
    cells[1].innerHTML = newScore;
    cells[2].innerHTML = newTeam;
    cells[3].innerHTML = newNr;
    cells[3].setAttribute('data-id', newTeamMemberId);
    me_apps.db.updateProtokollEntry(aktion, zeit, newNr, newTeam, undefined, undefined, newScore, createID);
};

me_apps.obj.protokollDialog.prototype.changeProtocolIsGoal = 
function(event) {
    event.stopPropagation();
    event.preventDefault();
    var thisclass = $(this).attr('class');
    var isgoal = thisclass.indexOf('isgoal') > -1;
    var row = $("#ProtokollTabelle tbody tr")[me_apps.protokollDialog.protokollIdx];
    var createID = row.getAttribute('createid');
    var cells = row.children;
    var score = cells[1].innerHTML;
    var arrScore = score.split(':');
    var team = cells[2].innerHTML;
    var nr = cells[3].innerHTML;
    var id = cells[3].getAttribute('data-id');
    var aktion = cells[4].innerHTML;
    var newaktion = cells[4].innerHTML;
    var zeit = cells[5].innerHTML;
    var newActionCode = "";

    if (isgoal && aktion === me_apps.protokollEintraege['7MOT']) {
        me_apps.protokollDialog.assignTor(team, id);
        newActionCode = '7MT';
        newaktion = me_apps.protokollEintraege['7MT'];
        if (team === 'H') {
            arrScore[0]++;
        } else {
            arrScore[1]++;
        }
    } else if (!isgoal && aktion === me_apps.protokollEintraege['7MT']) {
        me_apps.protokollDialog.invalidateTor(team, id);
        newActionCode = '7MOT';
        newaktion = me_apps.protokollEintraege['7MOT'];
        if (team === 'H') {
            arrScore[0]--;
        } else {
            arrScore[1]--;
        }
    }
    if (newActionCode !== "") {
        var newScore = arrScore[0] + ":" + arrScore[1];
        $("#DialogProtokollChange").popup('close');
        me_apps.log.important("Protkolländerung: 7m-Ergebnis  id=" + id + " team=" + team + "vorher: = aktion " + cells[4].innerHTML + " nachher: aktion=" + aktion);
        cells[1].innerHTML = newScore;
        cells[4].innerHTML = newaktion;
        cells[4].setAttribute('actioncode', newActionCode);
        me_apps.db.updateProtokollEntry(aktion, zeit, nr, team, newActionCode, newaktion, newScore, createID);
    }
};

me_apps.obj.protokollDialog.prototype.changeProtocolIsReport = 
function(event) {
    event.stopPropagation();
    event.preventDefault();
    var thisclass = $(this).attr('class');
    var isreport = thisclass.indexOf('isreport') > -1;
    var row = $("#ProtokollTabelle tbody tr")[me_apps.protokollDialog.protokollIdx];
    var createID = row.getAttribute('createid');
    var cells = row.children;
    var score = cells[1].innerHTML;
    var team = cells[2].innerHTML;
    var nr = cells[3].innerHTML;
    var id = cells[3].getAttribute('data-id');
    var aktion = cells[4].innerHTML;
    var newaktion = aktion;
    var oldActionCode = cells[4].getAttribute("actioncode");
    var zeit = cells[5].innerHTML;
    var mannschaft = (team === 'G' ? me_apps.spiel.gast : me_apps.spiel.heim);
    var spieler=mannschaft.getSpielerById(id);
    var newActionCode = "";

    if (isreport && oldActionCode === 'D') {
        newaktion = "Disqualifikation mit Bericht";
        newActionCode = "B";
        spieler.removeDiscipline(team, 'D');
        spieler.addDiscipline(team, 'B');
    } else if (!isreport && oldActionCode === 'B') {
        newaktion = "Disqualifikation";
        newActionCode = "D";
        spieler.removeDiscipline(team, 'B');
        spieler.addDiscipline(team, 'D');
    }
    $("#DialogProtokollChange").popup('close');
    if (newActionCode !== "") {
        me_apps.mainDialog.penaltiesToGUI(mannschaft, spieler);
        me_apps.log.important("Protkolländerung: Report id=" + id + " team=" + team + "vorher: = aktion " + cells[4].innerHTML + " nachher: aktion=" + aktion);
        cells[4].innerHTML = newaktion;
        cells[4].setAttribute('actioncode', newActionCode);
        me_apps.db.updateProtokollEntry(aktion, zeit, nr, team, newActionCode, newaktion, score, createID);
    }
};


/**
 * openChangeProtocolDialog
 * @param {event} event 
 */
me_apps.obj.protokollDialog.prototype.openChangeProtocolDialog = function (event) {
    // edit Protokoll
    var cells = event.currentTarget.children;
    me_apps.protokollDialog.protokollIdx = event.currentTarget.rowIndex - 1;
    var mannschaft = (cells[2].innerHTML === "H") ? me_apps.spiel.heim : me_apps.spiel.gast;
    var aktion = cells[4].innerHTML;
    var actionCode = cells[4].getAttribute("actioncode");

    var tabBarItems = $('#SelectTeam a');
    if (mannschaft.getGastAsInt() === 0) {
        tabBarItems.first().addClass('ui-btn-active');
        tabBarItems.last().removeClass('ui-btn-active');
    } else {
        tabBarItems.first().removeClass('ui-btn-active');
        tabBarItems.last().addClass('ui-btn-active');
    }

    var tabBarItemsGoal = $('#SelectGoal a');
    if (actionCode === '7MT') {
        tabBarItemsGoal.first().addClass('ui-btn-active');
        tabBarItemsGoal.last().removeClass('ui-btn-active');
    } else if (actionCode === '7MOT') {
        tabBarItemsGoal.first().removeClass('ui-btn-active');
        tabBarItemsGoal.last().addClass('ui-btn-active');
    }
    
    var tabBarItemsReport = $('#SelectReport a');
    if (actionCode === 'B') {
        tabBarItemsReport.first().addClass('ui-btn-active');
        tabBarItemsReport.last().removeClass('ui-btn-active');
    } else {
        tabBarItemsReport.first().removeClass('ui-btn-active');
        tabBarItemsReport.last().addClass('ui-btn-active');
    }
    $('#DialogProtokollChange > div > div > table > tbody > tr')[0].innerHTML = cells[0].parentNode.innerHTML;
    var html = "";
    $('#SelectReport').hide();
    $('#SelectGoal').hide();
    $('#SelectTeam').hide();
    $("#ButtonProtokollDelete").hide();
    if ((cells[2].innerHTML === "-")          ||
        (actionCode === 'Änderung') ||   
        (actionCode === 'Einsatz')
       ) {
        html = "Dieser Protokolleintrag kann nicht geändert werden";
    } else if (aktion.indexOf('Zusätzlich') > -1) {         // gbl 20190605 gibts den überhaupt noch?
        html = "Dieser Protokolleintrag kann nur gelöscht werden";
        $("#ButtonProtokollDelete").show();
    } else {
        $('#SelectTeam').show();
        $("#ButtonProtokollDelete").show();
        if (actionCode.startsWith("7M")) {
            $('#SelectGoal').show();
        } else if (actionCode === 'B' || actionCode === 'D') {
            $('#SelectReport').show();
        }
        if (actionCode !== "Auszeit") {
            html=me_apps.obj.protokollDialog.prototype.getDisciplineTargetsHTML(mannschaft, aktion.indexOf('Tor') === -1);
        }
    }
    $('#DialogProtokollChangeSpieler')[0].innerHTML = html;
    me_apps.spielerDialog.openPopup("#DialogProtokollChange");

    $("#ButtonProtokollDelete").unbind('click').bind('click', function () {
        event.stopPropagation();
        event.preventDefault();
        me_apps.protokollDialog.removeRow();
    });
    $("#SelectGoal a").unbind('click').bind('click', me_apps.obj.protokollDialog.prototype.changeProtocolIsGoal);
    $("#SelectReport a").unbind('click').bind('click', me_apps.obj.protokollDialog.prototype.changeProtocolIsReport);
    tabBarItems = $('#SelectTeam a');
    $('#SelectTeam a').unbind('vclick').bind('vclick', {aktion: aktion, cell: cells[2]}, function (event) {
        event.stopPropagation();
        var tabBarItems = $('#SelectTeam a');
        if (event.data.aktion === me_apps.protokollEintraege.Auszeit) {
	    var row = $("#ProtokollTabelle tbody tr")[me_apps.protokollDialog.protokollIdx];
	    var createID = row.getAttribute('createid')*1;
            if (!$(this).hasClass('ui-btn-active')) {
	        var newTeamAuszeit = '';
                if (tabBarItems.last().hasClass('ui-btn-active')) {
                    me_apps.spiel.addTimeout(me_apps.spiel.heim);
                    me_apps.spiel.removeTimeout(me_apps.spiel.gast);
                    newTeamAuszeit = "H";
                } else {
                    me_apps.spiel.addTimeout(me_apps.spiel.gast);
                    me_apps.spiel.removeTimeout(me_apps.spiel.heim);
                    newTeamAuszeit = "G";
                }
                event.data.cell.innerHTML = newTeamAuszeit;
                me_apps.mainDialog.createTimeoutButtons();

                newTeamAuszeit2  = (newTeamAuszeit === 'G') ? 'Gast' : 'Heim';	
                me_apps.log.important("Auszeit createID=" + createID + " auf " + newTeamAuszeit2 + " geändert");
                var score = cells[1].innerHTML;
                var team = cells[2].innerHTML;
                var nr = cells[3].innerHTML;
                var id = cells[3].getAttribute('data-id');
                var zeit = cells[5].innerHTML;
                me_apps.db.updateProtokollEntry(aktion, zeit, nr, team, undefined, undefined, score, createID);
            } 
            else {
                me_apps.log.warn("Auszeit createID=" + createID + " wurde nicht geändert");
            }
            $("#DialogProtokollChange").popup('close');
        } else {
            var html = "";
            if (tabBarItems.last().hasClass('ui-btn-active')) {
                tabBarItems.first().addClass('ui-btn-active');
                tabBarItems.last().removeClass('ui-btn-active');
                html=me_apps.obj.protokollDialog.prototype.getDisciplineTargetsHTML(me_apps.spiel.heim, aktion.indexOf('Tor') === -1);
            } else {
                tabBarItems.first().removeClass('ui-btn-active');
                tabBarItems.last().addClass('ui-btn-active');
                html=me_apps.obj.protokollDialog.prototype.getDisciplineTargetsHTML(me_apps.spiel.gast, aktion.indexOf('Tor') === -1);
            }
            $('#DialogProtokollChangeSpieler')[0].innerHTML = html;
        }
    });
};

/**
 * Produces a list of players (and officers) in the form of icons which can
 * be clicked to select the target of a diciplinatory action, or a goal.
 * @param {me_apps.obj.mannschaft} team the member should belong to 
 * @param {boolean} includeOfficers - if true, officers are returned as well
 * @returns {String} HTML String
 */
me_apps.obj.protokollDialog.prototype.getDisciplineTargetsHTML =
function(mannschaft, includeOfficers) {
    var html="";
    mannschaft.aktiveSpieler().forEach(function (spieler) {
        html += "<div class='PopupSpieler'><a data-role='button' data-inline='true' class='SpielerButton'"
                + "onclick=\"me_apps.protokollDialog.changeProtocolTeamMember('" + spieler.id + "'," + mannschaft.getGastAsInt() + ");\">" + spieler.bezeichner + "</a><div>" + spieler.vorname + "</div><div>" + spieler.name + "</div></div>";
    });
    if (includeOfficers) {
        mannschaft.aktiveOffizielle().forEach(function (offizieller) {
                html += "<div class='PopupSpieler'><a data-role='button' data-inline='true' class='SpielerButton'"
                + "onclick=\"me_apps.protokollDialog.changeProtocolTeamMember('" + offizieller.bezeichner + "'," + mannschaft.getGastAsInt() 
                + ");\">" + offizieller.bezeichner + "</a><div>" + offizieller.vorname + "</div><div>" + offizieller.name + "</div></div>";
        });
    }
    return html;
};

/**
 * assignTor
 * @param {String} team
 * @param {Integer} id
 */
me_apps.obj.protokollDialog.prototype.assignTor = 
function (team, id) {
    var mannschaft = (team === "G") ? me_apps.spiel.gast : me_apps.spiel.heim;
    var spieler = mannschaft.getSpielerById(id);
    me_apps.spiel.goalScored(mannschaft, spieler);
    me_apps.mainDialog.mannschaftGoalCountToGUI(mannschaft);
    me_apps.mainDialog.spielerGoalCountToGUI(mannschaft, spieler);
};

/**
 * invalidateTor
 * @param {String} team
 * @param {Integer} id
 */
me_apps.obj.protokollDialog.prototype.invalidateTor = 
function (team, id) {
    var mannschaft = (team === "G") ? me_apps.spiel.gast : me_apps.spiel.heim;
    var spieler = mannschaft.getSpielerById(id);
    me_apps.spiel.goalInvalidated(mannschaft, spieler);
    me_apps.mainDialog.mannschaftGoalCountToGUI(mannschaft);
    me_apps.mainDialog.spielerGoalCountToGUI(mannschaft, spieler);
};

/**
 * invalidateTimeout
 * @param {String} team
  */
me_apps.obj.protokollDialog.prototype.invalidateTimeout = 
function (team) {
    var mannschaft = (team === "G") ? me_apps.spiel.gast : me_apps.spiel.heim;
    me_apps.spiel.removeTimeout(mannschaft);
};
