if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}
if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.spiel = function (spielId, nr) {
    this.nr = (nr === undefined) ? me_apps.DEFAULT_SPIELNUMMER : nr;
    this.spielId = (spielId === undefined) ? me_apps.DEFAULT_SPIELID : spielId;
    this.initGUI();
};

// Please keep these values in the order defined in the spreadsheet.
// Remember to update copyValuesFromPermStore and copyValuesToPermStore as well.
me_apps.obj.spiel.prototype.setDefaultValues =
function() {
    this.spielId            = 0;
    this.nr                 = 0;    
    
    this.heimId             = 0;
    this.heim               = new me_apps.obj.mannschaft("Heim", "heim");
    this.home_sname         = "Heim";
    this.heim_status        ="";

    this.gastId             = 0;
    this.gast               = new me_apps.obj.mannschaft("Gast", "gast");
    this.guest_sname        = "Gast";
    this.gast_status        = "";

    this.datum              = "01.01.2013";
    this.startZeit          = "16:00";
    this.geschlecht         = "g";
    this.verband            = "--";
    this.staffelId          = 0;
    this.staffel            = "--";
    this.staffelKurz        = "--";
    this.hallenName         = "--";
    this.hallenStadt        = "--";
    this.harzregel          = "keine Angabe";

    this.pinCheckHeim       = null;
    this.pinCheckGast       = null;

    this.encryptedHeim      = false;
    this.encryptedGast      = false;

    this.dauerHalbzeit      = 30 * 60;
    this.extDauerHalbzeit   = 0;
    this.dauerTimeout       = 50;
    this.dauerHinausstellung= 120;
    this.anzahlTimeOut      = 2;
    this.anzahlSpieler      = 14;
    this.pokal              = false;

    this.buzzerNachTimeout  = false;
    this.buzzerAmEnde       = false;
    
    this.halbzeitStände     = [];
    this.finalized          = false;

    this.zeit               = 0;                // Time the game has been running, in seconds
    this.timeout            = 0;                // Time spent in the current timeout
    this.zeitLäuft          = false;            // Flag if the game is running, or before start/in timeout/half time/ended
    this.timeoutLäuft       = false;            // Flag if there is a timeout currently running
    this.passCheckLock      = "";               // password for player license check; empty string if unlocked
    
    this.blockGameTimeStartSeconds = 0

    me_apps.util.appstateToNavigationGUI();
};

me_apps.obj.spiel.prototype.copyValuesFromPermStore =
function(dbEntry) {

    this.spielId        = dbEntry.id;
    this.nr             = dbEntry.no;

    this.heimId         = dbEntry.home_id;
    this.heim           = new me_apps.obj.mannschaft("Heim", "heim");
    if (dbEntry.team_home) {
        this.heim.copyValuesFromPermStore(dbEntry.team_home);
        this.home_sname     = this.heim.name;
        this.heim_status    = this.heim.status;
    } else {
        this.home_sname     = dbEntry.home_sname;
        this.heim_status    = '';
    }

    this.gastId         = dbEntry.guest_id;
    this.gast           = new me_apps.obj.mannschaft("Gast", "gast");
    if (dbEntry.team_guest) {
        this.gast.copyValuesFromPermStore(dbEntry.team_guest);
        this.guest_sname    = this.gast.name;
        this.gast_status    = this.gast.status;
    } else {
        this.guest_sname    = dbEntry.guest_sname;
        this.gast_status   = '';
    }

    this.datum              = me_apps.util.datetimeToDDMMYYYY(dbEntry.datetime);
    this.startZeit          = me_apps.util.datetimeToHHMM(dbEntry.datetime);
    this.geschlecht         = dbEntry.sex;
    this.verband            = dbEntry.org_lname;
    this.staffelId          = dbEntry.class_id;                 // fehlt
    this.staffel            = dbEntry.class_lname;
    this.staffelKurz        = dbEntry.class_sname;
    this.hallenName         = dbEntry.gym_name;
    this.hallenStadt        = dbEntry.gym_town;
    this.harzregel          = dbEntry.wax_regulations;

    this.pinCheckHeim       = dbEntry.team_home  ? dbEntry.team_home.pin_check  : null;
    this.pinCheckGast       = dbEntry.team_guest ? dbEntry.team_guest.pin_check : null;

    this.encryptedHeim      = dbEntry.team_home  ? dbEntry.team_home.encrypted  !== false : false;      // !==false sets undefined/null to true
    this.encryptedGast      = dbEntry.team_guest ? dbEntry.team_guest.encrypted !== false : false;
    
    this.dauerHalbzeit      = dbEntry.playing_time_secs*1;
    this.extDauerHalbzeit   = dbEntry.ext_playing_time_secs*1;
    this.dauerTimeout       = (dbEntry.timeout_duration_secs || 50);
    this.dauerHinausstellung= (dbEntry.penalty_time_secs || 120);
    this.anzahlTimeOut      = dbEntry.num_team_timeouts * 1;
    this.anzahlSpieler      = (dbEntry.max_players || 14);
    this.pokal              = dbEntry.is_cup;

    this.buzzerNachTimeout  = false;
    this.buzzerAmEnde       = false;
    
    this.halbzeitStände     = dbEntry.halftimes;
    if (!this.halbzeitStände)       // new field so handle not getting it
        this.halbzeitStände=[];
    this.finalized          = dbEntry.finalized;

    this.zeit               = (dbEntry.game_time || 0);
    this.timeout            = 0;
    this.zeitLäuft          = false;
    this.timeoutLäuft       = false;
    this.passCheckLock      = dbEntry.pass_check_lock || "";
    
    // These are separate objects in the app, but properties of the game withing
    // the database

    me_apps.spielbericht = new me_apps.obj.spielbericht(this.id);
    me_apps.util.appstateToNavigationGUI();
    if (me_apps.mainDialog) {
        me_apps.mainDialog.setTimeoutButtonsEnabled(this.zeit > 0);
    }
};

// There is no copyValuesToPermStore function, as changing stuff should always
// run a method in db as well, which persists the change and uploads it
// to the server.

me_apps.obj.spiel.prototype.copyValuesToPermStore =
function() {
    me_apps.log.error("do not use this!");
};

me_apps.obj.spiel.prototype.onlineStateToGUI = function(state, marker) {
    var elem=$('#Connected');
    elem.html(marker);
    elem.css('color', state.color);
};

me_apps.obj.spiel.prototype.uploadEnabled = function () {
    return me_apps.onlineState.isUploadEnabled();
};

me_apps.obj.spiel.prototype.setUploadEnabled = function (enabled) {
    return me_apps.onlineState.setUploadEnabled(enabled);
};

me_apps.obj.spiel.prototype.upload = function () {
    me_apps.obj.db.prototype.scheduleUploadGameData(me_apps.obj.db.prototype.getCurrentGame());
    me_apps.obj.db.prototype.scheduleUploadTicker(me_apps.obj.db.prototype.getCurrentGame());
};

me_apps.obj.spiel.prototype.checkOnlineState = function() {
    if (navigator.onLine) {
        me_apps.db.checkServerConnection();
    } else {
        this.setUploadEnabled(false);
    }
};

me_apps.obj.startTimer = function () {
    me_apps.log.info("startTimer called");
    var start = Math.round(new Date().getTime() / 1000);

    function checkIfSecondChanged() {
        var seconds = Math.round(new Date().getTime() / 1000) - start;
        if (seconds >= 1) {
            start = Math.round(new Date().getTime() / 1000);
            if (me_apps.spiel && me_apps.spiel.zeit <= me_apps.spiel.dauerHalbzeit * 2 + 4 * me_apps.spiel.extDauerHalbzeit) {
                me_apps.spiel.tick(seconds);
            }
            window.setTimeout(checkIfSecondChanged, 950);
        } else {
            window.setTimeout(checkIfSecondChanged, 25);
        }
    }
    //get the timer started!
    window.setTimeout(checkIfSecondChanged, 950);
};

me_apps.obj.spiel.prototype.tick = function (seconds) {
    if (me_apps.spiel.blockGameTimeStartSeconds > 0) {
        me_apps.spiel.blockGameTimeStartSeconds --;
        if (me_apps.spiel.blockGameTimeStartSeconds <= 0) {
            me_apps.mainDialog.gameTimeRunningToGUI(me_apps.spiel.zeitLäuft, false);
        }
    }
    if (me_apps.spiel.zeitLäuft) {
        for (var i = 0; i < seconds; i++) {
            me_apps.spiel.zeit++;

            if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit                                                  //Ende 1.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2                                       //Ende 2.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit      //Ende 3.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 2  //Ende 4.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 3  //Ende 5.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 4) //Ende 6.Halbzeit
            {
                if (me_apps.spiel.buzzerAmEnde) {
                    document.getElementById('Buzzer').cloneNode(true).play();
                }
                if (me_apps.spiel.zeitLäuft) {
                    me_apps.spiel.blockGameTimeStartSeconds = 5;
                    me_apps.spiel.setZeitLäuft(false);
                }
                me_apps.protokollDialog.addProtocol(null, "-", "Stop");
                me_apps.mainDialog.timeoutAvailabilityToGUI();
                var hz_state = me_apps.spiel.periodenStand();

                $("#Ergebnisse").show();
                $('#hz_stand').text(hz_state);
                me_apps.db.scheduleUploadGameData(me_apps.db.getCurrentGame(), 0);
            }
            
            if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit                                                  //Ende 1.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit      //Ende 3.Halbzeit
                    || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 3  //Ende 5.Halbzeit
            ) {
                // #88: Am Ende der Halbzeit einen Timeout simulieren, damit
                // eine Pausenzeit läuft.
                me_apps.spiel.timeoutLäuft=true;
                me_apps.spiel.timeout = 0;
            }
            
            if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit*2-300) {
                me_apps.mainDialog.timeoutAvailabilityToGUI();
            }    
        }
        
        me_apps.mainDialog.gameTimeToGUI(me_apps.spiel.zeit);
        me_apps.db.setGameTime(me_apps.spiel.zeit);
        me_apps.mainDialog.suspensionsToGUI(me_apps.spiel.heim);
        me_apps.mainDialog.suspensionsToGUI(me_apps.spiel.gast);
    }

    if (me_apps.spiel.timeoutLäuft) {
        for (i = 0; i < seconds; i++) {
            me_apps.spiel.timeout++;
            me_apps.mainDialog.timeoutTimeToGUI(me_apps.spiel.timeout);
        }
    }
};

me_apps.obj.spiel.prototype.setZeitLäuft = function (flag) {
    me_apps.spiel.zeitLäuft = flag;
    me_apps.util.appstateToNavigationGUI();
    me_apps.mainDialog.gameTimeRunningToGUI(me_apps.spiel.zeitLäuft, me_apps.spiel.blockGameTimeStartSeconds > 0);
};

me_apps.obj.spiel.prototype.periodenStand = function () {
    var hz_state = '';
    if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit)
        hz_state = '1. Halbzeit';
    else if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2)
        hz_state = '2. Halbzeit';
    else if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit)
        hz_state = '1. Verl.';
    else if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 2)
        hz_state = '2. Verl.';
    else if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 3)
        hz_state = '3. Verl.';
    else if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 4)
        hz_state = '4. Verl.';
    return hz_state;

};

me_apps.obj.spiel.prototype.changeEinstellungen = function (event) {
    me_apps.log.debug("changeEinstellungen: "+event.target.id);
    switch (event.target.id) {
        case("HalbzeitMinuten") :
            var s = me_apps.spiel.dauerHalbzeit % 60;
            var m = parseInt(event.target.value);
            me_apps.spiel.dauerHalbzeit = m * 60 + s;
            me_apps.db.setGameSetting('playing_time_secs', me_apps.spiel.dauerHalbzeit);
            break;
        case("HalbzeitSekunden") :
            var s = parseInt(event.target.value);
            var m = ~~(me_apps.spiel.dauerHalbzeit / 60);
            me_apps.spiel.dauerHalbzeit = m * 60 + s;
            me_apps.db.setGameSetting('playing_time_secs', me_apps.spiel.dauerHalbzeit);
            break;
        case("Verlaengerung") :
            me_apps.spiel.pokal = event.target.checked;
            me_apps.db.setGameSetting('is_cup', me_apps.spiel.pokal);
            break;
        case("HalbzeitMinutenExtended") :
            var s = me_apps.spiel.dauerHalbzeit % 60;
            var m = parseInt(event.target.value);
            me_apps.spiel.extDauerHalbzeit = m * 60 + s;
            me_apps.db.setGameSetting('playing_time_ext_secs', me_apps.spiel.extdauerHalbzeit);
            break;
        case("HalbzeitSekundenExtended") :
            var s = parseInt(event.target.value);
            var m = ~~(me_apps.spiel.extDauerHalbzeit / 60);
            me_apps.spiel.extDauerHalbzeit = m * 60 + s;
            me_apps.db.setGameSetting('playing_time_ext_secs', me_apps.spiel.extdauerHalbzeit);
            break;
        case("HinausstellungMinuten") :
            var s = me_apps.spiel.dauerHinausstellung % 60;
            var m = parseInt(event.target.value);
            me_apps.spiel.dauerHinausstellung = m * 60 + s;
            me_apps.db.setGameSetting('penalty_time_secs', me_apps.spiel.dauerHinausstellung);
            break;
        case("HinausstellungSekunden") :
            var s = parseInt(event.target.value);
            var m = ~~(me_apps.spiel.dauerHinausstellung / 60);
            me_apps.spiel.dauerHinausstellung = m * 60 + s;
            me_apps.db.setGameSetting('penalty_time_secs', me_apps.spiel.dauerHinausstellung);
            break;
        case ("NoOfTimeouts") :
            me_apps.spiel.anzahlTimeOut = parseInt(event.target.value);
            me_apps.db.setGameSetting('num_team_timeouts', me_apps.spiel.anzahlTimeOut);
            break;
        case("AnzahlSpieler") :
            var invalid = false;
            me_apps.spiel.anzahlSpieler = parseInt(event.target.value);
            if (me_apps.spiel.anzahlSpieler < 7) { invalid = true; me_apps.spiel.anzahlSpieler = 7; }
            if (me_apps.spiel.anzahlSpieler > 20) { invalid = true; me_apps.spiel.anzahlSpieler = 20; }
            if (invalid) {
                event.target.value=me_apps.spiel.anzahlSpieler;
            }
            me_apps.db.setGameSetting('max_players', me_apps.spiel.anzahlSpieler);
            break;
        case ("BuzzerTimeout") :
            me_apps.spiel.buzzerNachTimeout = event.target.checked;
            // not saved in game settings
            break;
        case ("BuzzerEnd") :
            me_apps.spiel.buzzerAmEnde = event.target.checked;
            // not saved in game settings
            break;
        case("TimeOutMinuten") :
            var s = me_apps.spiel.dauerTimeout % 60;
            var m = parseInt(event.target.value);
            me_apps.spiel.dauerTimeout = m * 60 + s;
            me_apps.db.setGameSetting('timeout_duration_secs', me_apps.spiel.dauerTimeout);
            break;
        case("TimeOutSekunden") :
            var s = parseInt(event.target.value);
            var m = ~~(me_apps.spiel.dauerTimeout / 60);
            me_apps.spiel.dauerTimeout = m * 60 + s;
            me_apps.db.setGameSetting('timeout_duration_secs', me_apps.spiel.dauerTimeout);
            break;
    }
};

me_apps.obj.spiel.prototype.timeToMMSS = function (time) {
    return ("0" + Math.floor(time / 60)).slice(-2) + ":" + ("0" + time % 60).slice(-2);
};

me_apps.obj.spiel.prototype.toggleZeitLäuft = function () {
    if (!me_apps.spiel.zeitLäuft &&
            (   (me_apps.spiel.blockGameTimeStartSeconds > 0) ||
                (me_apps.spiel.zeit >= me_apps.spiel.dauerHalbzeit * 2 && !me_apps.spiel.pokal) ||
                (me_apps.spiel.zeit >= me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 4 && me_apps.spiel.pokal)
            )
        ) {
        return;
    }
    if (me_apps.spiel.zeit === 0) {
        // Aus Performancegründen diese Prüfung nur beim Spielstart machen aber
        // nicht bei Unterbrechungen
        var aktiveHeim = me_apps.spiel.heim.aktiveSpieler();
        var aktiveGast = me_apps.spiel.gast.aktiveSpieler();
        var countActiveHeim = aktiveHeim.length;
        var countActiveGast = aktiveGast.length;
        if (countActiveHeim > me_apps.spiel.anzahlSpieler || countActiveGast > me_apps.spiel.anzahlSpieler) {
            $("#DialogPlayerCount").popup("open");
            return;
        }
        var hasEmpty = 0;
        var hasDouble = 0;
        for (var i=0; i<countActiveHeim; i++) {
            if (aktiveHeim[i].bezeichner === "")
                hasEmpty++;
            else {
                for (var j=0; j<countActiveHeim; j++) {
                    if (i !== j && aktiveHeim[i].bezeichner === aktiveHeim[j].bezeichner) {
                        hasDouble++;
                        break;
                    }
                }
            }
        }
        for (var i=0; i<countActiveGast; i++) {
            if (aktiveGast[i].bezeichner === "")
                hasEmpty++;
            else {
                for (var j=0; j<countActiveGast; j++) {
                    if (i !== j && aktiveGast[i].bezeichner === aktiveGast[j].bezeichner) {
                        hasDouble++;
                        break;
                    }
                }
            }
        }
        if (hasEmpty>0 || hasDouble>0) {
            $("#DialogPlayerNoNumberCount").text(hasEmpty);
            $("#DialogPlayerDblNumberCount").text(hasDouble);
            $("#DialogPlayerNoNumber").popup("open");
            return;
        }
        me_apps.spiel.heim.calcQuota();
        me_apps.spiel.gast.calcQuota();
    }

    me_apps.spiel.setZeitLäuft(!me_apps.spiel.zeitLäuft);
    if (me_apps.spiel.zeitLäuft) {
        me_apps.log.important("Uhr gestartet: " + me_apps.spiel.timeToMMSS(me_apps.spiel.zeit));
        me_apps.protokollDialog.addProtocol(null, "-", "Start");

        me_apps.obj.mannschaft.changeAnzeigeNamenStrafen('heim', 'Strafen');
       	me_apps.obj.mannschaft.changeAnzeigeNamenStrafen('gast', 'Strafen');
        me_apps.mainDialog.setTimeoutButtonsEnabled(true);

        if (me_apps.spiel.zeit === 0) {
            me_apps.spiel.upload();
        }
        if (me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit                                                          //Beginn 2. Halbzeit
                || me_apps.spiel.zeit === 0                                                                             //Beginn 1. Halbzeit
                || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit              //Beginn 4. Halbzeit
                || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 2          //Beginn 5. Halbzeit
                || me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2 + me_apps.spiel.extDauerHalbzeit * 3          //Beginn 6. Halbzeit
                || (me_apps.spiel.extDauerHalbzeit > 0 && me_apps.spiel.zeit === me_apps.spiel.dauerHalbzeit * 2))      //Beginn 3. Halbzeit
        {
		if ((me_apps.spiel.zeit !== 0) && ($('#Ergebnisse').is(':visible'))) {
			me_apps.spiel.protokollPeriod();
		}
                me_apps.spiel.upload(false);
        }
        me_apps.mainDialog.disableTimeouts();
        this.stopTimeout();
        $("#Ergebnisse").hide();
    } else {
        me_apps.log.important("Uhr angehalten: " + me_apps.spiel.timeToMMSS(me_apps.spiel.zeit));
        me_apps.protokollDialog.addProtocol(null, "-", "Stop");
        me_apps.mainDialog.enableTimeouts();
    }
};

me_apps.obj.spiel.prototype.protokollPeriod = function () {

	me_apps.protokollDialog.addProtocol(null, '-', 'Halbzeit', 'Spielstand '+me_apps.spiel.periodenStand());
	setTimeout(function() {
            $('#Ergebnisse').hide();
        }, 600);
        var gamescore = me_apps.spiel.heim.getTore() + ":" + me_apps.spiel.gast.getTore();
        this.halbzeitStände.push(gamescore);
        me_apps.db.setGameSetting('halftimes', this.halbzeitStände);
};

me_apps.obj.spiel.prototype.getFirstAvailableTimeout = function(mannschaft) {
    if (me_apps.spiel.zeit < me_apps.spiel.dauerHalbzeit) {
        if (mannschaft.timeoutsTaken[0] === false)
            return 0;
        if (mannschaft.timeoutsTaken[1] === false && me_apps.spiel.anzahlTimeOut === 3)
            return 1;
    } else {
        if (mannschaft.timeoutsTaken[1] === false)
            return 1;
        if (mannschaft.timeoutsTaken[2] === false && me_apps.spiel.anzahlTimeOut === 3 && (
                me_apps.spiel.zeit < me_apps.spiel.dauerHalbzeit*2-300              // vor 5 minuten vor Schlüss darf 3. genommen werden
                || mannschaft.timeoutsTaken[1] < me_apps.spiel.dauerHalbzeit*2-300   // danach auch wenn der 2. früh genug war
        ))
            return 2;
    }
    return -1;
};

/**
 * Add a timeout to a team without changing the game timer/timeout timer.
 * Used within protocol corrections.
 * @param {me_apps.obj.mannschaft} mannschaft
 * @returns {Boolean} true if adding a timeout was possible, false if not
 */
me_apps.obj.spiel.prototype.addTimeout = function(mannschaft) {
    var index=this.getFirstAvailableTimeout(mannschaft);
    if (index>=0) {
        mannschaft.timeoutsTaken[index]=me_apps.spiel.zeit;
        me_apps.db.updateTeam(mannschaft);
        me_apps.mainDialog.timeoutAvailabilityToGUI(mannschaft);
    }
    return index>=0;
};

/**
 * Remove a timeout from a team. Used from the game protocol when correcting.
 * @param {type} mannschaft
 * @returns {Boolean} true if a timeout was removed, false if the team didn't
 * have any
 */
me_apps.obj.spiel.prototype.removeTimeout = function (mannschaft) {
    for (var index=me_apps.spiel.anzahlTimeOut-1; index>=0; index--) {
        if (mannschaft.timeoutsTaken[index]) {
            mannschaft.timeoutsTaken[index]=false;
            me_apps.db.updateTeam(mannschaft);
            me_apps.mainDialog.timeoutAvailabilityToGUI(mannschaft);
            return true;
        }
    }
    return false;
};

/**
 * Starts a timeout, adds it to the team data, and stops the game time / starts
 * the timeout timer. Checks if the team had a timeout available.
 * @param {me_apps.obj.mannschaft} mannschaft
 * @returns {boolean} true if the team had a timeout available. If the return
 * value is false, then the timeout wasn't started.
 */
me_apps.obj.spiel.prototype.startTimeout = function (mannschaft) {
    if (this.addTimeout(mannschaft)) {
        me_apps.spiel.setZeitLäuft(false);
        me_apps.spiel.timeoutLäuft = true;
        me_apps.spiel.timeout = 0;
        return true;
    }
    return false;
};

me_apps.obj.spiel.prototype.stopTimeout = function() {
    me_apps.spiel.timeoutLäuft = false;
    me_apps.spiel.timeout = 0;
    me_apps.mainDialog.timeoutTimeToGUI(me_apps.spiel.timeout);
};

me_apps.obj.spiel.prototype.setTime = function () {
    var minutes = parseInt($('#SpielzeitMinuten')[0].value);
    var seconds = parseInt($('#SpielzeitSekunden')[0].value);
    if (!isNaN(minutes) && !isNaN(seconds)) {
        me_apps.spiel.zeit = 60 * minutes + seconds;
        me_apps.log.important("Zeit geändert auf " + me_apps.spiel.timeToMMSS(me_apps.spiel.zeit));
    }
    me_apps.mainDialog.gameTimeToGUI(me_apps.spiel.zeit);
};

me_apps.obj.spiel.prototype.addTime = function (diff) {
    var tempNewTime = me_apps.spiel.zeit + diff;
    if (me_apps.util.halfFromTime(me_apps.spiel.zeit) !== me_apps.util.halfFromTime(tempNewTime)) {
        return;
    }
    me_apps.spiel.zeit += diff;
    me_apps.mainDialog.gameTimeToGUI(me_apps.spiel.zeit);
    me_apps.log.important("Zeit geändert auf " + me_apps.spiel.timeToMMSS(me_apps.spiel.zeit));
};

me_apps.obj.spiel.prototype.disableAll = function () {

};

/**
 * @function changed
 * @param {type} event
 */
me_apps.obj.spiel.prototype.changed = function (event) {
    var prop = event.currentTarget.attributes['id'].nodeValue.substring(3);
    me_apps.spielbericht[prop] = event.currentTarget.value;
    me_apps.db.saveSpielbericht(me_apps.spielbericht);
};

me_apps.obj.spiel.prototype.initGUI = function () {
    $('#ZweiMinutenheim  tbody')[0].innerHTML = "";
    $('#ZweiMinutengast  tbody')[0].innerHTML = "";
    $('#Protokoll tbody')[0].innerHTML = "";
    $("#Ergebnisse").hide();

    $(window).bind("beforeunload", function () {
//        me_apps.db.updateSpiel(false);
//        me_apps.db.saveSpielbericht();
        return "Ihr Spiel wird gespeichert";
    });

    $(window).bind("unload", function () {
        me_apps.log.important("Browserfenster wurde geschlossen");
    });
};

/*
 * Functions that handle events within the game
 *  
 */

/**
 * Called when a player scores a goal. This only needs to update spieler,
 * because the number of goals per team is always calculated as the sum
 * of the individual players, not stored within mannschaft.
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {me_apps.obj.spieler} spieler
 * @return {undefined}
 */
me_apps.obj.spiel.prototype.goalScored = function(mannschaft, spieler) {
    spieler.addGoal(mannschaft.selector);
};

me_apps.obj.spiel.prototype.goalInvalidated = function(mannschaft, spieler) {
    spieler.removeGoal(mannschaft.selector);
};

/*
 * *****************************
 * Functions that set up page content before/when a page is displayed.
 * These don't really belong into spiel.js, but well, there's no other
 * place for them now - gbl 20170818
 */

me_apps.obj.spiel.prototype.adjustViewport = function() {
    
    var h = screen.height * 0.9;
    var w = screen.width;

    var vscale = w / 1024;
    var hscale = h / 748;
    var scale  = (hscale < vscale ? hscale : vscale);
    var attr='width=1024, height=768, initial-scale='+scale+', maximum-scale='+(scale)+', user-scalable=0';
    var viewport = document.querySelector("meta[name=viewport]");
    viewport.setAttribute('content', attr);
    me_apps.log.important(attr);
};


me_apps.obj.spiel.prototype.setupIndexPage = function() {
    me_apps.log.debug("Haupt-Seite wird angezeigt");
    me_apps.log.logPosition();
    me_apps.onlineState.showState();
    $("#Nameheim").text(me_apps.spiel.heim.name);
    $("#Namegast").text(me_apps.spiel.gast.name);
    me_apps.teamMemberDialog.spielerToGUI(me_apps.spiel.heim);
    me_apps.teamMemberDialog.spielerToGUI(me_apps.spiel.gast);
    me_apps.teamMemberDialog.officersToGUI(me_apps.spiel.heim);
    me_apps.teamMemberDialog.officersToGUI(me_apps.spiel.gast);
    me_apps.spiel.heim.farbeToGUI();
    me_apps.spiel.gast.farbeToGUI();
    me_apps.protokollDialog.toGUI();
    me_apps.mainDialog.createTimeoutButtons();
    me_apps.mainDialog.mannschaftGoalCountToGUI(me_apps.spiel.heim);
    me_apps.mainDialog.mannschaftGoalCountToGUI(me_apps.spiel.gast);
    me_apps.mainDialog.suspensionsToGUI(me_apps.spiel.heim);
    me_apps.mainDialog.suspensionsToGUI(me_apps.spiel.gast);
    me_apps.mainDialog.gameTimeToGUI(me_apps.spiel.zeit);
    me_apps.mainDialog.gameTimeRunningToGUI(me_apps.spiel.zeitLäuft, me_apps.spiel.blockGameTimeStartSeconds > 0);
    me_apps.spiel.patchSettingsNavButton();
};

me_apps.obj.spiel.prototype.setupSpielwahlPage = function() {
    me_apps.log.debug("Spielwahl-Seite wird angezeigt");
    var cells = $("#SpieleListe tbody td");
    cells.removeClass("selected");
    me_apps.spieleDialog.zeigeSpiele();
    me_apps.spieleDialog.fillClubInput(me_apps.permStorage.defaultClub);
    if (navigator.onLine) {
        $("#SpieleLaden").removeClass("offline");
    } else {
        $("#SpieleLaden").addClass("offline");
    }
    me_apps.spieleDialog.fillMemoryDisplay();
    $('#spielwahl').page();
    me_apps.util.appstateToNavigationGUI();
    me_apps.spiel.patchSettingsNavButton();
};

me_apps.obj.spiel.prototype.setupSpielberichtPage=function () {
    me_apps.log.debug("Spielbericht-Seite wird angezeigt");
    $('#spielbericht').page();
    me_apps.spielbericht.bindTabs();
//
    var dateTimePicker = {preset: 'datetime', display: 'bubble', rows: 5, theme: 'jqm', mode: 'scroller', lang: 'de', stepMinute: 15,
        minDate: new Date(2017,0,1),
        onBeforeShow: function(html, inst) {
            if ((this.value === "") || (this.value.substring(0,3) === "NaN")) {
                var dat = new Date();
                var dattmp =[];
                dattmp.push(dat.getDate());
                dattmp.push(dat.getMonth());
                dattmp.push(dat.getFullYear());
                dattmp.push(dat.getHours());
                dattmp.push(0);
            }
            else {
                var dattmp = this.value.split(/\.| |:/);
                dattmp[1] = dattmp[1] -1;
            }
            inst.values = dattmp;
            inst.temp = dattmp;
        },
        onClose: function() {
            this.focus();
        }
    };

    var dateFields = [$('#sb4-A-Beginn'), $('#sb4-A-Ende'), $('#sb4-B-Beginn'), $('#sb4-B-Ende'),
        $('#sb5-ZN-Beginn'), $('#sb5-ZN-Ende'), $('#sb5-SK-Beginn'), $('#sb5-SK-Ende'),
        $('#sb6-SrB-Beginn'), $('#sb6-SrB-Ende'), $('#sb6-SpA-Beginn'), $('#sb6-SpA-Ende')];

    $.each(dateFields, function (index, value) {
        value.scroller(dateTimePicker).unbind('change').bind('change', function () {
            value.scroller('destroy').scroller(dateTimePicker);
        }).scroller('setDate', new Date());
    });

    var timePicker = {preset: 'time', display: 'bubble', rows: 5, theme: 'jqm', mode: 'scroller', lang: 'de', stepMinute: 1, width: 100};
    $('#sb7-Protokoll').scroller(timePicker).unbind('change').bind('change', function () {
            $('#sb7-Protokoll').scroller('destroy').scroller(timePicker);
    }).scroller('setDate', new Date());

    if (navigator.onLine) {
        $("#SpielAbschliessen").removeClass("offline");
    } else {
        $("#SpielAbschliessen").addClass("offline");
    }
// gbl 20210913 nee, die calculateSums Methode ist eh Schrott, das machen wir im changeHandler und dort in personFromGui.
//    $("#spielbericht div.Euro, #spielbericht div.EuroProKilometer, #spielbericht div.Kilometer").off("change");
//    $("#spielbericht div.Euro, #spielbericht div.EuroProKilometer, #spielbericht div.Kilometer").on( "change", me_apps.spielbericht.calculateSums);
    $("#spielbericht").unbind('change').bind('change', me_apps.spielbericht.changeHandler);

    me_apps.spielbericht.toGUI(me_apps.db.getCurrentGame());
    me_apps.spiel.patchSettingsNavButton();
};

/**
 * Things to do when the spielbericht page is hidden. We need to clear the
 * pins on page 10 when that happens.
 * @returns {undefined}
 */
me_apps.obj.spiel.prototype.hideSpielberichtPage=function () {
    me_apps.obj.spielbericht.prototype.clearPage10Pins();
};

me_apps.obj.spiel.prototype.setupEinstellungenPage = function () {
    me_apps.log.debug("Einstellungs-Seite wird angezeigt");
    var s = me_apps.spiel;

    $("#EinstellungenVersion").html(me_apps.getStageAndVersionString());
    $("#EinstellungenDeviceID").text(me_apps.db.uuid);
    // $("#EinstellungenDisplaySize").text(screen.width+" x "+screen.height);
    $("#EinstellungenDisplaySize").text(innerWidth+" x "+innerHeight);
    $("#HalbzeitMinuten").val(~~(s.dauerHalbzeit / 60));
    $("#HalbzeitSekunden").val(s.dauerHalbzeit % 60);
    $("#Verlaengerung").prop('checked', s.pokal).checkboxradio('refresh');
    $("#HinausstellungMinuten").val(~~(s.dauerHinausstellung / 60));
    $("#HinausstellungSekunden").val(s.dauerHinausstellung % 60);
    $("#AnzahlSpieler").val(s.anzahlSpieler);
    $("#NoOfTimeouts").val(s.anzahlTimeOut).selectmenu("refresh");
    $("#BuzzerTimeout").prop('checked', s.buzzerNachTimeout).checkboxradio('refresh');
    $("#BuzzerEnd").prop('checked', s.buzzerAmEnde).checkboxradio('refresh');
    $("#TimeOutMinuten").val(~~(s.dauerTimeout / 60));
    $("#TimeOutSekunden").val(s.dauerTimeout % 60);
    $("#einstellungen input, #einstellungen select").unbind("onchange");
    $("#einstellungen input, #einstellungen select").change(me_apps.spiel.changeEinstellungen);
    
    $("#EinstellungenSpieleLokal").text(me_apps.permStorage.finalizedGames.length);
    $("#EinstellungenMemoryGames").text(~~(localStorage.getItem("Appinfo").length/1000)+"KB");
    $("#EinstellungenMemoryLog").text(~~(localStorage.getItem("AppLog").length/1000)+"KB");
    
    if (me_apps.STAGE === "test") {
    // Make localStorage small, artifically. This is for testing only!
        var testString = new Array(4900*1000).join('a');
	try {
            localStorage.setItem('blob', testString);
	} catch (x) {
            localStorage.removeItem('blob');
	}
    } else {
        localStorage.removeItem('blob');
    }
        
    if (me_apps.util.checkLocalStorageCapacity(1000000)) {
        $("#EinstellungenMemoryFree").text(">1MB");
    } else {
        $("#EinstellungenMemoryFree").text(~~(me_apps.util.localStorageCapacity()/1000)+"KB");
    }
    
    me_apps.einstellungenDialog.setUploadButtonCount();
    me_apps.spiel.patchSettingsNavButton();
};

/**
 * Depending on whether or not the database has some uploads pending,
 * set the navigation button to orange so people know they have to
 * click there and check what's missing.
 * @returns {undefined}
 */
me_apps.obj.spiel.prototype.patchSettingsNavButton = function() {
    var count=me_apps.db.getPendingFinalizedUploads();
    var image=$(".navbutton-einstellungen img");
    if (count > 0) {
        image.prop("src", "images/einstellungen-oval-highlight.png");
    } else {
        image.prop("src", "images/einstellungen-oval.png");
    }
};

me_apps.obj.pageReady = function () {

    // me_apps.log won't work before the db object exists (at least if the
    // logging level is high enough)
    me_apps.db = new me_apps.obj.db();
    me_apps.db.initDB(false);
    me_apps.log.important("----------------- pageReady starting -----------------");
    me_apps.setStageFromURL(window.location.toString());
    
    $.mobile.page.prototype.options.domCache = true;
    $(document).bind("mobileinit", function () {
        $.mobile.defaultDialogTransition = "none";
        $.mobile.defaultPageTransition = "none";
    });
    // Textbausteine einfüllen
    $(document).on("pagebeforecreate", function (e, data) {
        if (e.target.id === "spielbericht") {
            var text;
            text=me_apps.util.formatTextsAsTemplate(me_apps.texts.penaltyreport, 'c');
            $("#texteRoteKarten").html(text);
            text=me_apps.util.formatTextsAsTemplate(me_apps.texts.various, 'a');
            $("#texteSonstige").html(text);
            
            new Image().src="images/Action-cancel-icon.png";
            new Image().src="images/Action-ok-icon.png";
            new Image().src="images/einstellungen-oval-highlight.png";
        }
    });
    me_apps.mainDialog = new me_apps.obj.mainDialog();
    me_apps.teamMemberDialog = new me_apps.obj.teamMemberDialog();
    me_apps.spieleDialog = new me_apps.obj.spieleDialog();
    me_apps.einstellungenDialog = new me_apps.obj.einstellungenDialog();
    me_apps.spielerDialog = new me_apps.obj.spielerDialog();
    me_apps.protokollDialog = new me_apps.obj.protokollDialog();
    me_apps.log.logPosition("dialogs created");

    me_apps.spieleDialog.verletzungsTexte = me_apps.texts.injury;

    $("#Spielerheim > div:first-child > table > tbody").bind(me_apps.IS_TOUCH ? "taphold" : "dblclick", me_apps.obj.mannschaft.editSpieler);
    $("#Spielergast > div:first-child > table > tbody").bind(me_apps.IS_TOUCH ? "taphold" : "dblclick", me_apps.obj.mannschaft.editSpieler);
    $("#Spielerheim  > table > tbody").bind(me_apps.IS_TOUCH ? "taphold" : "dblclick", me_apps.obj.mannschaft.editOffizieller);
    $("#Spielergast  > table > tbody").bind(me_apps.IS_TOUCH ? "taphold" : "dblclick", me_apps.obj.mannschaft.editOffizieller);
    $("#Spielerheim > div:first-child th:nth-child(3)").bind("click", me_apps.obj.mannschaft.toggleTeamNamenStrafen);
    $("#Spielerheim > div:first-child th:nth-child(4)").bind("click", me_apps.obj.mannschaft.toggleTeamNamenStrafen);
    $("#Spielergast > div:first-child th:nth-child(3)").bind("click", me_apps.obj.mannschaft.toggleTeamNamenStrafen);
    $("#Spielergast > div:first-child th:nth-child(4)").bind("click", me_apps.obj.mannschaft.toggleTeamNamenStrafen);
    $("#SpielerDialog").bind(me_apps.IS_TOUCH ? "tap taphold" : "click dblclick", me_apps.obj.mannschaft.spielerDialogHandler);
    me_apps.obj.startTimer();
    $("#Time").bind(me_apps.IS_TOUCH ? "taphold" : "dblclick", function (event) {
        me_apps.spielerDialog.openPopup('#SetTime');
        // Bug 17: openPopup ersetzt in den Eingabefeldern die Values durch die,
        // die im HTML selbst stehen, deshalb geht hier die aktuelle Zeit 
        // verloren.
        me_apps.mainDialog.gameTimeToGUI(me_apps.spiel.zeit);
    });
    me_apps.log.logPosition("timer started");

    $(document).off("pageshow", "#page").on("pageshow", "#page", me_apps.obj.spiel.prototype.setupIndexPage);
    me_apps.log.logPosition("pageshow #page set up");
    $(document).off('pageshow', '#spielwahl').on('pageshow', '#spielwahl', me_apps.obj.spiel.prototype.setupSpielwahlPage);
    me_apps.log.logPosition("pageshow #spielwahl set up");
    $(document).off('pageshow', '#spielbericht').on("pageshow", "#spielbericht", me_apps.obj.spiel.prototype.setupSpielberichtPage);
    me_apps.log.logPosition("pageshow #spielbericht set up");
    $(document).off('pagehide', '#spielbericht').on("pagehide", "#spielbericht", me_apps.obj.spiel.prototype.hideSpielberichtPage);
    me_apps.log.logPosition("pagehide #spielbericht set up");
    $(document).off('pageshow', '#einstellungen').on("pageshow", "#einstellungen", me_apps.obj.spiel.prototype.setupEinstellungenPage);
    me_apps.log.logPosition("pageshow #einstellungen set up");

    me_apps.db.loadRecentlyUsedGame();
    me_apps.spiel.setupIndexPage();
    me_apps.util.appstateToNavigationGUI();
};
