/*
 * Handle the game selection page (spielwahl.html)
 */

if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}

if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.spieleDialog = function () {};

me_apps.obj.spieleDialog.prototype.ladeSpiele = function () {
    me_apps.log.logCall();
    var vereine = $("#SelektierteVereine")[0].value;
    me_apps.permStorage.defaultClub=vereine;
    if (!me_apps.util.checkLocalStorageCapacity(270000)) {
        this.openOutOfMemoryPopup();
        return;
    }
    me_apps.db.downloadGameList(vereine, me_apps.spieleDialog.showGameListDownloadProgress);
};

me_apps.obj.spieleDialog.prototype.fillClubInput = function(val) {
    if (val !== undefined) {
        var vereine = $("#SelektierteVereine")[0];
        vereine.value=val;
    }
}

/*
 * When downloading the game list, show progress, and if this is going to
 * be the last message, redraw the game list and schedule the progress
 * popup to close.
 * @param {int} status 0 = ok, 1..n = error codes. Unused at the moment. Could
 *  be used to make the background of the status window red or green.
 * @param {string} message The message to display
 * @param {boolean} done if true, redisplay the game list and schedule hiding the popup
 * @returns {undefined}
 */
me_apps.obj.spieleDialog.prototype.showGameListDownloadProgress
= function(status, message, done) {
    $.mobile.loading( "show" , { text: message, textVisible: true, theme: "a" });
    if (done) {
        me_apps.spieleDialog.zeigeSpiele();
        me_apps.spieleDialog.fillMemoryDisplay();
        setTimeout(function() { $.mobile.loading("hide"); }, 3000);
    }
}
    
me_apps.obj.spieleDialog.prototype.zeigeSpiele = function() {  
    me_apps.log.logCall();
    var table = $("#SpieleListe tbody");
    var html = "";
    me_apps.db.enumGameList(
        function (game, isCurrent) /* display function */
        {
            var help=game.datetime;
            var dateShow=me_apps.util.datetimeToDDMMYYYY(help);
            var timeShow=me_apps.util.datetimeToHHMM(help);
            var trashCanDisplay="none";
            if (me_apps.util.gameHasDetailsLoaded(game)) {
                trashCanDisplay="block";
            }
            me_apps.log.debug("Game "+game.id+" trashcan is "+trashCanDisplay);
            me_apps.log.debug("row.referees is "+typeof(game.referees)+
                    ", team_guest is "+typeof(game.team_guest)+", team_home is "+typeof(game.team_home));

            html += "<tr"
                    + (isCurrent ? " class='rowSelected'" : "")
                    + ">"
                    + "<td>" + game.id + "</td>"
                    + "<td>" + game.no + "</td>"
                    + "<td>" + dateShow+ "</td>"
                    + "<td>" + timeShow + "</td>"
                    + "<td>" + game.class_sname + "</td>"
                    + "<td>" + game.home_sname + " - " + game.guest_sname + "</td>"
                    + "<td><img"
                        + " onclick='me_apps.spieleDialog.deleteGameWithConfirmation(\"" + game.id + "\")'"
                        + " style='display:"+trashCanDisplay+"'"
                        + " src='images/trashcan.png' alt='&lt;|-|&gt;' width='40px'></td>"
                    + "</tr>";
        }, 
        function(a, b) { /* order function */
            var result= a.datetime.localeCompare(b.datetime);
            if (result === 0)
                result= a.no - b.no;
            // me_apps.log.info("game compare "+a.no+"/"+a.datetime+" - "+b.no + "/" + b.datetime+" returns "+result);
            return result;
        }
    );
    table.html(html);
    me_apps.log.logPosition();

    var cells = $("#SpieleListe tbody > tr > td:not(:last-child)");
    cells.off("vclick").one("vclick", function (event) {
        me_apps.spieleDialog.spielSelected(event);
    });
};

me_apps.obj.spieleDialog.prototype.deleteGameWithConfirmation =
function(id, callback) {
    me_apps.log.logCall();
    me_apps.spielerDialog.openPopup('#DialogTeamDelete', true);
    $('#DialogConfirmDeleting').css('width', '100%');
    $('.confirmDelete').unbind('click').bind('click', function (e) {
        e.preventDefault();
        e.stopPropagation();

        var btnId = $(this).attr('id');
        me_apps.log.debug("Button "+btnId+" pressed");
        if (btnId === 'confirmDeleteOK') {
            me_apps.log.info("Game "+id+" deleted by user confirm");
            me_apps.db.deleteGame(id);
            me_apps.spieleDialog.zeigeSpiele();
            me_apps.spieleDialog.fillMemoryDisplay();
        }
        $('#DialogTeamDelete').popup('close');
    });
};

me_apps.obj.spieleDialog.prototype.spielSelected = function (event) {

    $("#Ergebnisse").hide();        // Pfusch! Das ist der "Ergebnisse Protokollieren" Button auf der Spielseite. So verschwindet er wenigstens wenn ein neues Spiel gestartet wird.
    event.stopPropagation();
    event.preventDefault();

    var row = event.currentTarget.parentElement;
    var spielID = row.children[0].innerHTML;
    var nummer = row.children[1].innerHTML;
    var datumStrings = row.children[2].innerHTML.split(".");

    if (!me_apps.util.gameHasDetailsLoaded(me_apps.db.getGameById(spielID))
    &&  !me_apps.util.checkLocalStorageCapacity(270000)) {
        this.openOutOfMemoryPopup();
        return;
    }

    $.mobile.loading( "show" , { text: "Laden des Spiels...", textVisible: true, theme: "a" });
    event.currentTarget.parentElement.className = "rowSelected";

    me_apps.db.setCurrentGame(nummer, spielID, datumStrings[2] + "-" + datumStrings[1] + "-" + datumStrings[0],
        function(status, game, message, done) {
            $.mobile.loading("show", { text: message , textVisible: true, theme: "a" });
            if (game!==null) {
                me_apps.spiel.copyValuesFromPermStore(game);
            }
            if (done) {
                me_apps.obj.spieleDialog.prototype.zeigeSpiele();
                setTimeout(function() {$.mobile.loading("hide")}, 3000);
            }
            me_apps.spieleDialog.fillMemoryDisplay();
        });

};

me_apps.obj.spieleDialog.prototype.fillMemoryDisplay = function(event) {
    if (me_apps.util.checkLocalStorageCapacity(1000000)) {
        $("#SpielwahlMemoryAvailable").text(">1MB");
    } else {
        $("#SpielwahlMemoryAvailable").text(~~(me_apps.util.localStorageCapacity()/1000)+"KB");
    }
};

me_apps.obj.spieleDialog.prototype.openOutOfMemoryPopup = function() {
    me_apps.spielerDialog.openPopup("#DialogOutOfMemory", true);
    $('#DialogConfirmOOM').css('width', '100%');
    $('.confirmOOM').unbind('click').bind('click', function (e) {
        e.preventDefault();
        e.stopPropagation();
        $('#DialogOutOfMemory').popup('close');
    });
};