if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}
if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.spieler =
function(id) {
    this.setDefaultValues(id);
};

me_apps.obj.spieler.prototype.setDefaultValues =
function(id, tore) {
    this.id                     = id;
    this.spielId                = "?";
    this.bezeichner             = 0;
    this.name                   = "--";
    this.vorname                = "-- ";
    this.geburtsjahr            = 2000;
    this.passNr                 = 0;
    this.DSR                    = false;
    this.aktiv                  = true;
    this.tore                   = (typeof (tore) === 'undefined') ? 0 : tore;
    this.ausweiskontrolliert    = false;
    this.encrypted              = false;
    this.hasPassbild            = false;
    
    this.countedInPlayerQuota   = false;
    this.countedInOfficerQuota  = false;
    
    this.disc_p2                = 0;            // penalty - 2 Minuten hinaus - H
    this.disc_d                 = 0;            // disqualified - rote Karte - D
    this.disc_w                 = 0;            // warning - verwarnung -V
    this.disc_r                 = 0;            // disqualified report - rote Karte mit Bericht - D
    this.disc_t2                = 0;            // team - 2 Minuten hinaus - T
};

/**
 * @function me_apps.obj.spieler.prototype.copyValuesFromPermStore
 * Copy everything the database knows about a player into the app object.
 * @param {object} dbdef the player in database format
 * If we get a team member who doesn't have an id, assign an id in the perm
 * storage as well.
 * Warning, everything that may be encrypted will be a string in the database,
 * not an integer or boolean value or anything.
 */

me_apps.obj.spieler.prototype.copyValuesFromPermStore =
function(dbdef) {
    if (dbdef.person_id === "") {
        dbdef.person_id=me_apps.util.getNextId();
    } else {
        me_apps.util.markIdSeen(dbdef.person_id);
    }
    this.id                     = dbdef.person_id;
    this.bezeichner             = dbdef.player_no;
    this.name                   = dbdef.name;
    this.vorname                = dbdef.prename;
    this.geburtsjahr            = dbdef.birthday;
    this.passNr                 = dbdef.lic_number;
    this.DSR                    = dbdef.dsr === "1";
    this.aktiv                  = dbdef.active === "1";
    this.tore                   = dbdef.num_goals;
    this.ausweiskontrolliert    = dbdef.lic_checked;
    this.hasPassbild            = dbdef.has_image;              // readonly
    this.encrypted              = dbdef.encrypted === true;     // set everything else to false
    
    this.countedInOfficerQuota  = dbdef.counted_officer_quota || false;
    this.countedInPlayerQuota   = dbdef.counted_player_quota || false;
    
    this.disc_p2                = dbdef.disc_p2;
    this.disc_d                 = dbdef.disc_d;
    this.disc_w                 = dbdef.disc_w;
    this.disc_r                 = dbdef.disc_r;
    this.disc_t2                = dbdef.disc_t2;
};

me_apps.obj.spieler.prototype.copyValuesToPermStore =
function(dbdef) {
    dbdef.person_id             = this.id;
    dbdef.player_no             = this.bezeichner;
    dbdef.name                  = this.name;
    dbdef.prename               = this.vorname;
    dbdef.birthday              = this.geburtsjahr;
    dbdef.lic_number            = this.passNr;
    dbdef.dsr                   = this.DSR ? "1" : "0";
    dbdef.active                = this.aktiv ? "1" : "0";
    dbdef.num_goals             = this.tore;
    dbdef.lic_checked           = this.ausweiskontrolliert;
    dbdef.encrypted             = this.encrypted;
    
    dbdef.counted_player_quota  = this.countedInPlayerQuota;
    dbdef.counted_officer_quota = this.countedInOfficerQuota;
    
    dbdef.disc_p2               = this.disc_p2;
    dbdef.disc_d                = this.disc_d;
    dbdef.disc_w                = this.disc_w;
    dbdef.disc_r                = this.disc_r;
    dbdef.disc_t2               = this.disc_t2;
};

/**
 * @function addSpielerPropertiesToOfficer
 * Add the properties to a spieler object that might not be set when
 * a officer is changed to a spieler. Be careful not to change anything
 * that is set; the object might have been turned from spieler to officer to
 * spieler again, and we don't want to lose any information in this case.
 * @returns {undefined}
 */
me_apps.obj.spieler.prototype.addSpielerPropertiesToOfficer =
function() {
    if (!this.id)               { this.id = id=me_apps.util.getNextId(); }
    // bezeichner needs to be set in officerRoleChange
    // name, vorname is set in officers as well
    if (!this.geburtsjahr)      { this.geburtsjahr = "00.00.0000"; }
    if (!this.passNr)           { this.passNr = "0"; }
    if (!this.DSR)              { this.DSR = false; }
    // active is set in officerRoleChange
    if (!this.tore)             { this.tore=0; }
    if (!this.ausweiskontrolliert)  { this.ausweiskontrolliert=false; }
    // encrypted is set in Officers as well
    if (!this.disc_p2 )         { this.disc_p2=0; }
    if (!this.disc_d  )         { this.disc_d =0; }
    if (!this.disc_w  )         { this.disc_w =0; }
    if (!this.disc_r  )         { this.disc_r =0; }
    if (!this.disc_t2 )         { this.disc_t2=0; }
};

me_apps.obj.spieler.prototype.isOfficer =
function() {
    me_apps.log.logCall();
    me_apps.log.debug("isOfficer: bezeichner='"+this.bezeichner+"', length="+this.bezeichner.length+", index="+"ABCD".indexOf(this.bezeichner));
    // return (this.bezeichner.length===1 && isNaN(this.bezeichner));
    return this.bezeichner.length===1 && "ABCD".indexOf(this.bezeichner)>=0;
};

me_apps.obj.spieler.prototype.getDisciplineString =
function() {
    var result='';
    for (var i=0; i<this.disc_w; i++) {
        result+='V';
    }
    for (var i=0; i<this.disc_p2; i++) {
        result+='H';
    }
    for (var i=0; i<this.disc_d; i++) {
        result+='D';
    }
    for (var i=0; i<this.disc_r; i++) {
        result+='B';
    }
    for (var i=0; i<this.disc_t2; i++) {
        result+='Z';
    }
    return result;
};

/**
 * @param {string} team home or guest
 * @param {string} type one of H, D, V, B, Z
 * @return {undefined}
 */

me_apps.obj.spieler.prototype.addDiscipline =
function(team, type) {
    var fieldname=this.getDisciplineFieldName(type);
    if (this[fieldname] === undefined)
        this[fieldname]=0;
    this[fieldname]++;
    me_apps.db.updateSpieler(this, team);
};

/**
 * @param {string} team home or guest
 * @param {string} type one of H, D, V, B, Z
 * @return {undefined}
 */

me_apps.obj.spieler.prototype.removeDiscipline =
function(team, type) {
    var fieldname=this.getDisciplineFieldName(type);
    if (this[fieldname]>0)
        this[fieldname]--;
    me_apps.db.updateSpieler(this, team);
};

me_apps.obj.spieler.prototype.addGoal =
function(team) {
    this.tore++;
    me_apps.db.updateSpieler(this, team);
};

me_apps.obj.spieler.prototype.removeGoal =
function(team) {
    this.tore--;
    me_apps.db.updateSpieler(this, team);
};

me_apps.obj.spieler.prototype.getDisciplineFieldName =
function(type) {
        if (type === 'H')         return "disc_p2";
        if (type === 'D')         return "disc_d";
        if (type === 'V')         return "disc_w";
        if (type === 'B')         return "disc_r";
        if (type === 'Z')         return "disc_t2";     // Zusätzliche Strafe - not Team - T conflicts with Tor
};