/*
 * Handle the various popups that happen in clicks on the main page (index.html)
 * while the game is running (7m, goal, ...).
 */

if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}

if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.spielerDialog = function () {};

me_apps.obj.spielerDialog.prototype.setTitle = function (text) {
    var p = $('#SpielerDialog p');
    p[0].innerHTML = text;
};

me_apps.obj.spielerDialog.prototype.openPopup = function (selector, outside) {

    me_apps.log.debug("Dialog  '" + selector + "' geöffnet.");

    var timeRegion = $(selector + ' div.PopupTime');
    timeRegion.hide();

    var p = $('#Popups');
    me_apps.log.debug('Popup-height ' + selector + ': ' + p.height());
    if (outside !== true) {
        $(selector + ' > div:first').css({
            'width': p.width(), 'height': p.height(),
            'max_width': p.width(), 'max-height': p.height(),
            'min_width': p.width(), 'min-height': p.height()
        });
    }
    var popup = $(selector);

    popup.popup('open', {positionTo: '#Popups', transition: "none"});
    timeRegion.show();
    // due to an browser erron in android force to redisplay the element
        // disabled this because it interferes with checkboxes in popups ?
        // var content = popup[0].innerHTML;
        // popup[0].innerHTML = content;
};

me_apps.obj.spielerDialog.rowHeight = 120;
/**
 * showToreDialog
 * @param {me_apps.obj.mannschaft} mannschaft
 */
me_apps.obj.spielerDialog.prototype.showToreDialog = function (mannschaft) {
    this.setTitle('Tor für ' + mannschaft.name + ' durch:');
    var html = "";
    mannschaft.aktiveSpieler().forEach(function (spieler) {
        var disabled = (spieler.disc_d !== 0 || spieler.disc_r !== 0) 
                    || mannschaft.istDraussen(spieler.bezeichner);
        html += "<div class='PopupSpieler'><a class='SpielerButton" + (disabled ? " ui-disabled' " : "' "
                + "data-handler='me_apps.spielerDialog.addTor(me_apps.spiel." + mannschaft.selector + "," + spieler.id + ");'") + ">"
                + ((spieler.bezeichner === null) ? '?' : spieler.bezeichner) + "</a><div>" + spieler.vorname + "</div><div>" + spieler.name + "</div></div>";
    });
    $('#SpielerDialog > div:first-child > div:nth-of-type(2)')[0].innerHTML = html;
    this.openPopup('#SpielerDialog');
    this.adjustPopupHeight(mannschaft.aktiveSpieler().length);
};

/**
 * showToreDialog
 * @param {me_apps.obj.mannschaft} mannschaft
 */
me_apps.obj.spielerDialog.prototype.showSiebenMeterDialog = function (mannschaft) {
    this.setTitle("7-m-Wurf für: " + mannschaft.name + ' durch:');
    var html = "";
    mannschaft.aktiveSpieler().forEach(function (spieler) {
        var disabled = spieler.disc_d !== 0  || spieler.disc_r !== 0 || mannschaft.istDraussen(spieler.bezeichner);
        html += me_apps.obj.spielerDialog.prototype.toIconHTML(spieler, disabled, mannschaft.selector,
            disabled ? 'reportActionImpossible' : 'showSiebenMeterTorDialog');
    });
    $('#SpielerDialog > div > div:nth-of-type(2)')[0].innerHTML = html;
    this.openPopup('#SpielerDialog');
    this.adjustPopupHeight(mannschaft.aktiveSpieler().length);
};
/**
 * showStrafenDialog
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {String} strafe 
 */
me_apps.obj.spielerDialog.prototype.showStrafenDialog = function (mannschaft, strafe) {
    var text = {
        "V": "Verwarnung für ",
        "D": "Disqualifikation für ",
        "H": "2 Minuten Strafe für "
    };
    var titleText=text[strafe] + mannschaft.name;

    var anzahlGelbeKartenSpieler = 0;
    var anzahlGelbeKartenOfficer = 0;
    var anzahlRoteKartenOfficer=0;
    if ((strafe.indexOf("V") !== -1)) {
        mannschaft.spieler.forEach(function (spieler) {
            if (spieler.disc_w !== 0) {
                anzahlGelbeKartenSpieler++;
            }
        });
        mannschaft.offizielle.forEach(function (officer) {
            if (officer.disc_w !== 0) {
                anzahlGelbeKartenOfficer++;
            }
            if (officer.disc_d !== 0 || officer.disc_r !== 0) {
                anzahlRoteKartenOfficer++;
            }
        });
        if (anzahlGelbeKartenSpieler>=3)
            titleText+=" ("+anzahlGelbeKartenSpieler+" Verwarnungen)";
    }
    this.setTitle(titleText);

    var html = "";
    mannschaft.aktiveSpieler().forEach(function (spieler) {
        var funcName='addStrafe';
        var disabled = false;
        var realStrafe=strafe;      // might be changed from H to Z for one player only

        // no yellow cards for players who have one, have a time penalty,
        // and no 3rd yellow card for one team, but give some leeway in some cases
        if (strafe.indexOf('V') !== -1) {
            if (spieler.disc_w>0 || spieler.disc_p2 >0 || anzahlGelbeKartenSpieler>=3) {
                disabled=true;
                funcName='confirmAddStrafe';
                if (spieler.disc_w>1 || anzahlGelbeKartenSpieler>=4) {
                    funcName='reportActionImpossible';
                }
            }
        }
        
        // No time penalty if the player already has a double one
        if (strafe.indexOf('H') !== -1
        &&  mannschaft.hasPlayerMaxPenaltyStarting(spieler)) {
            disabled=true;
            funcName='reportActionImpossible';
        }

        // Disqualified players cannot get another penalty. Except, 2 minutes
        // team penalty (Z, Zusätzliche Strafe), and even that only if the D/B was a
        // new 2 minute penalty; we can't add to a 4 minute penalty. See #207.
        if (spieler.disc_d !== 0 || spieler.disc_r !== 0) {
            if (strafe.indexOf('H') !== -1
            && me_apps.protokollDialog.disqualificationTime(spieler)===me_apps.spiel.zeit
            ) {
                // note that a possible disabled from being at 4:00 does not get reverted here
                realStrafe='Z';
            } else {
                disabled = true;
                funcName='reportActionImpossible';
            }
        }
        
        // Guideline from OP #985: If a player has just received a D, they can get
        // an additional D as long as the time hasn't changed. In this case, the
        // player gets a B and the disqualification time increases to 4 minutes.
        if (spieler.disc_d !== 0 && spieler.disc_r === 0 && strafe === 'D'
        &&  me_apps.protokollDialog.disqualificationTime(spieler)===me_apps.spiel.zeit) {
            realStrafe = 'B';
            disabled = 'bluecard';
            funcName = 'confirmAddDisqualification';
        }
        
        html += me_apps.obj.spielerDialog.prototype.toIconHTML(spieler, disabled, mannschaft.selector, funcName, realStrafe);
    });

    mannschaft.aktiveOffizielle().forEach(function (offizieller) {
        var funcName='addStrafe';
        var disabled = false;
        if (strafe.indexOf("V") !== -1) {
            if (offizieller.disc_w !== 0 || anzahlGelbeKartenOfficer >0) {
                disabled=true;
                funcName='confirmAddStrafe';
                if (offizieller.disc_w>1 || anzahlGelbeKartenOfficer>=2) {
                    funcName='reportActionImpossible';
                }
            }
        }
        // Telefonat Andreas Jakob 20180321 - Zusätzliche Strafe ist nur Spieler, keine Offiziellen.
        if (strafe.indexOf("D") !== -1
        && (offizieller.disc_d !== 0 || offizieller.disc_r !== 0 || anzahlRoteKartenOfficer !== 0)) {
            disabled=true;
            funcName='reportActionImpossible';
        }
        html += me_apps.obj.spielerDialog.prototype.toIconHTML(offizieller, disabled, mannschaft.selector, funcName, strafe);
    });
    $('#SpielerDialog > div:first-child > div:nth-of-type(2)')[0].innerHTML = html;
    this.openPopup('#SpielerDialog');
    this.adjustPopupHeight(mannschaft.aktiveSpieler().length + mannschaft.offizielle.length);
};

me_apps.obj.spielerDialog.prototype.adjustPopupHeight = function(numberOfContentEntries) {
    var rowCount = Math.ceil((numberOfContentEntries) / 5);
    var PopUpHeight = rowCount * me_apps.obj.spielerDialog.rowHeight;
    PopUpHeight = (PopUpHeight < 410) ? 410 : PopUpHeight;
    $('#SpielerDialog > div:first-child > div:nth-of-type(2)').height(PopUpHeight);
    if (rowCount > 4) {
        $('#SpielerDialog-popup').css('top', '40px');
        $('#SpielerDialog' + ' > div:first').css({
            'height': PopUpHeight,
            'max-height': PopUpHeight,
            'min-height': PopUpHeight
        });
    }
};

/**
 * showSiebenMeterTorDialog
 * This is called from after we select a player who threw the 7m; it replaces
 * the contents of the current dialog (player selection) with the goal yes/no
 * question.
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {Integer} id 
 */
me_apps.obj.spielerDialog.prototype.showSiebenMeterTorDialog = function (mannschaft, id) {
    var spielerIcons = $('#SpielerDialog > div:first > div:nth-of-type(2)');
    spielerIcons[0].innerHTML = "<div id='DialogTextHead'><span class='Farbe" + mannschaft.selector + "'>Hat der Spieler mit Nr " + mannschaft.getSpielerById(id).bezeichner + " ein Tor erzielt ?</span><br>"
            + "<button data-handler='me_apps.spielerDialog.addTor(me_apps.spiel." + mannschaft.selector + ", " + id + ", true);' data-inline='true'>Ja</button>"
            + "<button data-handler='me_apps.spielerDialog.addSiebenMeterOhneTor(me_apps.spiel." + mannschaft.selector + ", " + id + ");' data-inline='true'>Nein</button></div>";
    spielerIcons.trigger('create');
};

/**
 * 
 * @param {me_apps.obj.spieler} spieler
 * @param {boolean} disabled
 * @param {string} heimgast "heim" or "gast"
 * @param {string} method name of the method to call on click
 * @param {string} extra string parameter to give to the method
 * @returns {String}
 */
me_apps.obj.spielerDialog.prototype.toIconHTML =
function(spieler, disabled, heimgast, method, extra) {
    var extraparm = extra ? ", '"+extra+"'" : "";
    
    // As in some other contexts as well, officers don't neccesarily have an ID,
    // so use their letter instead.
    var id=spieler.id;
    if (spieler.isOfficer())
        id="'"+spieler.bezeichner+"'";
    else
        id="'"+id+"'";
    
    return ''
        + '<div class="PopupSpieler">'
        + '<a class="SpielerButton'
            + (disabled ? ' ui-disabled' : '')
            + (typeof(disabled) === "string" ? ' '+disabled : '')
            + '"   data-handler="me_apps.spielerDialog.'+method+'(me_apps.spiel.'+heimgast+','+id+extraparm+');"'
        + '>'
        + (spieler.bezeichner === "" ? "?" : spieler.bezeichner)
        + '</a>'
        + '<div>'+spieler.vorname+'</div>'
        + '<div>'+spieler.name+'</div>'
        + '</div>';
};

/**
 * @function addTor
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {Integer} spielerId
 * @param {Bool} siebenMeter
 */
me_apps.obj.spielerDialog.prototype.addTor = function (mannschaft, spielerId, siebenMeter) {
    me_apps.mainDialog.addTor(mannschaft.selector === 'heim' ? 'H' : 'G', spielerId, siebenMeter);
    $('#SpielerDialog').popup('close');
};

me_apps.obj.spielerDialog.prototype.addSiebenMeterOhneTor = function (mannschaft, spielerId) {
    me_apps.protokollDialog.addProtocol(mannschaft, spielerId, "7MOT");
    $("#SpielerDialog").popup("close");
};

me_apps.obj.spielerDialog.prototype.reportActionImpossible = function(mannschaft, spielerId, strafe) {
    var dialogContent = $('#SpielerDialog > div:first > div:nth-of-type(2)');
    dialogContent[0].innerHTML="<div id='DialogTextHead'>Nicht möglich. Bitte SR informieren.</div>";
    dialogContent.trigger('create');
};

/**
 * confirmAddStrafe - confirm penalty actions that are questionable (3 yellow
 * cards for one team etc.)
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {int} spielerId
 * @param {string} strafe
 * @returns {undefined}
 */

me_apps.obj.spielerDialog.prototype.confirmAddStrafe = function(mannschaft, spielerId, strafe) {
    var spieler = mannschaft.getSpielerById(spielerId);
    var dialogContent = $('#SpielerDialog > div:first > div:nth-of-type(2)');
    dialogContent[0].innerHTML = "<div id='DialogTextHead'> Spieler " + ((spieler.bezeichner === null) ? '?' : spieler.bezeichner) + "<br>"
            + "<button data-handler='me_apps.spielerDialog.addStrafeByTeamNumber(" + mannschaft.getGastAsInt() + ", \"" + spielerId + "\", \""+strafe+"\")' data-inline='true'>Ja</a>"
            + "<button data-handler='$(\"#SpielerDialog\").popup(\"close\");'  data-inline='true'>Nein</button></div>";
    dialogContent.trigger('create');
};

/**
 * confirmAddDisqualification - confirm extra blue card after red card
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {int} spielerId
 * @param {string} strafe
 * @returns {undefined}
 */
me_apps.obj.spielerDialog.prototype.confirmAddDisqualification = function(mannschaft, spielerId, strafe) {
    var spieler = mannschaft.getSpielerById(spielerId);
    this.setTitle("Zusätzliche Disqualifikation für "+mannschaft.name);
    var dialogContent = $('#SpielerDialog > div:first > div:nth-of-type(2)');
    dialogContent[0].innerHTML = "<div id='DialogTextHead'> Spieler " + ((spieler.bezeichner === null) ? '?' : spieler.bezeichner) + "<br>"
            + "<button data-handler='me_apps.spielerDialog.addStrafeByTeamNumber(" + mannschaft.getGastAsInt() + ", \"" + spielerId + "\", \"B\")' data-inline='true' data-theme='blauekarte'>Ja</a>"
            + "<button data-handler='$(\"#SpielerDialog\").popup(\"close\");'  data-inline='true' data-theme='rotekarte' >Nein</button></div>";
    dialogContent.trigger('create');
};

/**
 * addStrafeByTeamNumber - just like addStrafe, but using 0 or 1 instead of the team
 * object. Neccesary because we can't (easily) tell the UI to pass an object
 * in a dynamically generated link.
 * @param {type} mannschaftAsInt - 0 home, 1 guest
 * @param {integer} spielerId
 * @param {string} strafe
 */
me_apps.obj.spielerDialog.prototype.addStrafeByTeamNumber = function(mannschaftAsInt, spielerId, strafe) {
    var mannschaft = me_apps.spiel.heim;
    if (mannschaftAsInt === 1) {
        mannschaft = me_apps.spiel.gast;
    }
    return me_apps.spielerDialog.addStrafe(mannschaft, spielerId, strafe);
};

/**
 * addStrafe
 * @param {me_apps.obj.mannschaft} mannschaft
 * @param {Integer} spielerId 
 * @param {String} strafe entweder V H D , muss in dieser Form in me_apps.protokollEintraege stehen
 */
me_apps.obj.spielerDialog.prototype.addStrafe = function (mannschaft, spielerId, strafe) {
    var spieler = mannschaft.getSpielerById(spielerId);
    if (strafe === "D") {
        var dialogContent = $('#SpielerDialog > div:first > div:nth-of-type(2)');
        dialogContent[0].innerHTML = "<div id='DialogTextHead'>Disqualifikation des Spielers " + ((spieler.bezeichner === null) ? '?' : spieler.bezeichner) + " mit Bericht?<br>"
                + "<button data-handler='me_apps.spielerDialog.addRoteKarte(" + mannschaft.getGastAsInt() + ", \"" + spielerId + "\", false)' data-inline='true' id='roteKarte' data-theme='rotekarte'>ohne<br>Bericht</button>"
                + "<button data-handler='me_apps.spielerDialog.addRoteKarte(" + mannschaft.getGastAsInt() + ", \"" + spielerId + "\", true)' data-inline='true' id='blaueKarte' data-theme='blauekarte'>mit<br>Bericht</button>"
                + "</div>";
        dialogContent.trigger('create');
    } else {
        spieler.addDiscipline(mannschaft.selector, strafe);
        me_apps.obj.mainDialog.prototype.penaltiesToGUI(mannschaft, spieler);
        me_apps.protokollDialog.addProtocol(mannschaft, spielerId, strafe);
        
        // bei 3. Zweiminuten Strafe gibt es rot, aber nicht, wenn der Spieler
        // schon disqualifiziert ist und die Hinausstellung eine Zusätzliche Strafe war
        if (spieler.disc_p2 >= 3 && spieler.disc_d === 0 && spieler.disc_r === 0) {
            me_apps.spielerDialog.addRoteKarte(mannschaft.getGastAsInt(), spielerId, false, true);
        }
        if (strafe === 'H' || strafe === 'Z' || strafe === 'B') {
            me_apps.spielerDialog.addZweiMinutenStrafe(mannschaft, spielerId);
        }
        $('#SpielerDialog').popup('close');
    }
};

me_apps.obj.spielerDialog.prototype.addRoteKarte = function (mannschaftAsInt, spielerId, mitBericht, isAutoDisqualify) {
    var mannschaft = me_apps.spiel.heim;
    if (mannschaftAsInt === 1) {
        mannschaft = me_apps.spiel.gast;
    }

    var spieler = mannschaft.getSpielerById(spielerId);
    var penaltyType='D';
    if (spieler !== null) {
        if (mitBericht) {
            spieler.disc_r++;
            penaltyType="B";
        } else {
            spieler.disc_d++;
        }
        me_apps.obj.mainDialog.prototype.penaltiesToGUI(mannschaft, spieler);
        me_apps.db.updateSpieler(spieler, mannschaft.selector);
    }
    $('#SpielerDialog').popup('close');
    me_apps.protokollDialog.addProtocol(mannschaft, spielerId, penaltyType);
    // 2 minute penalty after disqualification, but not if it's an automatic disqualify because third 2-min-penality
    if (!isAutoDisqualify)
        me_apps.spielerDialog.addZweiMinutenStrafe(mannschaft, spielerId);
};

me_apps.obj.spielerDialog.prototype.addZweiMinutenStrafe = function (mannschaft, spielerId, duration) {

    var dauerHinausstellung = me_apps.spiel.dauerHinausstellung;
    if (duration !== undefined) {
        var arrTime = duration.split(':');
        dauerHinausstellung = parseInt(arrTime[0]) * 60 + parseInt(arrTime[1]);
    }
    var hasHinausstellung = mannschaft.hinausstellungen.filter(function (element) {
        return (element.id === spielerId);
    });

    var endOfStandardPenalty=me_apps.spiel.zeit + me_apps.spiel.dauerHinausstellung;
    var endOfDoublePenalty  =me_apps.spiel.zeit + me_apps.spiel.dauerHinausstellung*2;
    // If we already have a 4 minute penalty that hasn't been started yet, we don't add anything.
    // If we have a 2 minute penalty that hasn't started, we extend it.
    // Else we create a new one.
    // We need to check the *last* penalty of that player - the one that ends last.
    var needNew=true;
    if (hasHinausstellung.length > 0) {
        if (hasHinausstellung[hasHinausstellung.length-1].ende === endOfDoublePenalty)
            return;
        if (hasHinausstellung[hasHinausstellung.length-1].ende === endOfStandardPenalty) {
            var spieler = mannschaft.getSpielerById(spielerId);
            if (!spieler.isOfficer()) {
                // Officers don't get double penalties.
                hasHinausstellung[hasHinausstellung.length-1].ende += dauerHinausstellung;
            }
            needNew=false;
        }
    }
    if (needNew) {
        var newHinausstellung = {
            id: spielerId,
            ende: me_apps.spiel.zeit + dauerHinausstellung
        };
        mannschaft.hinausstellungen.push(newHinausstellung);
    }

    mannschaft.hinausstellungen.sort(function(a, b){ return a.ende-b.ende; });
    me_apps.db.updateTeam(mannschaft, true);
    me_apps.mainDialog.suspensionsToGUI(mannschaft);
};
