/* global CryptoJS */

if (typeof (me_apps) === 'undefined') {
    me_apps = {};
}

if (typeof (me_apps.obj) === 'undefined') {
    me_apps.obj = {};
}

me_apps.obj.util = function() {
    this.serializer = new XMLSerializer();
};

me_apps.obj.util.prototype.formatTextsAsTemplate = function(object, area) {
    var result="";
    result+='<div data-role="collapsible-set">';
    for (var section in object) {
        result+=
            '<div data-collapsed="true" data-role="collapsible">'+
            '<h3>'+section+'</h3>'+
            '<ul data-role="listview">';
        for (var item in object[section]) {
            result+=
                '<li>'+
                '<a onClick="me_apps.spielbericht.addTextToBericht(\''+area+'\',\'' +object[section][item].split("'").join("\\'") +'\')">'+
                '<h3>'+item+'</h3>'+
                '<p class="ui-li-desc">'+object[section][item]+'</p>'+
                '</a>'+
                '</li>';
        }
        result+=
            '</ul>'+
            '</div>';
    }
    result+='</div>';
    return result;
};

me_apps.obj.util.prototype.createUUID = function() {
    var uuid = CryptoJS.MD5(new Date().getTime().toString()).toString();
    return uuid;
};

me_apps.obj.util.prototype.transfomXSLfromString = function(xmlAsString, xslURL, paramName, paramValue, outputAsText)
{
    var xml = new DOMParser().parseFromString(xmlAsString, "text/xml");
    var xsl = me_apps.util.loadXMLDoc(xslURL);
// code for IE
    if (window.ActiveXObject)
    {
        return xml.transformNode(xsl);
    }
// code for Mozilla, Firefox, Opera, etc.
    else if (document.implementation && document.implementation.createDocument)
    {
        var xsltProcessor = new XSLTProcessor();
        if (typeof (paramName) !== 'undefined') {
            xsltProcessor.setParameter(null, paramName, paramValue);
        }
        xsltProcessor.importStylesheet(xsl);
        var doc = document.implementation.createDocument("", "", null);
        if (outputAsText) {
            return xsltProcessor.transformToFragment(xml, doc).textContent;
        } else {
            try {
                var umu = xsltProcessor.transformToFragment(xml, document);
            } catch (err) {
                if (me_apps.LOG) {
                    console.log(err.stack);
                }
                me_apps.log.warn("Fehler bei der XSLT ransformation: '" + xslURL + "'");
            }
            return this.serializer.serializeToString(umu);

        }
    }
};

me_apps.obj.util.prototype.quote = function(string) {
	return string.replace(/'/g, '\'\'');
};

/**
 * @function datetimeToHHMM
 * Converts a date string in ISO 8601 format (YYYY-MM-DD HH:MM:SS) into a 
 * hour/minute time string suitable for display (HH:MM)
 * @param {string} s - The ISO 8601 string
 * @returns {string} the display string
 */

me_apps.obj.util.prototype.datetimeToHHMM =
function(s) {
    return s.substring(11,16);
};

/**
 * @function datetimeToHHMM
 * Converts a date string in ISO 8601 format (YYYY-MM-DD HH:MM:SS) into a 
 * hour/minute time string suitable for display (DD.MM.YYYY)
 * @param {string} s - The ISO 8601 string
 * @returns {string} the display string
 */

me_apps.obj.util.prototype.datetimeToDDMMYYYY =
function(s) {
    return s.substring(8, 10)+"."+s.substring(5, 7)+"."+s.substring(0, 4);
};

/*
 * Convert a dmy and a hm string to a datetime string.
 * @param {String} ddmmyyyy
 * @param {String} hhmm
 * @returns {String}
 */
me_apps.obj.util.prototype.toDatetime =
function(ddmmyyyy, hhmm) {
    return ddmmyyyy.substr(6, 4)+"-"+
           ddmmyyyy.substr(3, 2)+"-"+
           ddmmyyyy.substr(0, 2)+" "+
           hhmm+":00";
};

me_apps.obj.util.prototype.validateDDMMYYYY =
function(input) {
    var reg = /^(((0[1-9]|[12]\d|3[01])\.(0[13578]|1[02])\.((19|[2-9]\d)\d{2}))|((0[1-9]|[12]\d|30)\.(0[13456789]|1[012])\.((19|[2-9]\d)\d{2}))|((0[1-9]|1\d|2[0-8])\.02\.((19|[2-9]\d)\d{2}))|(29\.02\.((1[6-9]|[2-9]\d)(0[48]|[2468][048]|[13579][26])|(([1][26]|[2468][048]|[3579][26])00))))$/;
    return input.match(reg);
};

/**
 * @function{gameHasDetailsLoaded}
 * Gets a game structure as parameter, and checks if this game has its details
 * loaded, or only the header information. Used to show the trash can icon
 * in the games list, and to check if we need to load details when choosing
 * the active game.
 * 
 * @param {type} game - the game structure as given in permanent storage
 * 
 * @returns {Boolean} -true if we have details, false if we don't
 */

me_apps.obj.util.prototype.gameHasDetailsLoaded =
function(game) {
    if (typeof(game.referees)  !== "undefined"
      ||typeof(game.team_guest) !== "undefined"
      ||typeof(game.team_home)  !== "undefined"
    ) {
        return true;
    }
    return false;
};

/**
 * Enable or disable a single navigation button.
 * @param {string} page The name of the page that should be dis/enabled
 * @param {boolean} enabled enable or disable? null, or undefined, means no change.
 */
me_apps.obj.util.prototype.setNavButton=
function(page, enabled) {
    var list=$(".navbutton-"+page);
    if (enabled === true) {
        list.removeClass("disabled");
        list.attr("href", page+".html");
    } else if (enabled === false) {
        list.addClass("disabled");
        list.attr("href", "");
    }
};

/**
 * Set all navigation buttons to given parameters.
 * @param {type} spielwahl      Should the spielwahl page be enabled?
 * @param {type} index          Should the index page be enabled?
 * @param {type} spielbericht   Should the bericht page be enabled?
 * @param {type} einstellungen  Should the einstellungen page be enabled?
 */
me_apps.obj.util.prototype.setPageNavButtons =
function(spielwahl, index, spielbericht, einstellungen) {
    this.setNavButton("spielwahl", spielwahl);
    this.setNavButton("index", index);
    this.setNavButton("spielbericht", spielbericht);
    this.setNavButton("einstellungen", einstellungen);
};

/**
 * @function appstateToNavigationGUI
 * Try to infer the current app state (game loaded, players decrypted, 
 * game time running) from various game objects and update the navigation 
 * buttons accordingly.
 * This should be called whenever the application state changes:
 * - spiel.copyValuesFromPermStore calls this when a game is loaded 
 * - spielbericht.handlePinInput calls this when encryption status changes
 * - toggleZeitLäuft calls this 
 * 
 * NB: calling this from mannschaft.copyValuesFromPermStore will not work.
 * This is because the app objects have the encryption status in the spiel
 * object, not the mannschaft object.
 */
me_apps.obj.util.prototype.appstateToNavigationGUI =
function() {
    // Default state: only game selection is allowed.
    var allowSpielwahl=true;
    var allowIndex=false;
    var allowSpielbericht=false;
    var allowEinstellungen=false;
    
    if (me_apps.db.hasCurrentGame() && this.gameHasDetailsLoaded(me_apps.db.getCurrentGame())) {
        // After a game is loaded, Spielbericht and Einstellungen are allowed
        allowSpielbericht=true;
        allowEinstellungen=true;
        // Index page isn't allowed until at least one of the teams are decrypted
        if (me_apps.spiel.encryptedHeim === false 
        ||  me_apps.spiel.encryptedGast === false)
            allowIndex=true;
    }
    
    if (me_apps.spiel && me_apps.spiel.zeitLäuft) {
        // If the game is running, we aren't allowed to switch pages
        allowSpielwahl=allowSpielbericht=allowEinstellungen=false;
    }
    this.setPageNavButtons(allowSpielwahl, allowIndex, allowSpielbericht, allowEinstellungen);
};

/**
 * @function getNextId
 * Gets a unique ID for a player who is generated manually, or an officer
 * who doesn't have an ID in the database.
 * IDs have numeric values, but are always strings, because they can be
 * encrypted when loading data from the server.
 * @returns {String} nextId
 */
me_apps.obj.util.prototype.getNextId = function() {
    var id=me_apps.nextId--;
    // IDs sind *immer* Strings, weil sie ursprünglich verschlüsselt 
    // sind/sein können.
    return ""+id;
};

/**
 * @function markIdSeen
 * Tells the ID generator that a certain ID was seen, for example when
 * loading previously generated IDs from the database, to make sure
 * subsequently generated IDs do not match this one.
 * @param {string} seen The ID that has been seen in an object.
 */
me_apps.obj.util.prototype.markIdSeen = function(seen) {
    if (parseInt(seen)<=me_apps.nextId)
        me_apps.nextId=seen-1;
};

/**
 * Get the half that a certain playing time is in (first half = 1, second
 * half = 2, optionals = 3,4...
 * @param {number} secs - the number of seconds in the game
 * @returns {number} half
 */
me_apps.obj.util.prototype.halfFromTime = function(secs) {
    var game = me_apps.db.getCurrentGame();
    if (secs <= game.playing_time_secs)         return 1;
    if (secs <= game.playing_time_secs*2)       return 2;
    secs -= game.playing_time_secs * 2;
    return 3 + parseInt((secs-1) / game.ext_playing_time_secs);
};

me_apps.obj.util.prototype.checkLocalStorageCapacity = function(bytes) {
    var result;
    try {
        var testString = new Array(bytes + 1).join('a');
        localStorage.setItem('test', testString);
        result = true;
    } catch (e) {
        result = false;
    }
    localStorage.removeItem('test');
    return result;
};
/**
 * Check how much more space can be allocated in localStorage
 */

me_apps.obj.util.prototype.localStorageCapacity = function() {
    var i = 0;
    var min, max = 10000000, mid;
    for (i = 20000; i <= max; i *=2) {
        if (me_apps.util.checkLocalStorageCapacity(i)) {
            min = i;
        } else {
            max = i;
            break;
        }
    }

    while (min+1000 < max) {
        mid = ~~((max+min)/2);
        if (me_apps.util.checkLocalStorageCapacity(mid)) {
            min = mid;
        } else {
            max = mid;
        }
    }
    return (max+min)/2;
};

me_apps.util = new me_apps.obj.util();
